#include "common.h"
#include "main.h"
#include "lcd_dep.h"
#include "glib_dep.h"
#include "memo_app.h"
#include "post_dev.h"

LCD_WINDOW Window_Control;
u32 alpha, startx, starty, movex, movey, move_y, move_x, inited = 0;

void MemoPad_Init(u32 start_x, u32 start_y, u32 size_x, u32 size_y, u32 alpha_value, LCD_WINDOW eWin)
{
	Window_Control = eWin;
	alpha = alpha_value;
	startx = start_x + 10;
	starty = start_y + 10;
	movex = size_x - 20;
	movey = size_y - 20;
	move_y = starty;
	move_x = startx;
	inited = 1;
		
	Switch_GLIBControl(eWin);
	GLIB_ClearFrame(NO_MASKING_COLOR, 0);
	GLIB_FillRectangle(start_x, start_y, size_x, size_y, BLACK);
	GLIB_FillRectangle(start_x, start_y, size_x, 5, WHITE);
	GLIB_FillRectangle(start_x, start_y, 5, size_y, WHITE);
	GLIB_FillRectangle(start_x, (start_y + size_y - 5), size_x, 5, WHITE);
	GLIB_FillRectangle((start_x + size_x - 5), start_y, 5, size_y, WHITE);
}

void MemoPad_On(void)
{
	if(inited == 0) return;
	LCD_Blend_ColorKeyOn(alpha, alpha, alpha, 0, 0, 0, NO_MASKING_COLOR, Window_Control);
	MemoPad_Clear();
}

void MemoPad_Update(const char *fmt,...)
{
	u32 temp;
	va_list ap;
	char string[256];

	va_start(ap,fmt);
	vsprintf(string,fmt,ap);
	if(inited)
	{
		Switch_GLIBControl(Window_Control);	

		temp = GLIB_PutString(move_x, move_y, WHITE, NO_BackGroundColor, movex + startx, string);
		
		if(temp != move_y)
		{
			move_y = temp;
			move_x = startx;
		}
		else
		{
			move_x += strlen(string) * 8;
		}
	}

	UART_PutStr(string);
	va_end(ap);
}

void MemoPad_Clear(void)
{
	if(inited == 0) return;
	Switch_GLIBControl(Window_Control);
	GLIB_FillRectangle(startx, starty, movex, movey, BLACK);
	move_y = starty;
}

void MemoPad_Off(void)
{
	if(inited == 0) return;
	LCD_Blend_ColorKeyOn(0x0, 0x0, 0x0, 0, 0, 0, NO_MASKING_COLOR, Window_Control);
	Switch_GLIBControl(Window_Control);
}

static POST	oPost;

void Select_View_Icon(u32 low_x, u32 low_y, u32 frm1, u32 high_x, u32 high_y, u32 frm2, u32 sel, u32 over, char *name)
{
	POST_InitCh(POST_A, &oPost);
	if(sel)
	{
		if(over)
		{
			GLIB_FillRectangle(120, 45, 60, 45, BLACK);
			GLIB_PutString(135, 60, WHITE, NO_BackGroundColor, 0, name);
		}
		else
		{
			GLIB_FillRectangle(121, 46, 58, 43, WHITE);
			GLIB_PutString(135, 60, BLACK, NO_BackGroundColor, 0, name);
		}

		// 2 Ȯ...
		POST_InitIp1(low_x,  low_x,  120, 44,  60, 48, frm1, RGB24,
					 high_x, high_y, 240, 88, 120, 96, frm2, RGB24,
					 1, false, ONE_SHOT, POST_DMA, POST_DMA, &oPost);
	}
	else
	{
		if(over)
		{
			GLIB_FillRectangle(20, 45, 60, 45, BLACK);
			GLIB_PutString(35, 60, WHITE, NO_BackGroundColor, 0, name);
		}
		else
		{
			GLIB_FillRectangle(21, 46, 58, 43, WHITE);
			GLIB_PutString(35, 60, BLACK, NO_BackGroundColor, 0, name);
		}

		// 2 Ȯ...
		POST_InitIp1(low_x,  low_x,  20, 44,  60, 48, frm1, RGB24,
					 high_x, high_y, 40, 88,  120, 96, frm2, RGB24,
					 1, false, ONE_SHOT, POST_DMA, POST_DMA, &oPost);

	}

	POST_StartProcessing(&oPost);
	while (!POST_IsFreeRunDone(&oPost));

}

u32 Select_View_Menu(char* message, char* Sel1, char* Sel2)
{
	u32 x, y, select, pre_select;
	u32 low_x, low_y;
	u32 high_x, high_y;
	u32 frm1, frm2;
	
	LCD_GetFrmSz(&high_x, &high_y, WIN0);
	
	high_x /= 2, high_y /= 2;
	low_x = high_x / 2;
	low_y = high_y / 2;
	LCD_GetFbEndAddr(&frm1, WIN4);
	frm2 = LCD_GetFbStartAddr(WIN4);
	
	// x : 200, y : 120
	GLIB_Init(frm1, low_x, low_y, RGB24);
	GLIB_ClearFrame(WHITE, 0);
	GLIB_PutRectangle(0, 0, low_x, low_y, BLACK);
	GLIB_PutLine(0, 21, low_x, 21, BLACK);
	GLIB_Printf(5, 3, BLACK, NO_BackGroundColor, UART, 0, message);
	printf(" : ");
	GLIB_PutRectangle(20, 45, 60, 45, BLACK);
	GLIB_Printf(35, 60, BLACK, NO_BackGroundColor, UART, 0, Sel1);
	printf("(y) / ");
	GLIB_PutRectangle(120, 45, 60, 45, BLACK);
	GLIB_Printf(135, 60, BLACK, NO_BackGroundColor, UART, 0, Sel2);
	printf("(n) ? ");

	// 2 Ȯ...
	POST_InitCh(POST_A, &oPost);
	POST_InitIp((low_x), (low_y), frm1, RGB24, high_x, high_y, frm2, RGB24, 1, false, ONE_SHOT, &oPost);
	POST_StartProcessing(&oPost);
	while (!POST_IsFreeRunDone(&oPost));
	
	LCD_Blend_ColorKeyOn(0xf, 0xf, 0xf, 0, 0, 0, NO_MASKING_COLOR, WIN4);
	
	select = 0;
	while(1)
	{
		switch(UART_GetStatus())
		{
			case 'y':
			case 'Y':
				select = 1;	printf("\b ");
				break;
			case 'n':
			case 'N':
				select = 2;	printf("\b ");
				break;
			case 0:			break;
			default:
				printf("\b ");
				break;
		}

		if(Get_Touch_XY(&x, &y))
		{
			pre_select = select;
			select = 0;
			if(x > (low_x + 40) && y > (low_y + 90) && x < (low_x + 160) && y < (low_y + 180))			select = 1;
			else if(x > (low_x + 240) && y > (low_y + 90) && x < (low_x + 360) && y < (low_y + 180))	select = 2;
			
			if(select != pre_select)
			{
				if(select)		Select_View_Icon(low_x, low_y, frm1, high_x, high_y, frm2, select - 1, 1, (select - 1)?Sel2:Sel1);
				if(pre_select)	Select_View_Icon(low_x, low_y, frm1, high_x, high_y, frm2, pre_select - 1, 0, (pre_select - 1)?Sel2:Sel1);
			}
		}
		else
		{
			if(select)
			{
				printf("%s\n", (select - 1)?Sel2:Sel1);
				break;
			}
			select = 0;
		}
	}
	
	LCD_Blend_ColorKeyOn(0, 0, 0, 0, 0, 0, NO_MASKING_COLOR, WIN4);

	return select - 1;
}























