#include "common.h"
#include "main.h"
#include "pjboot_app.h"
#include "lcd_dep.h"
#include "glib_dep.h"
#include "picture_indep.h"
#include "common_app.h"

u32 Image_Analysis(u32 address, u32 size);

static void *pjboot_main_function[][6] =
{
//	Function					Icon Name					Pos. X		Pos. Y		BMP Icon			Move up
	(void*)1,					(char*)"USB Down",			(u32*)150,	(u32*)200,	(u8*)BMP_USB_down,	(void*)ICON_NOMOVE,
	(void*)1,					(char*)"SERIAL Down",		(u32*)350,	(u32*)200,	(u8*)BMP_UART_down,	(void*)ICON_NOMOVE,
	(void*)NandMap_Viewer,		(char*)"Nand Flash Map",	(u32*)550,	(u32*)200,	(u8*)PJBOOT_Cfg,	(void*)ICON_NOMOVE,
	0, 0, 0, 0, 0, 0
};


void Draw_PJBoot_Win0(void)
{
	Switch_GLIBControl(WIN0);
	GLIB_ClearFrame(WHITE, 1);
	Uart_Logo_View("PJBoot64 Config Tree", 0);
	Draw_Title_Box("PJBoot64 Config Mode");
	Position_Menu_Draw(pjboot_main_function, NULL, 1);
}

void Draw_Maker_Win0(void)
{
	char temp1[100];
	u32 sizex, sizey;
	
	Switch_GLIBControl(WIN0);
	GLIB_ClearFrame(WHITE, 1);
	GLIB_PutRectangle(310, 5, 180, 31, BLACK);
	GLIB_PutString(344, 15, BLACK, NO_BackGroundColor, 0, "PJBoot64 Maker");
	
	printf("\n");
	printf("+-----------------------------------------------------+\n");
	sprintf(temp1, "PJBoot64 %-6s (%s)", VERSION, REL_DATE);
	sizex = (PJBoot64.lcd_xsize - (strlen(temp1)*8)) / 2;
	sizey = (PJBoot64.lcd_ysize - 120) / 2;
	GLIB_PutString(sizex, sizey, BLACK, NO_BackGroundColor, 0, temp1);
	printf("|  %s                       |\n", temp1);
	printf("+-----------------------------------------------------+\n");
	sprintf(temp1, "Embedded Crazy Boys Cafe");
	sizex = (PJBoot64.lcd_xsize - (strlen(temp1)*8)) / 2;
	GLIB_PutString(sizex, sizey + 60, BLUE, NO_BackGroundColor, 0, temp1);
	printf("|  %s                           |\n", temp1);
	sprintf(temp1, "(http://cafe.naver.com/embeddedcrazyboys)");
	sizex = (PJBoot64.lcd_xsize - (strlen(temp1)*8)) / 2;
	GLIB_PutString(sizex, sizey + 90, BLACK, NO_BackGroundColor, 0, temp1);
	printf("|  %s          |\n", temp1);
	printf("+-----------------------------------------------------+\n");
	sprintf(temp1, "Create By (Pyeong-Jeong Lee)");
	sizex = (PJBoot64.lcd_xsize - (strlen(temp1)*8)) / 2;
	GLIB_PutString(sizex, sizey + 30, BLACK, NO_BackGroundColor, 0, temp1);
	printf("|  %s                 |\n", temp1);
	printf("+-----------------------------------------------------+\n");
}

void PJBoot_Menu(void)
{
	u32 select, filesize, temp;
	
	while(1)
	{
		select = Simple_Select_Check(pjboot_main_function);

		switch(select)
		{
			case 1:
				MemoPad_On();
				filesize = Usb_Loader(PJBoot64.ram.Temp, 1);
				if(filesize)
				{
					temp = Image_Analyser(9, filesize, PJBoot64.ram.Temp);
					if(temp == 1) 
					{
						MemoPad_Update("PJBoot64 Binary Found!\n");
						
						if(!Select_View_Menu("Nand Write","Yes", "No"))
							Nand_Write_Process(0, PJBoot64.nand.Bootloader, filesize, PJBoot64.ram.Temp, 1);
						else
							MemoPad_Update("Nand Flash Write Cancle!\n");	
					}
					else if(temp == 2)
					{
						MemoPad_Update("Icon Image Found!\n");
						
						if(!Select_View_Menu("Nand Write","Yes", "No"))
							Nand_Write_Process(0, PJBoot64.nand.Boot_Icon, filesize, PJBoot64.ram.Temp, 1);
						else
							MemoPad_Update("Nand Flash Write Cancle!\n");
					}
				}
				else	MemoPad_Update("Not Load Memory!!\n");
				Wait_Key();
				Switch_GLIBControl(WIN1);
				Position_Menu_Draw(pjboot_main_function, NULL, 1);
				MemoPad_Off();
				break;
			case 2:
				MemoPad_On();
				filesize = Uart_Loader(PJBoot64.ram.Temp);
				if(filesize)
				{
					temp = Image_Analyser(9, filesize, PJBoot64.ram.Temp);
					if(temp == 1) 
					{
						MemoPad_Update("PJBoot64 Binary Found!\n");
						
						if(!Select_View_Menu("Nand Write","Yes", "No"))
							Nand_Write_Process(0, PJBoot64.nand.Bootloader, filesize, PJBoot64.ram.Temp, 1);
						else
							MemoPad_Update("Nand Flash Write Cancle!\n");	
					}
					else if(temp == 2)
					{
						MemoPad_Update("Icon Image Found!\n");
						
						if(!Select_View_Menu("Nand Write","Yes", "No"))
							Nand_Write_Process(0, PJBoot64.nand.Boot_Icon, filesize, PJBoot64.ram.Temp, 1);
						else
							MemoPad_Update("Nand Flash Write Cancle!\n");
					}
				}
				else	MemoPad_Update("Not Load Memory!!\n");
				Wait_Key();
				Switch_GLIBControl(WIN1);
				Position_Menu_Draw(pjboot_main_function, NULL, 1);
				MemoPad_Off();
				break;
			case 3:
				if((u32)pjboot_main_function[select - 1][0] != 1) ((void(*)(void))(pjboot_main_function[select - 1][0]))();
				Draw_PJBoot_Win0();
				LCD_Move_Arrow_Win(POSITION_RANDOM, WIN1);
				break;
			case 10:
				return;
			default:
				Uart_Logo_View("PJBoot64 Config Tree", 0);
				Position_Menu_Draw(pjboot_main_function, NULL, 1);
				break;
		}
	}
}

void NandMap_Viewer(void)
{
	Switch_GLIBControl(WIN1);
	GLIB_ClearFrame(WHITE, 1);
	Uart_Logo_View("Nand Flash Map", PJBoot64.nand_id);
	Draw_Title_Box("PJBoot64 Nand Flash Map");
	
	GLIB_PutString(83, 120, BLACK, NO_BackGroundColor, 0, "PJBoot64");
	GLIB_PutRectangle(50, 140, 130, 240, BLACK);
	GLIB_PutLine(50, 200, 180, 200, BLACK);
	GLIB_PutLine(50, 260, 180, 260, BLACK);
	GLIB_PutLine(50, 320, 180, 320, BLACK);
	GLIB_PutString(285, 120, BLACK, NO_BackGroundColor, 0, "Linux");
	GLIB_PutRectangle(240, 140, 130, 240, BLACK);
	GLIB_PutLine(240, 220, 370, 220, BLACK);
	GLIB_PutLine(240, 300, 370, 300, BLACK);
	GLIB_PutString(463, 120, BLACK, NO_BackGroundColor, 0, "Firmware");
	GLIB_PutRectangle(430, 140, 130, 240, BLACK);
	GLIB_PutLine(430, 260, 560, 260, BLACK);
	GLIB_PutString(665, 120, BLACK, NO_BackGroundColor, 0, "WinCE");
	GLIB_PutRectangle(620, 140, 130, 240, BLACK);
	GLIB_PutLine(620, 260, 750, 260, BLACK);
	
	
	printf("| ");
	GLIB_Printf(600, 40, BLUE, NO_BackGroundColor, NO_UART, 0, "Flash ID : %x", PJBoot64.nand_id);
	GLIB_Printf(600, 60, BLUE, NO_BackGroundColor, UART, 0, "Total Block : %d Block", PJBoot64.nand_total_block);
	printf("                            |\n| ");
	GLIB_Printf(600, 80, BLUE, NO_BackGroundColor, UART, 0, "1 Block : %d Bytes", PJBoot64.nand_1_block);
	printf("                              |\n+-----------------------------------------------------+\n| ");
	
	GLIB_Printf(75, 334, BLACK, NO_BackGroundColor, UART, 0, "StepLoader ");
	GLIB_Printf(71, 350, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Booter, PJBoot64.nand.Bootloader - 1);
	printf("                              |\n| ");
	GLIB_Printf(83, 274, BLACK, NO_BackGroundColor, UART, 0, "PJBoot64 ");
	GLIB_Printf(71, 290, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Bootloader, PJBoot64.nand.Boot_CFG - 1);
	printf("                                |\n| ");
	GLIB_Printf(71, 214, BLACK, NO_BackGroundColor, UART, 0, "Boot Config ");
	GLIB_Printf(71, 230, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Boot_CFG, PJBoot64.nand.Boot_Icon - 1);
	printf("                             |\n| ");
	GLIB_Printf(79, 154, BLACK, NO_BackGroundColor, UART, 0, "Boot ICON ");
	GLIB_Printf(71, 170, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Boot_Icon, PJBoot64.nand.Linux_Kernel - 1);
	printf("                               |\n| ");
	GLIB_Printf(257, 324, BLACK, NO_BackGroundColor, UART, 0, "Linux Kernel ");
	GLIB_Printf(261, 340, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Linux_Kernel, PJBoot64.nand.Android_root - 1);
	printf("                            |\n| ");
	GLIB_Printf(257, 244, BLACK, NO_BackGroundColor, UART, 0, "Android Root ");
	GLIB_Printf(261, 260, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Android_root, PJBoot64.nand.Android_data - 1);
	printf("                            |\n| ");
	GLIB_Printf(257, 164, BLACK, NO_BackGroundColor, UART, 0, "Android Data ");
	GLIB_Printf(261, 180, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Android_data, PJBoot64.nand.Etc - 1);
	printf("                            |\n| ");
	GLIB_Printf(463, 304, BLACK, NO_BackGroundColor, UART, 0, "Firmware ");
	GLIB_Printf(451, 320, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Firmware, PJBoot64.nand.Firmware_etc - 1);
	printf("                                |\n| ");
	GLIB_Printf(447, 184, BLACK, NO_BackGroundColor, UART, 0, "Firmware ETC ");
	GLIB_Printf(451, 200, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Firmware_etc, PJBoot64.nand.Etc - 1);
	printf("                            |\n| ");
	GLIB_Printf(641, 304, BLACK, NO_BackGroundColor, UART, 0, "Wince Eboot ");
	GLIB_Printf(641, 320, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Wince_Eboot, PJBoot64.nand.Wince_Kernel - 1);
	printf("                             |\n| ");
	GLIB_Printf(637, 184, BLACK, NO_BackGroundColor, UART, 0, "Wince Kernel ");
	GLIB_Printf(641, 200, RED, NO_BackGroundColor, UART, 0, "%04d - %04d", PJBoot64.nand.Wince_Kernel, PJBoot64.nand.Etc - 1);
	printf("                            |\n+-----------------------------------------------------+\n");
	

	Wait_Key();
}

void Maker_Menu(void)
{
	Wait_Key();
}
/*
// ٿ  ̹ ˻.. (Bootloader and Icon)
static u32 Image_Analysis(u32 address, u32 size)
{
	u8*  addr = (u8*)address;
	u16* addr16;
//	u16* addr16 = (u16*)(address + (KERNEL_RECOVERY - LCD_ICON_BUFFER));
	u32 i;
	
	// Icon ̹ ˻..
	addr16 = (u16*)(address + (KERNEL_BOOT - LCD_ICON_BUFFER));
	
	printf("Debug : %x, %x, %x\n", address, addr16, *addr16);
	if(size > (KERNEL_RECOVERY - LCD_ICON_BUFFER) && *(addr16) == 0x4d42)
	{
		addr16 = (u16*)(address + (KERNEL_RECOVERY - LCD_ICON_BUFFER));
		if(*(addr16) == 0x4d42) return 2;
	}
	
	// Bootloader ̹ ˻...
	for(i=0;i<size / 4;i++)
		if(StrCmp(addr + i, "D:\work\PJBoot64")) return 1;
	
	return 0;
}
*/
