/**************************************************************************************
* 
*	Project Name : S3C6400 Validation
*
*	Copyright 2006 by Samsung Electronics, Inc.
*	All rights reserved.
*
*	Project Description :
*		This software is only for validating functions of the S3C6400.
*		Anybody can use this software without our permission.
*  
*--------------------------------------------------------------------------------------
* 
*	File Name : library.c
*  
*	File Description : This file implements library functions.
*
*	Author : Haksoo,Kim
*	Dept. : AP Development Team
*	Created Date : 2006/11/08
*	Version : 0.1 
* 
*	History
*	- Created(Haksoo,Kim 2006/11/08)
*  
**************************************************************************************/

//#include <stdio.h>
//#include <stdlib.h>
#include "common.h"
//#include "linux/ctype.h"

#include "option.h"
#include "library.h"
//#include "system.h"
//#include "def.h"
#include "main.h"
//#include "sfr6410.h"
#include "s3c6410.h"
#include "gpio_dev.h"

#define SYSCON_BASE			ELFIN_CLOCK_POWER_BASE

#define rAPLL_LOCK			(SYSCON_BASE+0x000)
#define rMPLL_LOCK			(SYSCON_BASE+0x004)
#define rEPLL_LOCK			(SYSCON_BASE+0x008)
#define rAPLL_CON			(SYSCON_BASE+0x00c)
#define rMPLL_CON			(SYSCON_BASE+0x010)
#define rEPLL_CON0			(SYSCON_BASE+0x014)
#define rEPLL_CON1			(SYSCON_BASE+0x018)
#define rCLK_SRC			(SYSCON_BASE+0x01c)
#define rCLK_SRC2			(SYSCON_BASE+0x10c)
#define rCLK_DIV0			(SYSCON_BASE+0x020)
#define rCLK_DIV1			(SYSCON_BASE+0x024)
#define rCLK_DIV2			(SYSCON_BASE+0x028)
#define rCLK_OUT			(SYSCON_BASE+0x02c)
#define rHCLK_GATE			(SYSCON_BASE+0x030)
#define rPCLK_GATE			(SYSCON_BASE+0x034)
#define rSCLK_GATE			(SYSCON_BASE+0x038)
#define rMEM0_CLK_GATE		(SYSCON_BASE+0x03c)

#define rAHB_CON0			(SYSCON_BASE+0x100)
#define rAHB_CON1			(SYSCON_BASE+0x104)
#define rAHB_CON2			(SYSCON_BASE+0x108)
#define rSDMA_SEL			(SYSCON_BASE+0x110)
#define rSW_RST				(SYSCON_BASE+0x114)
#define rSYS_ID				(SYSCON_BASE+0x118)
#define rMEM_SYS_CFG		(SYSCON_BASE+0x120)
#define rQOS_OVERRIDE0		(SYSCON_BASE+0x124)
#define rQOS_OVERRIDE1		(SYSCON_BASE+0x128)
#define rMEM_CFG_STAT		(SYSCON_BASE+0x12c)

#define rPWR_CFG			(SYSCON_BASE+0x804)
#define rEINT_MASK			(SYSCON_BASE+0x808)
#define rNORMAL_CFG			(SYSCON_BASE+0x810)
#define rSTOP_CFG			(SYSCON_BASE+0x814)
#define rSLEEP_CFG			(SYSCON_BASE+0x818)
#define rSTOP_MEM_CFG		(SYSCON_BASE+0x81C)
#define rOSC_FREQ			(SYSCON_BASE+0x820)
#define rOSC_STABLE			(SYSCON_BASE+0x824)
#define rPWR_STABLE			(SYSCON_BASE+0x828)
#define rFPC_STABLE			(SYSCON_BASE+0x82c)
#define rMTC_STABLE			(SYSCON_BASE+0x830)
#define rBUS_CACHEABLE_CON	(SYSCON_BASE+0x838)

#define rOTHERS				(SYSCON_BASE+0x900)
#define rRST_STAT			(SYSCON_BASE+0x904)
#define rWAKEUP_STAT		(SYSCON_BASE+0x908)
#define rBLK_PWR_STAT		(SYSCON_BASE+0x90c)
#define rINFORM0			(SYSCON_BASE+0xA00)
#define rINFORM1			(SYSCON_BASE+0xA04)
#define rINFORM2			(SYSCON_BASE+0xA08)
#define rINFORM3			(SYSCON_BASE+0xA0c)
#define rINFORM4			(SYSCON_BASE+0xA10)
#define rINFORM5			(SYSCON_BASE+0xA14)
#define rINFORM6			(SYSCON_BASE+0xA18)
#define rINFORM7			(SYSCON_BASE+0xA1c)

u32 delayLoopCount;
void Delay(u32 uTime)
{
	u32 i=0,j=delayLoopCount;
//	printf("delay count :: %d.\n", delayLoopCount);

	for(;uTime>0;uTime--)
	{
		for(;i<j;i++);
	}
}

void GetSysClk(void)
{
	u8 divApll, divHclkx2, divHclk, divPclk;
	u16 pllM, pllP, pllS;
	u32 temp, g_HCLKx2;
	
	////
	// clock division ratio	
	temp = Inp32(rCLK_DIV0);
	divApll = temp & 0xf;
	divHclkx2 = (temp>>9) & 0x7;
	divHclk = (temp>>8) & 0x1;
	divPclk = (temp>>12) & 0xf;
/*
	////
	// Operating Mode
	temp = Inp32(rOTHERS);
	temp = (temp>>8)&0xf;
	if(temp)
	{
		g_SYNCACK = 1;
	}
	else
	{
		g_SYNCACK = 0;
	}
*/
	////
	// ARMCLK

	if(Inp32(rCLK_SRC) & 0x1)	//FOUT
	{			
		temp = Inp32(rAPLL_CON);
		pllM = (temp>>16)&0x3ff;
		pllP = (temp>>8)&0x3f;
		pllS = (temp&0x7);

		PJBoot64.apll = ((FIN>>pllS)/pllP)*pllM;
	}
	else	//FIN
	{
		PJBoot64.apll = FIN;
	}
	
	PJBoot64.armclk = PJBoot64.apll/(divApll+1);
	
	////
	// HCLK
	if((Inp32(rOTHERS)>>7) & 0x1)	//synchronous mode
	{
		g_HCLKx2 = PJBoot64.apll/(divHclkx2+1);
		temp = Inp32(rMPLL_CON);
		pllM = (temp>>16)&0x3ff;
		pllP = (temp>>8)&0x3f;
		pllS = (temp&0x7);

		PJBoot64.mpll = ((FIN>>pllS)/pllP)*pllM;
	}
	else
	{
		if((Inp32(rCLK_SRC)>>1) & 0x1)	//FOUT
		{						
			temp = Inp32(rMPLL_CON);
			pllM = (temp>>16)&0x3ff;
			pllP = (temp>>8)&0x3f;
			pllS = (temp&0x7);
			PJBoot64.mpll = ((FIN>>pllS)/pllP)*pllM;
		}
		else	//FIN
		{
			PJBoot64.mpll = FIN;
		}
		g_HCLKx2 = PJBoot64.mpll/(divHclkx2+1);		
	}
	
	PJBoot64.hclk = g_HCLKx2/(divHclk+1);
	
	////
	// PCLK
	PJBoot64.pclk = g_HCLKx2/(divPclk+1);

	return;
}

u32 GetBitPosition(u32 uValue)
{
	u8 bitshift;

	for(bitshift=0 ; bitshift<32 ; bitshift++)
	{
	  if(uValue & (1<<bitshift))
	  	return bitshift;
	}
	return 0;  
}
