#include "common.h"
#include "option.h"
#include "library.h"
#include "s3c6410.h"
#include "system.h"

#include "gpio_dev.h"
#include "sysc.h"
//#include "rtc.h"
//#include "adcts.h"
#include "intc.h"
//#include "sdhc.h"

#define dprintf	UART_Printf
//#define dprintf


//
#define rAPLL_LOCK			(ELFIN_CLOCK_POWER_BASE+0x000)
#define rMPLL_LOCK			(ELFIN_CLOCK_POWER_BASE+0x004)
#define rEPLL_LOCK			(ELFIN_CLOCK_POWER_BASE+0x008)
#define rAPLL_CON			(ELFIN_CLOCK_POWER_BASE+0x00c)
#define rMPLL_CON			(ELFIN_CLOCK_POWER_BASE+0x010)
#define rEPLL_CON0			(ELFIN_CLOCK_POWER_BASE+0x014)
#define rEPLL_CON1			(ELFIN_CLOCK_POWER_BASE+0x018)
#define rCLK_SRC				(ELFIN_CLOCK_POWER_BASE+0x01c)
#define rCLK_SRC2			(ELFIN_CLOCK_POWER_BASE+0x10c)
#define rCLK_DIV0			(ELFIN_CLOCK_POWER_BASE+0x020)
#define rCLK_DIV1			(ELFIN_CLOCK_POWER_BASE+0x024)
#define rCLK_DIV2			(ELFIN_CLOCK_POWER_BASE+0x028)
#define rCLK_OUT				(ELFIN_CLOCK_POWER_BASE+0x02c)
#define rHCLK_GATE			(ELFIN_CLOCK_POWER_BASE+0x030)
#define rPCLK_GATE			(ELFIN_CLOCK_POWER_BASE+0x034)
#define rSCLK_GATE			(ELFIN_CLOCK_POWER_BASE+0x038)
#define rMEM0_CLK_GATE		(ELFIN_CLOCK_POWER_BASE+0x03c)

//
#define rAHB_CON0			(ELFIN_CLOCK_POWER_BASE+0x100)
#define rAHB_CON1			(ELFIN_CLOCK_POWER_BASE+0x104)
#define rAHB_CON2			(ELFIN_CLOCK_POWER_BASE+0x108)
#define rSDMA_SEL			(ELFIN_CLOCK_POWER_BASE+0x110)
#define rSW_RST				(ELFIN_CLOCK_POWER_BASE+0x114)
#define rSYS_ID				(ELFIN_CLOCK_POWER_BASE+0x118)
#define rMEM_SYS_CFG		(ELFIN_CLOCK_POWER_BASE+0x120)
#define rQOS_OVERRIDE0		(ELFIN_CLOCK_POWER_BASE+0x124)
#define rQOS_OVERRIDE1		(ELFIN_CLOCK_POWER_BASE+0x128)
#define rMEM_CFG_STAT		(ELFIN_CLOCK_POWER_BASE+0x12c)
//
#define rPWR_CFG			(ELFIN_CLOCK_POWER_BASE+0x804)
#define rEINT_MASK			(ELFIN_CLOCK_POWER_BASE+0x808)
#define rNORMAL_CFG			(ELFIN_CLOCK_POWER_BASE+0x810)
#define rSTOP_CFG			(ELFIN_CLOCK_POWER_BASE+0x814)
#define rSLEEP_CFG			(ELFIN_CLOCK_POWER_BASE+0x818)
#define rSTOP_MEM_CFG		(ELFIN_CLOCK_POWER_BASE+0x81C)
#define rOSC_FREQ			(ELFIN_CLOCK_POWER_BASE+0x820)
#define rOSC_STABLE			(ELFIN_CLOCK_POWER_BASE+0x824)
#define rPWR_STABLE			(ELFIN_CLOCK_POWER_BASE+0x828)
#define rFPC_STABLE			(ELFIN_CLOCK_POWER_BASE+0x82c)
#define rMTC_STABLE			(ELFIN_CLOCK_POWER_BASE+0x830)
#define rBUS_CACHEABLE_CON	(ELFIN_CLOCK_POWER_BASE+0x838)

// 
#define rOTHERS				(ELFIN_CLOCK_POWER_BASE+0x900)
#define rRST_STAT			(ELFIN_CLOCK_POWER_BASE+0x904)
#define rWAKEUP_STAT		(ELFIN_CLOCK_POWER_BASE+0x908)
#define rBLK_PWR_STAT		(ELFIN_CLOCK_POWER_BASE+0x90c)
#define rINFORM0			(ELFIN_CLOCK_POWER_BASE+0xA00)
#define rINFORM1			(ELFIN_CLOCK_POWER_BASE+0xA04)
#define rINFORM2			(ELFIN_CLOCK_POWER_BASE+0xA08)
#define rINFORM3			(ELFIN_CLOCK_POWER_BASE+0xA0c)
#define rINFORM4			(ELFIN_CLOCK_POWER_BASE+0xA10)
#define rINFORM5			(ELFIN_CLOCK_POWER_BASE+0xA14)
#define rINFORM6			(ELFIN_CLOCK_POWER_BASE+0xA18)
#define rINFORM7			(ELFIN_CLOCK_POWER_BASE+0xA1c)

void SYSC_SetPLL(PLL_eTYPE  ePLL, u32 uMdiv, u32 uPdiv, u32 uSdiv, u32 uKdiv)
{
	u32 temp, uRegValue;


	switch(ePLL)
	{
		case eAPLL: 
			// Check the Divider Value.
			if( uMdiv < 64 || uMdiv > 1023)
			{
				UART_Printf(" Wrong  Mdiv Value, Correct Value Range = (56 ~ 1023) (%d)\n", uMdiv);
			}
			if( uPdiv < 1 || uPdiv > 63)
			{
				UART_Printf(" Wrong  Pdiv Value, Correct Value Range = (1 ~ 63) (%d)\n", uPdiv);
			}
			if( uSdiv > 5)
			{
				UART_Printf(" Wrong  Sdiv Value, Correct Value Range = (0 ~ 5) (%d)\n", uSdiv);
			}
      			// Check the Fvco Range

      			temp = ((FIN/uPdiv)*uMdiv)/1000000;
			if( temp <800 || temp > 1600)
			{

				UART_Printf(" Please select the proper M,P,S divider value\n");
				UART_Printf(" Fvco Range = (800MHz ~ 1600MHz),  Current Value is (%d)MHz\n", temp);
			}

			uRegValue = (u32)(((u32)(0x1<<31))|(uMdiv<<16)|(uPdiv<<8)|(uSdiv<<0));
			Outp32(rAPLL_CON, uRegValue);
			break;

		case eMPLL: 
			// Check the Divider Value.
			if( uMdiv < 64 || uMdiv > 1023)
			{
				UART_Printf(" Wrong  Mdiv Value, Correct Value Range = (56 ~ 1023) (%d)\n", uMdiv);
			}
			if( uPdiv < 1 || uPdiv > 63)
			{
				UART_Printf(" Wrong  Pdiv Value, Correct Value Range = (1 ~ 63) (%d)\n", uPdiv);
			}
			if( uSdiv > 5)
			{
				UART_Printf(" Wrong  Sdiv Value, Correct Value Range = (0 ~ 5) (%d)\n", uSdiv);
			}
      			// Check the Fvco Range

      			temp = ((FIN/uPdiv)*uMdiv)/1000000;
			if( temp <800 || temp > 1600)
			{
				UART_Printf(" Please select the proper M,P,S divider value\n");
				UART_Printf(" Fvco Range = (800MHz ~ 1600MHz),  Current Value is (%d)MHz\n", temp);
			}

			uRegValue = (u32)(((u32)(0x1<<31))|(uMdiv<<16)|(uPdiv<<8)|(uSdiv<<0));
			Outp32(rMPLL_CON, uRegValue);	
			break;

		case eEPLL: 
			// Check the Divider Value.
			if( uMdiv < 16 || uMdiv > 255)
			{
				UART_Printf(" Wrong  Mdiv Value, Correct Value Range = (56 ~ 1023) (%d)\n", uMdiv);
			}
			if( uPdiv < 1 || uPdiv > 63)
			{
				UART_Printf(" Wrong  Pdiv Value, Correct Value Range = (1 ~ 63) (%d)\n", uPdiv);
			}
			if( uSdiv > 5)
			{
				UART_Printf(" Wrong  Sdiv Value, Correct Value Range = (0 ~ 5) (%d)\n", uSdiv);
			}
			if( uKdiv >65535)
			{
				UART_Printf(" Wrong  Kdiv Value, Correct Value Range = (0 ~ 65535) (%d)\n", uKdiv);
			}
		      // Check the Fvco Range

		       temp = (FIN/uPdiv)*(uMdiv+(uKdiv/(2^16)))/1000000;
			if( temp <300 || temp > 600)
			{
				UART_Printf(" Please select the proper M,P,S divider value\n");
				UART_Printf(" Fvco Range = (300MHz ~ 600MHz),  Current Value is (%d)MHz\n", temp);
			}

			Outp32(rEPLL_CON1, uKdiv);
			uRegValue = (u32)(((u32)(0x1<<31))|(uMdiv<<16)|(uPdiv<<8)|(uSdiv<<0));
			Outp32(rEPLL_CON0, uRegValue);
			break;
		}

	// Get Information
	GetSysClk();
}

//////////
// Function Name : SYSC_StopPLL
// Function Description : This function stop PLL.
//				  Fout = (mdiv * Fin) / (pdiv x 2^sdiv),   Fout = ((mdiv+k/2^16) * Fin) / (pdiv x 2^sdiv)
// Input : 				ePLL  : eAPLL,eMPLL, eEPLL
//
//						
// Output : NONE
// Version : 
void SYSC_StopPLL(PLL_eTYPE  ePLL)
{
	u32  uRegValue;


	switch(ePLL)
	{
		case eAPLL: 

			uRegValue = Inp32(rAPLL_CON);
			uRegValue = (uRegValue & ~(0x1<<31)); 
			Outp32(rAPLL_CON, uRegValue);
			break;

		case eMPLL: 
			uRegValue = Inp32(rMPLL_CON);
			uRegValue = (uRegValue & ~(0x1<<31)); 
			Outp32(rMPLL_CON, uRegValue);	
			break;

		case eEPLL: 
			uRegValue = Inp32(rEPLL_CON0);
			uRegValue = (uRegValue & ~(0x1<<31)); 
			Outp32(rEPLL_CON0, uRegValue);
			break;
		}

	
}
/*
/////////
// Function Name : SYSC_RdLockDetect
// Function Description : This function reads Lock Detect bit of the A/MPLL (Hidden)
// Input : 		ePLL  : eAPLL, eMPLL
// Output :            TRUE/FALSE
// Version : 

u8  SYSC_RdLockDetect(PLL_eTYPE  ePLL)
{
	u32 uRegValue, uLockDetect;

	switch(ePLL)
	{
		case eAPLL: 

			uRegValue = Inp32(rAPLL_CON);
			uLockDetect = (uRegValue >>30)&(0x1); 
			break;

		case eMPLL: 
			uRegValue = Inp32(rMPLL_CON);
			uLockDetect = (uRegValue >>30)&(0x1); 
			break;

	}

	if(uLockDetect == 1)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}

	
}
*/

#define uTIME_OF_XTAL	(1.0e6/(float)FIN)	

//////////
// Function Name : SYSC_SetLockTime
// Function Description : This function control APLL Lock Time (PLL9025X), Max 100us
//				  Fout = (mdiv * Fin) / (pdiv x 2^sdiv)
// Input : 				uLockTime : Max 100us, us order
//						ePLL		  : PLL Type
// Output : NONE
// Version : 
void SYSC_SetLockTime( PLL_eTYPE  ePLL, u32 uLockTime)
{
	u32 uLockCount;
	
	uLockCount = (unsigned int)((float)uLockTime/((float)uTIME_OF_XTAL));
//	UART_Printf("uLockCount : 0x%x\n", uLockCount);

	switch(ePLL)
	{
		case eAPLL: 
			Outp32(rAPLL_LOCK, uLockCount);  break;
		case eMPLL: 
			Outp32(rMPLL_LOCK, uLockCount); break;
		case eEPLL: 
			Outp32(rEPLL_LOCK, uLockCount); break;
					
	}

}

//boaz.kim
//////////
// Function Name : SYSC_ClkSrc
// Function Description : This function select Clock Source
//				 
// Input : 		  CLKSRC_eId
// Output : NONE
// Version : 
void SYSC_ClkSrc( CLKSRC_eId  eClkSrc)
{
	u32 uReg, uCtrl, uOffset, uFuct;
	u32 uRegValue;

	uReg = (eClkSrc&0x10000)>>16;
	uCtrl = (eClkSrc&0xF000)>>12;
	uOffset = (eClkSrc&0x0FF0)>>4;
	uFuct = eClkSrc&0x000F;

	if(uReg==0)
	{
		uRegValue = Inp32(rCLK_SRC);
		uRegValue = (uRegValue & ~(uCtrl<<(uOffset))) | (uFuct<<(uOffset));
		Outp32(rCLK_SRC, uRegValue);		
	}
	else if(uReg==1)
	{
		uRegValue = Inp32(rCLK_SRC2);
		uRegValue = (uRegValue & ~(uCtrl<<(uOffset))) | (uFuct<<(uOffset));
		Outp32(rCLK_SRC2, uRegValue);		
	}
	else Assert(0);
}
/*
//////////
// Function Name : SYSC_SetDIV
// Function Description : This function set Clock divider0 register ratio
//				 
// Input : CLKDIV_eId, division value
//
// Output : NONE
// Version : 
void SYSC_SetDIV( CLKDIV_eId eClkDiv, u32 uDivVal )
{
	u32 uReg, uCtrl, uOffset;
	u32 uRegValue;

	uReg = (eClkDiv&0xF000)>>12;
	uCtrl = (eClkDiv&0xF00)>>8;
	uOffset = (eClkDiv&0xFF);


	if(uReg==0)
	{
		uRegValue = Inp32(rCLK_DIV0);
		uRegValue = (uRegValue & ~(uCtrl<<(uOffset))) | (uDivVal<<(uOffset));
		Outp32(rCLK_DIV0, uRegValue);		
	}
	else if(uReg==1)
	{
		uRegValue = Inp32(rCLK_DIV1);
		uRegValue = (uRegValue & ~(uCtrl<<(uOffset))) | (uDivVal<<(uOffset));
		Outp32(rCLK_DIV1, uRegValue);				
	}
	else if(uReg==2)
	{
		uRegValue = Inp32(rCLK_DIV2);
		uRegValue = (uRegValue & ~(uCtrl<<(uOffset))) | (uDivVal<<(uOffset));
		Outp32(rCLK_DIV2, uRegValue);				
	}
	else Assert(0);
	

}


//////////
// Function Name : SYSC_SetDIV0
// Function Description : This function set Clock divider0 register ratio
//				 
// Input : 		  	uAratio 	: DIV_APLL ( 0 ~ 15) 
//					uMratio 	: DIV_MPLL ( 0~1)
//					uHratio 	: DIV_HCLK (0~1),  HCLKx2
//					uHx2ratio: DIV_HCLKx2(0~7), (APLL or MPLL)
//					uPratio	: DIV_PCLK (0~15),  HCLKx2
//					uONDratio: DIV_ONDCLK(0~3), HCLKx2
//					uSECUratio:DIV_Security(0~3), HCLKx2
//					uCAMratio: DIV_CAM(0~15), HCLKx2
//					uJPEGratio: DIV_JPEG(0~15), HCLKx2
//					uMFCratio: DIV_MFC(0~15), HCLKx2
// Output : NONE
// Version : 
void SYSC_SetDIV0( u32 uAratio, u32 uMratio, u32 uHratio, u32 uHx2ratio, u32 uPratio, 
	u32 uONDratio, u32 uSECUratio, u32 uCAMratio, u32 uJPEGratio, u32 uMFCratio )
{
	u32 uRegValue;

	uRegValue =
		( (uMFCratio<<28)|		// HCLKx2/(uMFCratio + 1)
		(uJPEGratio<<24) |		// HCLKx2/(uJPEGratio + 1)
		(uCAMratio<<20) |		// HCLKx2/(uCAMratio + 1)
		(uSECUratio<<18)|		// HCLKx2/(uSECUratio + 1)
		(uONDratio<<16) |		// HCLKx2/(uONDratio +1 )
		(uPratio<<12) |			// HCLKx2/(uPratio+1)
		(uHx2ratio<<9) |
		(uHratio<<8)|			// HCLKx2(uHratio+1)
		(uMratio<<4)|
		(uAratio<<0)) ;


	Outp32(rCLK_DIV0, uRegValue);

}

//////////
// Function Name : SYSC_SetDIV0_all
// Function Description : This function set Clock divider0 register ratio
// Input : 	None
// Version : 
// add by rb1004
void SYSC_SetDIV0_all(u32 uSetRatio)
{
	Outp32(rCLK_DIV0, uSetRatio);
}


//////////
// Function Name : SYSC_GetDIV0
// Function Description : This function get Clock divider0 register ratio
// Input : 	None
// Version : 
// add by rb1004
u32 SYSC_GetDIV0( void)
{
	return  Inp32(rCLK_DIV0);
}





//////////
// Function Name : SYSC_SetDIV1
// Function Description : This function set Clock divider1 register ratio
//				 
// Input : 		  	uMMC0ratio 	: (0~15)
//					uMMC1ratio	: (0~15)
//					uMMC2ratio	: (0~15)
//					uLCDratio	: (0~15)
//					uSCALERratio : (0~15)
//					uHOSTratio	: (0~15)
// Output : NONE
// Version : 
void SYSC_SetDIV1( u32 uMMC0ratio, u32 uMMC1ratio, u32 uMMC2ratio, u32 uLCDratio, u32 uSCALERratio, u32 uHOSTratio) 
{
	u32 uRegValue;

	uRegValue =
		((uHOSTratio<<20) |		
		(uSCALERratio<<16) |			
		(uLCDratio<<12) |
		(uMMC2ratio<<8)|			
		(uMMC1ratio<<4)|
		(uMMC0ratio<<0)) ;

	Outp32(rCLK_DIV1, uRegValue);

}

//////////
// Function Name : SYSC_SetDIV2
// Function Description : This function set Clock divider2 register ratio
//				 
// Input : 		  	uSPI0ratio 	: (0~15)
//					uSPI1ratio	: (0~15)
//					uAUDIO0ratio	: (0~15)
//					uAUDIO1ratio	: (0~15)
//					uUARTratio : (0~15)
//					uIRDAratio	: (0~15)
// Output : NONE
// Version : 
void SYSC_SetDIV2( u32 uSPI0ratio	, u32 uSPI1ratio, u32 uAUDIO0ratio, u32 uAUDIO1ratio, u32 uUARTratio, u32 uIRDAratio) 
{
	u32 uRegValue;

	uRegValue =
		((uIRDAratio<<20) |		
		(uUARTratio<<16) |			
		(uAUDIO1ratio<<12) |
		(uAUDIO0ratio<<8)|			
		(uSPI1ratio<<4)|
		(uSPI0ratio<<0)) ;

	Outp32(rCLK_DIV2, uRegValue);

}


//////////
// Function Name : SYSC_CtrlCLKGate
// Function Description : This function control HCLK_GATE Register
//				 
// Input : 		  	eGATE
//					uCtrl      :  Enable_CLK : (1), 
//							  Disable_CLK : (0)
// Output : NONE
// Version : 
void SYSC_CtrlCLKGate( CLOCK_eGATE eGate,  u32 uCtrl)
{
	u32 uRegValue, uTemp;

	uTemp = eGate;

	
	uRegValue =Inp32(rHCLK_GATE);
	uRegValue = (uRegValue & ~(0x1<<(uTemp))) | (uCtrl<<(uTemp));
	Outp32(rHCLK_GATE, uRegValue);

}


//////////
// Function Name : SYSC_CtrlHCLKGate
// Function Description : This function control HCLK_GATE Register
//				 
// Input : 		  	H_eGATE
//					uCtrl      :  Enable_CLK : (1), 
//							  Disable_CLK : (0)
// Output : NONE
// Version : 
void SYSC_CtrlHCLKGate( H_eGATE eHCLKGATE	, u32 uCtrl)
{
	u32 uRegValue, uTemp;

	uTemp = eHCLKGATE;

	
	uRegValue =Inp32(rHCLK_GATE);
	uRegValue = (uRegValue & ~(0x1<<(uTemp))) | (uCtrl<<(uTemp));
	Outp32(rHCLK_GATE, uRegValue);
}

//////////
// Function Name : SYSC_CtrlPCLKGate
// Function Description : This function control PCLK_GATE Register
//				 
// Input : 		  	P_eGATE
//					uCtrl      :  Enable_CLK : (1), 
//							  Disable_CLK : (0)
// Output : NONE
// Version : 
void SYSC_CtrlPCLKGate( P_eGATE ePCLKGATE	, u32 uCtrl)
{
	u32 uRegValue, uTemp;

	uTemp = ePCLKGATE;

	
	uRegValue =Inp32(rPCLK_GATE);
	uRegValue = (uRegValue & ~(0x1<<(uTemp))) | (uCtrl<<(uTemp));
	Outp32(rPCLK_GATE, uRegValue);

}

//////////
// Function Name : SYSC_CtrlSCLKGate
// Function Description : This function control SCLK_GATE Register
//				 
// Input : 		  	S_eGATE
//					uCtrl      :  Enable_CLK : (1), 
//							  Disable_CLK : (0)
// Output : NONE
// Version : 
void SYSC_CtrlSCLKGate( S_eGATE eSCLKGATE	, u32 uCtrl)
{
	u32 uRegValue, uTemp;

	uTemp = eSCLKGATE;

	
	uRegValue =Inp32(rSCLK_GATE);
	uRegValue = (uRegValue & ~(0x1<<(uTemp))) | (uCtrl<<(uTemp));
	Outp32(rSCLK_GATE, uRegValue);

}

//////////
// Function Name : SYSC_SetCLKGateAll
// Function Description : This function control SCLK_GATE Register
//				 
// Input : NONE
// Output : NONE
// Version : 

void SYSC_SetCLKGateAll(u32 uOnOff)
{
	if(uOnOff==1) {
		Outp32(rPCLK_GATE, 0xffffffff);
		Outp32(rSCLK_GATE, 0xffffffff);
		Outp32(rHCLK_GATE, 0xffffffff);
	}	
	else if(uOnOff==0) {
		Outp32(rPCLK_GATE, 0x0);
		Outp32(rSCLK_GATE, 0x0);
		Outp32(rHCLK_GATE, 0x0);
	}	
		
}

*/

//////////
// Function Name : SYSC_CtrlCLKOUT
// Function Description : This function select CLK output 
//				 
// Input : 		  	CLKOUT_eTYPE :  FOUT_APLL, FOUT_EPLL, HCLK, CLK27M, CLK48M, RTC, Tick, DOUT
//					uDivVAL     :  (0~15)
//							  
// Output : NONE
// Version : 
void SYSC_CtrlCLKOUT( CLKOUT_eTYPE eCLKOUT_TYPE, u32 uDivVAL)
{
	u32 uRegValue;
    u32 uFunct;

        
    GPIO_SetFunctionEach(eGPIO_F, eGPIO_14, 3);	// Select GPIO_SELECT
    GPIO_SetPullUpDownEach(eGPIO_F, eGPIO_14, 0);
       	
    uFunct = eCLKOUT_TYPE;
	uRegValue =Inp32(rCLK_OUT);
	uRegValue = (uRegValue & ~(0xFF<<12)) | ((uFunct<<12)|(uDivVAL<<16));
	Outp32(rCLK_OUT, uRegValue);

}

/*
//////////
// Function Name : SYSC_CtrlDCLK
// Function Description : This function control  DCLK output 
//				 
// Input : 		  	uDCLKCMP :(0~15) DCLK compare value, clock toggle value, 
//					uDCLKDIV : (0~15) DCLK divid value,  F_DCLK= Src_CLK/(uDCLKDIV+1)
//					uDCLKSEL :  "0" PCLK,  "1" 48MHz
//					uDCLKEN   : Enable_CLK : (1), 
//							    Disable_CLK : (0)
// Output : NONE
// Version : 
void SYSC_CtrlDCLK( u32 uDCLKCMP, u32 uDCLKDIV, u32 uDCLKSEL, u32 uDCLKEN)
{
	u32 uRegValue;
		
	uRegValue =Inp32(rCLK_OUT);
	uRegValue = (uRegValue & ~(0xFFF<<0)) | ((uDCLKCMP<<8)|(uDCLKDIV<<4)|(uDCLKSEL<<1)|(uDCLKEN<<0));
	Outp32(rCLK_OUT, uRegValue);

}

	// Value & 0xF00 : The BUS Control Register ID, OffSet
	// Value & 0x0F0 : The BUS Priority Type ID
	// Value & 0x00F : The Fixed Priority Order


//////////
// Function Name : SYSC_CtrlBUSPrio
// Function Description : This function control AHB Bus Priority
//				 
// Input : 		  BUSCTRL_eId
// Output : NONE
// Version : 
void SYSC_CtrlBUSPrio( BUSCTRL_eId  eBusId)
{
	u32 uOffset, uPType, uForder, uOffset1;
	u32 uRegValue;

	uOffset = (eBusId&0xF00)>>8;
	uPType = (eBusId&0x0F0)>>4;
	uForder = eBusId&0x00F;

	if(uOffset <= 3)
	{
		uOffset1 = uOffset;
		uRegValue = Inp32(rAHB_CON0);
		uRegValue = (uRegValue & ~(0xFF<<(uOffset1*7))) | ((uPType<<(uOffset1*7 + 4))|(uForder<<(uOffset1*7)));
		Outp32(rAHB_CON0, uRegValue);
	}
	else if(uOffset >= 4 && uOffset <= 7)
	{
		uOffset1 = uOffset - 4;
		uRegValue = Inp32(rAHB_CON1);
		uRegValue = (uRegValue & ~(0xFF<<(uOffset1*7))) | ((uPType<<(uOffset1*7 + 4))|(uForder<<(uOffset1*7)));
		Outp32(rAHB_CON1, uRegValue);
	}
	else if(uOffset >= 8 && uOffset <= 9)
	{
		uOffset1 = uOffset - 8;
		uRegValue = Inp32(rAHB_CON2);
		uRegValue = (uRegValue & ~(0xFF<<(uOffset1*7))) | ((uPType<<(uOffset1*7 + 4))|(uForder<<(uOffset1*7)));
		Outp32(rAHB_CON2, uRegValue);
	}

}



//////////
// Function Name : SYSC_CtrlBUSLock
// Function Description : This function control HLOCK for each bus
//				 
// Input : 		  
// Output : NONE
// Version : 


//  TBD



//////////
// Function Name : SYSC_SeletDMA
// Function Description : This function select between SDMA and General DMA
//				 
// Input : 		  	SEL_eSDMA : 
//					uSEL     :  #define		SEL_SDMA		(0)
//							 #define		SEL_GDMA		(1)
//							  
// Output : NONE
// Version : 
void SYSC_SelectDMA( DMASELECT_eID eSEL_DMA, u32 uSEL)
{
	u32 uRegValue;
	u32 uOffset;

	uOffset = eSEL_DMA;	
	uRegValue =Inp32(rSDMA_SEL);
	
	uRegValue = (uRegValue & ~(0x1<<uOffset)) | (uSEL<<uOffset);
	Outp32(rSDMA_SEL, uRegValue);

}



//////////
// Function Name : SYSC_CtrlEBIPrio
// Function Description : This function control EBI Priority
//				 
// Input : 		  EBIPrio_eID
// Output : NONE
// Version : 
void SYSC_CtrlEBIPrio( EBIPrio_eID  eEBIId)
{
	u32 uPType, uForder;
	u32 uRegValue;

	uPType = (eEBIId&0xF0)>>4;
	uForder = eEBIId&0x0F;

	uRegValue = Inp32(rMEM_SYS_CFG);
	uRegValue = (uRegValue & ~(0xF<<8)) | ((uPType<<11)|(uForder<<8));
	Outp32(rMEM_SYS_CFG, uRegValue);

}

//////////
// Function Name : SYSC_CtrlCSMEM0
// Function Description : This function control static memory chip selection muxing of MEM0
//				 
// Input : 		  			eM0CSn2 : eCS_SROMC, eCS_ONDC, eCS_NFC
//							eM0CSn3 : eCS_SROMC, eCS_ONDC, eCS_NFC
//							eM0CSn4 : eCS_SROMC, eCS_CFC
//							eM0CSn5 : eCS_SROMC, eCS_CFC
// Output : NONE
// Version : 
void SYSC_CtrlCSMEM0( SelCS_eMEM0  eM0CSn2, SelCS_eMEM0 eM0CSn3, SelCS_eMEM0 eM0CSn4, SelCS_eMEM0 eM0CSn5 )
{
	u32 uRegValue;

	uRegValue = Inp32(rMEM_SYS_CFG);
	uRegValue = (uRegValue & ~(0x3F<<0)) | ((eM0CSn5<<5)|(eM0CSn4<<4)|(eM0CSn3<<2)|(eM0CSn2<<0));
	Outp32(rMEM_SYS_CFG, uRegValue);

}


//////////
// Function Name : SYSC_SetQos
// Function Description : This function set QOS_OV_ID
//				 
// Input : 		  
// Output : NONE
// Version : 


//  TBD

*/
//////////
// Function Name : SYSC_RdMEMCFG
// Function Description : This function read MEM_CFC_STAT Register
//				 
// Input :   NONE	  			
// Output : NONE
// Version : 
void SYSC_RdMEMCFGSTAT( void )
{
	u32 uRegValue;
	u32 uTemp;

	uRegValue = Inp32(rMEM_CFG_STAT);

	// EBI Priority Scheme
	uTemp = ( uRegValue >>15 ) & 0x1;
	UART_Printf(" Current EBI Priority Scheme 	(0: Fixed, 1: Circular) :    %d \n",  uTemp);
	uTemp = (uRegValue >> 12) & 0x7 ;
	UART_Printf(" Current EBI Fixed Priority setting					 :    %d \n",  uTemp);

	// CF I/F
	uTemp = ( uRegValue >>10 ) & 0x1;
	UART_Printf(" Current CF I/F Setting  (0: EBI, 1: Independet)	 :    %d \n",  uTemp);

	// NAND Type Setting
	uTemp = ( uRegValue >>9 ) & 0x1;
	UART_Printf(" Current NAND Type (0:OneNAND, 1: NAND)		 :    %d \n",  uTemp);
	uTemp = ( uRegValue >>3 ) & 0x1;
	UART_Printf(" Current NAND Init Setting  (0: Normal NAND, 1: Advanced NAND)	 :    %d \n",  uTemp);
	uTemp = ( uRegValue >>2 ) & 0x1;
	UART_Printf(" Show address cycle init. setting of NAND  	 :    %d \n",  uTemp);
	uTemp = ( uRegValue >>0 ) & 0x1;
	UART_Printf(" Show NAND Page Size	 :    %d \n",  uTemp);

	uTemp = ( uRegValue >>8 ) & 0x1;
	UART_Printf(" Current CS0 Bus Width  (0: 8-bit, 1: 16-bit)		 :    %d \n",  uTemp);
	uTemp = ( uRegValue >>1 ) & 0x1;
	UART_Printf(" Show CS0 Bus width (init. Setting)  (0:8bit, 1: 16bit)	 :    %d \n",  uTemp);

	uTemp = ( uRegValue >>7 ) & 0x1;
	UART_Printf(" NAND Booting   (0: not Used, 1: used)	 		:    %d \n",  uTemp);

	uTemp = ( uRegValue >>5 ) & 0x3;
	UART_Printf(" Current Booting Type  (0: NFCON, 1: SROMC, 2: ONDC,  3: Internal ROM)	 :    %d \n",  uTemp);

	uTemp = ( uRegValue >>4 ) & 0x1;
	UART_Printf(" Current ADDR Expand  (0: Used MEM1 Data, 1:Used MEM0 Addr)	 :    %d \n",  uTemp);


}

u32 SYSC_ReadMemStat(void)
{
	return Inp32(rMEM_CFG_STAT);
}

/*
//////////
// Function Name : SYSC_RdMEMCFG
// Function Description : This function gets MEM_CFC_STAT Register
//				 
// Input :   NONE	  			
// Output : NONE
// rb1004
u32 SYSC_GetMEMCFGSTAT( void )
{
	return Inp32(rMEM_CFG_STAT);
}


//////////
// Function Name : SYSC_SetCFGWFI
// Function Description : This function configure ARM1176 STANBYWFI ( CFG_STANDBY (in PWR_CFG Register))
//				  ( CFG_STANDBYWFI & OSC Control in Power Mode )
// Input : 		  	eWFIMode : eIGNORE, eIDLE, eSTOP, eSLEEP
//					uOSCCLK     :  #define		Disable_CLK		(0)
//								 #define		Enable_CLK		(1)
//							  
// Output : NONE
// Version : 
void SYSC_SetCFGWFI( CFG_eWFI eWFIMode, u32 uOSCCLK)
{
	u32 uRegValue, uRegValue1;

	uRegValue =Inp32(rPWR_CFG);

	switch(eWFIMode)
	{
		case eIGNORE: 
			uRegValue = (uRegValue & ~(0x61<<0)) | ((eWFIMode<<5)|(1<<0));		// CFG_STANDBY= ignore,  OSC_27 = Enable
			Outp32(rPWR_CFG, uRegValue);  break;
		case eIDLE: 
			uRegValue = (uRegValue & ~(0x61<<0)) | ((eWFIMode<<5)|(uOSCCLK<<0));		
			Outp32(rPWR_CFG, uRegValue);  break;
		case eSTOP: 
			uRegValue = (uRegValue & ~(0x61<<0)) | ((eWFIMode<<5)|(uOSCCLK<<0));
			uRegValue1=Inp32(rSTOP_CFG);
			uRegValue1 = (uRegValue1 & ~(0x3<<0)) | ((uOSCCLK<<1)|(uOSCCLK<<0));
			//uRegValue1 = (uRegValue1 & ~(0x3<<0)) | ((uOSCCLK<<1)|(1<<0));
			Outp32(rPWR_CFG, uRegValue); 
			Outp32(rSTOP_CFG, uRegValue1); 
			break;
		case eSLEEP: 
			uRegValue = (uRegValue & ~(0x61<<0)) | ((eWFIMode<<5)|(uOSCCLK<<0));
			uRegValue1=Inp32(rSLEEP_CFG);
			uRegValue1 = (uRegValue1 & ~(0x61<<0)) | ((uOSCCLK<<0));
			Outp32(rPWR_CFG, uRegValue); 
			Outp32(rSLEEP_CFG, uRegValue1); 
			break;
								
	}

}

//////////
// Function Name : SYSC_SetBATF
// Function Description : This function configure BATFLT ( (in PWR_CFG Register))
//				  
// Input : 		  	eBATFLT : eFLT_IGNORE, eFLT_INT, eFLT_SLEEP
//					uINTSRC     :  (0) Wake-up Source in ESLEEP :  nWRESET
//								(1) Wake-up Source in ESLEEP :  SLEEP Wake-up Source
//		
//							  
// Output : NONE
// Version : 
void SYSC_SetBATF( BATFLT_eMODE eBATFLT, u32 uINTSRC)
{
	u32 uRegValue;

	uRegValue =Inp32(rPWR_CFG);

	switch(eBATFLT)
	{
		case eFLT_IGNORE: 
			uRegValue = (uRegValue & ~(0x1F<<0)) | ((eBATFLT<<3)|(1<<0));		
			Outp32(rPWR_CFG, uRegValue);  break;
		case eFLT_INT: 
			uRegValue = (uRegValue & ~(0x1F<<0)) | ((eBATFLT<<3)|(1<<0));		
			Outp32(rPWR_CFG, uRegValue);  break;
		case eFLT_SLEEP: 
			uRegValue = (uRegValue & ~(0x1F<<0)) | ((eBATFLT<<3)|(uINTSRC<<2)|(1<<0));
			Outp32(rPWR_CFG, uRegValue); 
			break;					
	}

}

//////////
// Function Name : SYSC_SetNORCFG
// Function Description : This function configure Power Control of the internal block in the Stop Mode 
//				  
// Input : 		  	uIROM :  Power Control of the IROM Sub Block (0: OFF, 1:ON)
//					uDM_ETM :  Power Control of the DOMAIN_ETM Sub Block (0: OFF, 1:ON)
//					uDM_S :  Power Control of the DOMAIN_S Sub Block (0: OFF, 1:ON)
//					uDM_F :  Power Control of the DOMAIN_F Sub Block (0: OFF, 1:ON)
//					uDM_P :  Power Control of the DOMAIN_P Sub Block (0: OFF, 1:ON)
//					uDM_I :  Power Control of the DOMAIN_I Sub Block (0: OFF, 1:ON)
//					uDM_SS :  Power Control of the DOMAIN_SS Sub Block (0: ON, 1:OFF)
//					uDM_G :  Power Control of the DOMAIN_G Sub Block (0: OFF, 1:ON)
//					uDM_V :  Power Control of the DOMAIN_V Sub Block (0: OFF, 1:ON)
// Output : NONE
// Version : 
void SYSC_SetNORCFG( u32 uIROM, u32 uDM_ETM, u32 uDM_S, u32 uDM_F, u32 uDM_P, u32 uDM_I, u32 uDM_SS, u32 uDM_G, u32 uDM_V)
{
	u32 uRegValue;

	uRegValue =Inp32(rNORMAL_CFG);
	uRegValue = (uRegValue & ~(1<<30|0xFF<<9)) | ((uIROM<<30)|(uDM_ETM<<16)|(uDM_S<<15)|(uDM_F<<14)|(uDM_P<<13)
		             |(uDM_I<<12)|(uDM_SS<<11)|(uDM_G<<10)|(uDM_V<<9));
	Outp32(rNORMAL_CFG, uRegValue); 
}


//////////
// Function Name : SYSC_SetSTOPCFG
// Function Description : This function configure Power Control of the internal block in the Stop Mode 
//				  
// Input : 		  	uMEM_ARM :  Power Control of the Memory Block in ARM Module (0: OFF, 1:ON)
//					uMEM_TOP :  Power Control of the Memory Block in Top Module (0: OFF, 1:ON)
//					uLogic_ARM :  Power Control of the Logic Block in ARM Module (0: OFF, 1:ON)
//					uLogic_ARM :  Power Control of the Logic Block in Top Module (0: OFF, 1:ON)		  
// Output : NONE
// Version : 
void SYSC_SetSTOPCFG( u32 uMEM_ARM, u32 uMEM_Top, u32 uLogic_ARM, u32 uLogic_Top)
{
	u32 uRegValue;

	uRegValue =Inp32(rSTOP_CFG);
#if 1 //original
	uRegValue = (uRegValue & ~(1<<29|1<<20|1<<17|1<<8|1)) | ((uMEM_ARM<<29)|(uMEM_Top<<20)|(uLogic_ARM<<17)|(uLogic_Top<<8)|(1));
#elif 0 // sub-block on
	uRegValue = (uRegValue & ~(0x3FFFFF<<8)) | ((uMEM_ARM<<29)|(uMEM_Top<<20)|(uLogic_ARM<<17)|(uLogic_Top<<8)|(0xfd<<9));
#else // all on
	uRegValue=0xffffffff;
#endif

	Outp32(rSTOP_CFG, uRegValue); 
}


//////////
// Function Name : SYSC_SetSTOPMEMCFG
// Function Description : This function configure Power Control of the internal memory block in the Stop Mode 
//				  
// Input : 		  	uMODEM : Power Control of the Memory Block in MODEM I/F (0: OFF, 1:RETENTION)
//					uHOST : Power Control of the Memory Block in HOST I/F (0: OFF, 1:RETENTION)
//					uOTG : Power Control of the Memory Block in OTG (0: OFF, 1:RETENTION)
//					uHSMMC : Power Control of the Memory Block in HSMMC (0: OFF, 1:RETENTION)
//					uIROM: Power Control of the Memory Block in IROM (0: OFF, 1:RETENTION)
//					uIRDA : Power Control of the Memory Block in IRDA (0: OFF, 1:RETENTION)
//					uNFCON : Power Control of the Memory Block in NFCON(Stepping Stone) (0: OFF, 1:RETENTION)
// Output : NONE
// Version : 
void SYSC_SetSTOPMEMCFG( u32 uMODEM, u32 uHOST, u32 uOTG, u32 uHSMMC, u32 uIROM, u32 uIRDA, u32 uNFCON)
{
	u32 uRegValue;

	uRegValue =Inp32(rSTOP_MEM_CFG);
	uRegValue = (uRegValue & ~(0x7F)) | ((uMODEM<<6)|(uHOST<<5)|(uOTG<<4)|(uHSMMC<<3)|(uIROM<<2)|(uIRDA<<1)|(uNFCON<<0));
	Outp32(rSTOP_MEM_CFG, uRegValue); 
}



//////////
// Function Name : SYSC_SetWakeMASK
// Function Description : This function control Wake-up source Mask  
//				  
// Input : 		  	WKUPMSK_eID : Wake-up Source ID
//					uMaskCtrl     :  (0) Mask disable , Wake-up source Enable
//								 (1) Mask Enable,  Wake-up source disable
//		
//							  
// Output : NONE
// Version : 
void SYSC_SetWakeMASK( WKUPMSK_eID eWKUPSRC, u32 uMaskCtrl)
{
	u32 uRegValue;
	u32 uRegOffset, uMaskID;

	uRegOffset = (eWKUPSRC&0xF00)>>8;
	uMaskID = eWKUPSRC&0x0FF;

	if(uRegOffset == 0)
	{
		uRegValue =Inp32(rPWR_CFG);
		uRegValue = (uRegValue & ~(0x1<<(uMaskID+7))) | (uMaskCtrl<<(uMaskID+7));	
		Outp32(rPWR_CFG, uRegValue);
	}
	else if(uRegOffset == 1)
	{
		uRegValue = Inp32(rEINT_MASK);
		uRegValue = (uRegValue & ~(0x1<<(uMaskID))) | (uMaskCtrl<<(uMaskID));
		Outp32(rEINT_MASK, uRegValue);
	}
	
}


//////////
// Function Name : SYSC_CtrlNORCFG
// Function Description : This function control Block Power off of the internal logic during normal operation 
//				  
// Input : 		  	eCFGId : Internal block  Domain Id
//					uPwrCtrl     :   (0) LP mode,  Block Power Off
//								 (1) Active mode, Block Power ON
//		
//							  
// Output : NONE
// Version : 
void SYSC_CtrlNORCFG( NORGFC_eID eCFGId, u32 uPwrCtrl)
{
	u32 uRegValue, uTemp;

	uRegValue =Inp32(rNORMAL_CFG);
	uTemp = eCFGId;

       uRegValue = (uRegValue & ~(1<<uTemp)) | (uPwrCtrl <<uTemp);
	Outp32(rNORMAL_CFG, uRegValue);
}


//////////
// Function Name : SYSC_CtrlSTOPCFG
// Function Description : This function control Block Power off of the internal logic during normal operation 
//				  
// Input : 		  	eCFGId : Internal block  Domain Id
//					uPwrCtrl     :   (0) LP mode,  Block Power Off
//								 (1) Active mode, Block Power ON
//		
//							  
// Output : NONE
// Version : 
void SYSC_CtrlSTOPCFG( STOPCFG_eID eCFGId, u32 uPwrCtrlStop)
{
	u32 uRegValue, uTemp;

	uRegValue =Inp32(rSTOP_CFG);
	uTemp = eCFGId;

       uRegValue = (uRegValue & ~(1<<uTemp)) | (uPwrCtrlStop <<uTemp);
	Outp32(rSTOP_CFG, uRegValue);
}

//////////
// Function Name : SYSC_CtrlSTOPMEMCFG
// Function Description : This function control Block Power off of the internal logic during normal operation 
//				  
// Input :			eStopMemCfgId : Top Memory Id
//					uPwrCtrlStopMem :	 (0) LP mode,  Block Power Off
//								 (1) Active mode, Block Power ON
//		
//							  
// Output : NONE
// Version : 
void SYSC_CtrlSTOPMEMCFG(STOPMEMCFG_eID eStopMemCfgId, u32 uPwrCtrlStopMem )
{
	u32 uRegValue, uTemp;

	uRegValue =Inp32(rSTOP_MEM_CFG);
	uTemp = eStopMemCfgId;

       uRegValue = (uRegValue & ~(1<<uTemp)) | (uPwrCtrlStopMem <<uTemp);
	Outp32(rSTOP_MEM_CFG, uRegValue);			
}



*/
//////////
// Function Name : SYSC_ChangeMode
// Function Description : This function change Operating Mode
//				  
// Input : 		  	
//					eOPMode     :   eASYNC_MODE(0) Asynchronous mode
//								  eSYNC_MODE (1) Synchronous mode
//		
//							  
// Output : NONE
// Version : 
void SYSC_ChangeMode(SYSC_eOPMode eOPMode)
{
	u32 uRegValue,  uCurrMode, uDIV0, uDIV0_org, uSRC, uSRC_org;
	u32 uHCLKx2_Ratio;

	uRegValue =Inp32(rOTHERS);
	uCurrMode = (uRegValue >> 8 ) & 0xF ;
	uDIV0_org = Inp32(rCLK_DIV0);
	uSRC_org = Inp32(rCLK_SRC);
	uHCLKx2_Ratio=(uDIV0_org>>9)&0x7;

	
	if((eOPMode == 0)&&(uCurrMode == 0xF))				// Current = SyncMode, Next = AsyncMode	
	{
//		UART_Printf("Mode is changed to Asyn. from Sync.\n");
		// For FirmWare Test => APLL Out = MPLL*2
		if(uHCLKx2_Ratio == 0)
		{
			uDIV0_org = uDIV0_org;
		}
		else 
		{
			uDIV0_org = (uDIV0_org & ~(0x7<<9)) | (0 <<9);
		}
		
		uSRC = (uSRC_org & ~(0x3<<0)) | (0 <<0);				// A&MPLL Change to Clock Source
		uDIV0 = (uDIV0_org & ~(0xFFFF<<0)) | (0x1100 <<0);	// Clock Divider 1:1 ??

		Outp32(rCLK_SRC, uSRC);
		Outp32(rCLK_DIV0, uDIV0);
		
		uRegValue = (uRegValue & ~(0x3<<6)) | (1<<6);		// SYNCMODE(SYNCREQ) = ASYNC, VICEN= Async, MUXSEL=Sync
		Outp32(rOTHERS, uRegValue);

		while(((Inp32(rOTHERS)>>8)&0xF));				// Wait SYNCMODEACK = 0x0

		uRegValue = Inp32(rOTHERS);
		uRegValue = (uRegValue&~(0x1<<6)) | (0<<6);		// Change SYNCMUX to AsyncMode
		Outp32(rOTHERS, uRegValue);

		Outp32(rCLK_DIV0, uDIV0_org);
		Outp32(rCLK_SRC, uSRC_org);

		
	}
	else if ((eOPMode == 1)&&(uCurrMode == 0x0))			// Current = AsyncMode, Next = SyncMode	
	{
//		UART_Printf("Mode is changed to Syn. from Aync.\n");
		uSRC = (uSRC_org & ~(0x3<<0)) | (0 <<0);				// A&MPLL Change to Clock Source
		uDIV0 = (uDIV0_org & ~(0xFFFF<<0)) | (0x1100 <<0);	      // Clock Divider 1:1 ??

		if(uHCLKx2_Ratio == 0)
		{
			uDIV0_org = (uDIV0_org & ~(0x7<<9)) | (1 <<9);
		}
		else 
		{
			uDIV0_org = uDIV0_org;
			
		}

		Outp32(rCLK_SRC, uSRC);
		Outp32(rCLK_DIV0, uDIV0);
	
		uRegValue = (uRegValue & ~(0x1<<6)) | (1<<6);		// Change SYNCMUX to SyncMode
		Outp32(rOTHERS, uRegValue);

		asm volatile ("nop");
		asm volatile ("nop");
		asm volatile ("nop");
		asm volatile ("nop");
		asm volatile ("nop");

		uRegValue = (uRegValue & ~(0x1<<7)) | (1<<7);		// SYNCMODE = SYNC, VICEN=SYNC, MUXSEL=SYNC
		Outp32(rOTHERS, uRegValue);
		
		while(!(((Inp32(rOTHERS)>>8)&0xf) == 0xF));			// Wait SYNCMODEACK = 0xF

		
		Outp32(rCLK_DIV0, uDIV0_org);
	
		Outp32(rCLK_SRC, uSRC_org);
	
	}

	Delay(10);
		
}

void SYSC_ChangeSYSCLK_1(APLL_eOUT  eAPLLMPS, APLL_eOUT eMPLLMPS, u32 uARM_Ratio, u32 uHCLKx2_Ratio, u32 uPCLK_Ratio)
{
	u32 uRegValue_A;
	u32 uRegValue_M;//, uRegValue_E;
	u32 uMVAL, uPVAL, uSVAL;
	u32 uMVAL_M, uPVAL_M, uSVAL_M;
	u32 uSYNCMODE;
	u32 uRegValue_DIV0, uRegValue_SRC, uTemp, uTemp1;
	
	uMVAL = (eAPLLMPS&0x3FF0000)>>16;
	uPVAL = (eAPLLMPS&0xFF00)>>8;
	uSVAL = eAPLLMPS&0x00FF;
	uRegValue_A = (u32)(((u32)(0x1<<31))|(uMVAL<<16)|(uPVAL<<8)|(uSVAL<<0));

	uMVAL_M = (eMPLLMPS&0x3FF0000)>>16;
	uPVAL_M = (eMPLLMPS&0xFF00)>>8;
	uSVAL_M = eMPLLMPS&0x00FF;


	uSYNCMODE = (Inp32(rOTHERS)>>8)&0xF;


	if(uSYNCMODE==0xF )		// Sync Mode
	{
	
		uRegValue_M = Inp32(rMPLL_CON);

	}
	else			// ASync Mode
	{
		uRegValue_M = (u32)(((u32)(0x1<<31))|(uMVAL_M<<16)|(uPVAL_M<<8)|(uSVAL_M<<0));
		//uRegValue_E = Inp32(rEPLL_CON0);
	}


	// Clock Divider Change 1:2:2
	uRegValue_DIV0 = Inp32(rCLK_DIV0);
	uTemp = (uRegValue_DIV0&~(0xFFFF))|(1<<12)|((0<<9)|(1<<8)|(1<<4)|(0<<0));		// ARM:HCLKx2:HCLK:PCLK = 1:1:2:2

	// Set-up Divider Value
	uRegValue_DIV0 = (uRegValue_DIV0 & ~(0xFE07)) | ((uPCLK_Ratio<<12)|(uHCLKx2_Ratio<<9)|(uARM_Ratio<<0));

	uRegValue_SRC = Inp32(rCLK_SRC);
	uTemp1 = (uRegValue_SRC&~(0x7))|0x0;

	Outp32(rCLK_SRC, uTemp1);	// Clock Source Change
	Outp32(rCLK_DIV0, uTemp);	// Change Clock Divider
	
	
	// PLL Change
	if(uSYNCMODE == 0)
	{
		Outp32(rMPLL_CON, uRegValue_M);
	}

	Outp32(rAPLL_CON, uRegValue_A);

	while(((Inp32(rAPLL_CON)>>30)&0x1)==0);


	// Change Divider Value
	Outp32(rCLK_DIV0, uRegValue_DIV0);
	// Change Clock SRC
	Outp32(rCLK_SRC, uRegValue_SRC);

	
}
/*
//////////
// Function Name : SYSC_CheckOpMode
// Function Description : This function return the current operation mode
//				  
// Input : 		  	
//							  
// Output : NONE
// Version : 
u32 SYSC_CheckOpMode(void)
{
	u32 uRegValue;

	uRegValue =Inp32(rOTHERS);
       uRegValue &= (1<<7);

	return (uRegValue>>7);		
}


//////////
// Function Name : SYSC_ClrBATFINT
// Function Description : This function clear int pending bit about nBATF Int.
//				  
// Input : 		  	
//							  
// Output : NONE
// Version : 
void SYSC_ClrBATFINT(void)
{
	u32 uRegValue;

	uRegValue =Inp32(rOTHERS);
       uRegValue |= (1<<12);
       Outp32(rOTHERS, uRegValue);
		
}






//////////
// Function Name : SYSC_RdRSTSTAT
// Function Description : This function read Rest Status Register
//				  
// Input : NONE		  	
//							  
// Output : Status Value
// Version : 
u32 SYSC_RdRSTSTAT(u32 uPRINT)
{
	u32 uRegValue;
	u32 uRstId;

	uRegValue =Inp32(rRST_STAT);

	if(uPRINT == 1)
	UART_Printf("Reset Status Register:	0x%x\n",  uRegValue);

	if( uRegValue & (1<<0))
	{
		if(uPRINT == 1)
		UART_Printf("External reset by XnRESET  \n\n");
		uRstId = 0;
	}
	else if (uRegValue & (1<<1))
	{
		if(uPRINT == 1)
		UART_Printf("Warm reset by XnWRESET \n\n");
		uRstId = 1;
		
	}
	else if (uRegValue & (1<<2))
	{
		if(uPRINT == 1)
		UART_Printf("Watch dog timer reset by WDTRST\n\n");
		uRstId = 2;
		
	}
	else if (uRegValue & (1<<3))
	{
		if(uPRINT == 1)
		UART_Printf("Reset by SLEEP Mode Wake-up \n\n");
		uRstId = 3;
		
	}
	else if (uRegValue & (1<<4))
	{
		if(uPRINT == 1)
		UART_Printf("Reset by ESLEEP mode Wake-up \n\n");
		uRstId = 4;
	}
	else if (uRegValue & (1<<5))
	{
		if(uPRINT == 1)
		UART_Printf("Software reset by SWRESET\n\n");
		uRstId = 5;		
	}	

	return uRstId;
}

//////////
// Function Name : SYSC_RdBLKPWR
// Function Description : This function read Block Power Status Register
//				  
// Input : 		  	
//							  
// Output : NONE
// Version : 
void SYSC_RdBLKPWR(void)
{
	u32 uRegValue;

	uRegValue =Inp32(rBLK_PWR_STAT);
	UART_Printf("Block Power Status Register	0x%x\n",  uRegValue);
#if 0
	UART_Printf("Block Top Power	%d\n",  uRegValue&(0x1<<0));
	UART_Printf("Block V    Power	%d\n",  (uRegValue&(0x1<<1))>>1);
	UART_Printf("Block I	    Power	%d\n",  (uRegValue&(0x1<<2))>>2);	
	UART_Printf("Block P    Power	%d\n",  (uRegValue&(0x1<<3))>>3);	
	UART_Printf("Block F	    Power	%d\n",  (uRegValue&(0x1<<4))>>4);	
	UART_Printf("Block S    Power	%d\n",  (uRegValue&(0x1<<5))>>5);	
	UART_Printf("Block ETM    Power	%d\n",  (uRegValue&(0x1<<6))>>6);	
#endif		
}

//////////
// Function Name : SYSC_WaitingForBLKPWR
// Function Description : This function read Block Power Status Register
//				  
// Input : eCFGId : Internal block power status Id	
//		 uOnOff  : On or Off
//							  
// Output : NONE
// Version : 
void SYSC_WaitingForBLKPWR(BLKPWRSTA_eID eCFGId, u32 uOnOff)
{
	u32 uRegValue;

	while(!(Inp32(rBLK_PWR_STAT)&(1<<eCFGId)==uOnOff));
	
}

//////////
// Function Name : SYSC_WaitingForBLKPWR
// Function Description : This function read Block Power Status Register
//				  
// Input : eCFGId : Internal block power status Id	
//		 uOnOff  : On or Off
//							  
// Output : NONE
// Version : 
void SYSC_WaitingForBLKPWRAll(u32 uOnOff)
{
	u32 uRegValue;

	if(uOnOff==1)
		while(!(Inp32(rBLK_PWR_STAT)&(0xff)==0xff));
	else if(uOnOff==0)
		while(!(Inp32(rBLK_PWR_STAT)&(0xff)==0x800));
	else Assert(0);

	
}



//////////
// Function Name : SYSC_ClrWKUPSTAT
// Function Description : This function clear Wake-up Status Register
//				  
// Input : 		  	
//							  
// Output : NONE
// Version : 
void SYSC_ClrWKUPSTAT(void)
{
	u32 uRegValue;

	uRegValue =Inp32(rWAKEUP_STAT);

//	UART_Printf("Wake-up Status Register:0x%x\n",  uRegValue);

	switch(uRegValue&0xfff) {
		case 0x1:
			UART_Printf("___ Wake-up by EINT[0x%x] \n", uRegValue);
			uRegValue |= (1<<0);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x2:
			UART_Printf("___ Wake-up by RTC ALARM[0x%x] \n", uRegValue);
			uRegValue |= (1<<1);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x4:
			UART_Printf("___ Wake-up by RTC Tick[0x%x] \n", uRegValue);
			uRegValue |= (1<<2);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x8:
			UART_Printf("___ Wake-up by Touch Screen [0x%x] \n", uRegValue);
			uRegValue |= (1<<3);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x10:
			UART_Printf("___ Wake-up by KeyPad [0x%x] \n", uRegValue);
			uRegValue |= (1<<4);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x20:
			UART_Printf("___ Wake-up by MSM Modem[0x%x] \n", uRegValue);
			uRegValue |= (1<<5);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x40:
			UART_Printf("___ Wake-up by BATFLT[0x%x] \n", uRegValue);
			uRegValue |= (1<<6);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x80:
			UART_Printf("___ Wake-up by Warm Reset[0x%x] \n", uRegValue);
			uRegValue |= (1<<7);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x100:
			UART_Printf("___ Wake-up by HSI[0x%x] \n", uRegValue);
			uRegValue |= (1<<8);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x200:
			UART_Printf("___ Wake-up by MMC0[0x%x] \n", uRegValue);
			uRegValue |= (1<<9);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x400:
			UART_Printf("___ Wake-up by MMC1[0x%x] \n", uRegValue);
			uRegValue |= (1<<10);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		case 0x800:
			UART_Printf("___ Wake-up by MMC2[0x%x] \n", uRegValue);
			uRegValue |= (1<<11);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
		default:
			UART_Printf("___ Wake-up by Wrong[0x%x] \n", uRegValue);
			uRegValue |= (1<<11);
			Outp32(rWAKEUP_STAT,uRegValue);
			break;
	}	
}


//////////
// Function Name : SYSC_InitData_SDRAM
// Function Description : This function initializes data in the DRAM
//					 
// Input : 	check_start - start address, check_size- data size
// Output : 	
//			
// Version : v0.1
void SYSC_InitData_SDRAM(int check_start, int check_size)
{
    int i;
#if DBG_PRINT
    UART_Printf("[SDRAM Initialization]\n");
    UART_Printf("Fill SDRAM for self-refresh test.\n");
#endif
    for(i=check_start;i<(check_start+check_size);i+=4)
    	*((unsigned int *)i)=i^0x55555555;
    	//*((unsigned int *)i)=0x55555555;
#if DBG_PRINT    
    UART_Printf("Filling SDRAM is completed.\n");
#endif
    
}

//////////
// Function Name : SYSC_CheckData_SDRAM
// Function Description : This function check data in the DRAM 
//					  
// Input : 	check_start - start address, check_size- data size
// Output : 	
//			
// Version : v0.1
void SYSC_CheckData_SDRAM(int check_start, int check_size)
{
    int i;
    int error=0;
	
    dprintf("Check SDRAM for self-refresh test\n");
    for(i=check_start;i<(check_start+check_size);i+=4)
    {
    	if(*((unsigned int *)i)!=(i^0x55555555))
    	{
    	    dprintf("Mem Error:%x=%x(%x)\n",i,*((unsigned int *)i),i^0x55555555);
    	    error++;
    	}
    	if(error>20)
	    break;
    }
    if(error)
    	{
      	UART_Printf("SDRAM self-refresh test:FAILED\n"); 
	while(1);	
      	}
    else
    	{
    	UART_Printf("SDRAM self-refresh test:O.K.\n");	
    	}

}


//////////
// Function Name : SYSC_WDT_Reset
// Function Description : WatchDog Reset Setting
// Input : 	None
// Output :	None 
// Version : v0.1
void SYSC_WDT_Reset(void)
{

	Outp32(0x7e004000,(0<<8|1<<5|0<<3|0<<2|1));
	Outp32(0x7e004004, 0xffff);
	Outp32(0x7e004004, 0xffff);
}

/*
void SYSC_SDHC_Clock_On(u32 uCh)
{
	SDHC sCh;
	int uDivisor = 0;
	SDHC_OpenMediaWithMode( 1, SDHC_POLLING_MODE, SDHC_EPLL, 0, uCh, &sCh);
	Outp32( sCh.m_uBaseAddr+SDHC_CONTROL2, (0x0<<15)|(0x0<<14)|(0x1<<8)|(SDHC_EPLL<<4) );
	Outp16( sCh.m_uBaseAddr+SDHC_CLK_CTRL, (Inp16(sCh.m_uBaseAddr+SDHC_CLK_CTRL)&(~(0xff<<8)))|(uDivisor<<8)|(1<<0) );
	SDHC_SetSdClockOnOff(1, &sCh); // If the sd clock is to be changed, you need to stop sd-clock.
}
*/

