#include "common.h"
#include "option.h"
#include "library.h"
#include "system.h"
#include "timer.h"
#include "intc.h"
#include "sysc.h"
#include "gpio_dev.h"
#include "main.h"

// Interrupt Handler Global valiable
   volatile u32 g_uIntCounterOS = 0;
   volatile u32 g_uIntCounter0 = 0;
   volatile u32 g_uIntCounter1 = 0;
   volatile u32 g_uIntCounter2 = 0;
   volatile u32 g_uIntCounter3 = 0;
   volatile u32 g_uIntCounter4 = 0;
   //-daedoo
//   volatile u32 g_uIntCounterWT = 0;
   volatile u32 g_PWMDmaDone;


void PWM_Select(u32 uTimer,u32 uPrescaler,u32 uDivider,u32 uTCNTB,u32 uTCMPB,u32 uEnDz,u32 uDzlen,u32 uEnDMA,u32 uAutoreload,u32 uEnInverter,u32 uEnInt)
{

	u32	uTemp0;
	u32	uTemp1;
	u32	uTemp2;
	u32  uTemp3;


	if(uEnDMA == 1) uEnDMA = uTimer + 1;

	uTemp0 = Inp32(rTCFG1);
	uTemp0 = (uTemp0 & (~(0xf<<4*uTimer))& (~(1<<20))) |(uEnDMA<<20)|(uDivider<<4*uTimer);
	Outp32(rTCFG1,uTemp0);

	uTemp0 = Inp32(rTINT_CSTAT);
	uTemp0 = (uTemp0 & (~(1<<uTimer)))|(uEnInt<<(uTimer));
	Outp32(rTINT_CSTAT,uTemp0);

	switch(uTimer)
	{
	case 0:													//Timer0
		uTemp0 = Inp32(rTCON) & (0xfffffffe);
		Outp32(rTCON, uTemp0); 								// Timer0 stop

		uTemp0 = (Inp32(rTCFG0) & (~(0xff00ff))) | ((uPrescaler-1)<<0) |(uDzlen<<16);
		Outp32(rTCFG0, uTemp0);
		
		Outp32(rTCNTB0, uTCNTB);
		Outp32(rTCMPB0, uTCMPB);


		uTemp1 = (Inp32(rTCON) & (~(0x1f))) |(uEnDz<<4)|(uAutoreload<<3)|(uEnInverter<<2)|(1<<1)|(0<<0);
		Outp32(rTCON, uTemp1);									//timer0 manual update
		uTemp1 = (uTemp1 & (~(0x1f))) |(uEnDz<<4)|(uAutoreload<<3)|(uEnInverter<<2)|(0<<1)|(1<<0);				
		Outp32(rTCON, uTemp1);									// timer0 start
		break;

	case 1:													//Timer1
		uTemp0 = Inp32(rTCON) & (0xfffffeff);
		Outp32(rTCON, uTemp0); 								// Timer1 stop

		uTemp0 = (Inp32(rTCFG0) & (~(0xff00ff))) | ((uPrescaler-1)<<0) |(uDzlen<<16);
		Outp32(rTCFG0, uTemp0);
		
		Outp32(rTCNTB1, uTCNTB);
		Outp32(rTCMPB1, uTCMPB);

		uTemp1 = (Inp32(rTCON) & (~(0xf00))) | (uAutoreload<<11)|(uEnInverter<<10)|(1<<9)|(0<<8);
		Outp32(rTCON,uTemp1); 									// timer1 manual update
		uTemp1 = (uTemp1 & (~(0xf00))) | (uAutoreload<<11)|(uEnInverter<<10)|(0<<9)|(1<<8);				
		Outp32(rTCON,uTemp1); 									// timer1 start
		break;
			
	case 2:													//Timer2
		uTemp0 = Inp32(rTCON) & (0xffffefff);
		Outp32(rTCON, uTemp0); 								// Timer2 stop

		uTemp2 = (Inp32(rTCFG0) & (~(0xffff00)))|((uPrescaler-1)<<8)|(uDzlen<<16);
		Outp32(rTCFG0, uTemp2);
		
		Outp32(rTCNTB2, uTCNTB);
		Outp32(rTCMPB2, uTCMPB);

		uTemp3 = (Inp32(rTCON) & (~(0xf000)))|(uAutoreload<<15)|(uEnInverter<<14)|(1<<13)|(0<<12);
		Outp32(rTCON, uTemp3); 								// timer2 manual update

		uTemp3 = (uTemp3 & (~(0xf000)))|(uAutoreload<<15)|(uEnInverter<<14)|(0<<13)|(1<<12);
		Outp32(rTCON,uTemp3);									 // timer2 start
		break;
		
	case 3:														//Timer3
		uTemp0 = Inp32(rTCON) & (0xfffeffff);
		Outp32(rTCON, uTemp0);									 // Timer3 stop

		uTemp2 = (Inp32(rTCFG0) & (~(0xffff00)))|((uPrescaler-1)<<8)|(uDzlen<<16);
		Outp32(rTCFG0, uTemp2);
		
		Outp32(rTCNTB3, uTCNTB);
		Outp32(rTCMPB3, uTCMPB);

		uTemp3 = (Inp32(rTCON) &(~(0xf0000)))|(uAutoreload<<19)|(uEnInverter<<18)|(1<<17)|(0<<16);
		Outp32(rTCON, uTemp3); 								// timer3 manual update
		uTemp3 = (uTemp3 &(~(0xf0000)))|(uAutoreload<<19)|(uEnInverter<<18)|(0<<17)|(1<<16);
		Outp32(rTCON,uTemp3); 									// timer3 start
		break;

	case 4:														//Timer4
		uTemp0 = Inp32(rTCON) & (0xffefffff);
		Outp32(rTCON, uTemp0);									 // Timer4 stop
		
		uTemp2 = (Inp32(rTCFG0) & (~(0xffff00)))|((uPrescaler-1)<<8)|(uDzlen<<16);
		Outp32(rTCFG0, uTemp2);
		
		Outp32(rTCNTB4, uTCNTB);

		uTemp3 = (Inp32(rTCON) & (~(0xf00000)))|(uAutoreload<<22)|(1<<21)|(0<<20);
		Outp32(rTCON,uTemp3); 									// timer4 manual update

		uTemp3 = (uTemp3 & (~(0xf00000)))|(uAutoreload<<22)|(0<<21)|(1<<20);				
		Outp32(rTCON, uTemp3);									 // timer4 start
		break;

	}
}

void PWM_stopAll(void)
{
	Outp32(rTCON,0);
}

void PWM_stop(u32 uNum)
{
	u32 uTemp;

	uTemp = Inp32(rTCON);
	if(uNum == 0)	uTemp &= ~(0x1);
	else			uTemp &= ~((0x10)<<(uNum*4));
	Outp32(rTCON,uTemp);
}

u32 PWM_ReadrTCMPB(u32 uNum)
{
	u32 RegTmp;

	switch(uNum)
	{
		case 0:	RegTmp = Inp32(rTCMPB0);	break;
		case 1:	RegTmp = Inp32(rTCMPB1);	break;			
		case 2:	RegTmp = Inp32(rTCMPB2);	break;
		case 3:	RegTmp = Inp32(rTCMPB3);	break;			
	}

	return RegTmp;
}

/*
void __irq Isr_TIMEROS(void)
{
	u32 uTmp;
	u32 uCnt;
	
	uTmp = Inp32(rTINT_CSTAT);

	for(uCnt = 0 ; uCnt <50 ; uCnt++)
//	UART_Printf("#%d : rTINT_CSTAT = %x\n",uCnt,uTmp);

//	uTmp = (uTmp & 0x1f)| (1<<5);
	Outp32(rTINT_CSTAT,uTmp);       //  Timer0 Interrupt Clear register
    
//	UART_Printf("Timer0\n");
//	UART_Printf("IntCounter = %d \n ",g_uIntCounterOS);
	
	g_uIntCounterOS++;

    INTC_ClearVectAddr();
}

void __irq Isr_TIMER0(void)
{
	u32 uTmp;
	
	uTmp = Inp32(rTINT_CSTAT);
	uTmp = (uTmp & 0x1f)| (1<<5);
	Outp32(rTINT_CSTAT,uTmp);       //  Timer0 Interrupt Clear register
    
//    UART_Printf("Timer0\n");
	g_uIntCounter0++;

    INTC_ClearVectAddr();
}

void __irq Isr_TIMER1(void)
{
 	u32 uTmp;
	
	uTmp = Inp32(rTINT_CSTAT);
	uTmp = (uTmp & 0x1f) | (1<<6);
	Outp32(rTINT_CSTAT,uTmp);      //  Timer1 Interrupt Clear register
    
//    UART_Printf("Timer1\n");
    g_uIntCounter1++;
	
    INTC_ClearVectAddr();    
}

void __irq Isr_TIMER2(void)
{
	u32 uTmp;
	
	uTmp = Inp32(rTINT_CSTAT);
	uTmp = (uTmp & 0x1f) |(1<<7);
	Outp32(rTINT_CSTAT,uTmp);       //  Timer2 Interrupt Clear register

//	UART_Printf("Timer2\n");
	 g_uIntCounter2++;

	INTC_ClearVectAddr();    
}

void __irq Isr_TIMER3(void)
{
	u32 uTmp;
	
	uTmp = Inp32(rTINT_CSTAT);
	uTmp = (uTmp & 0x1f) | (1<<8);
	Outp32(rTINT_CSTAT,uTmp);       //  Timer3 Interrupt Clear register
    
//    UART_Printf("Timer3\n");
    g_uIntCounter3++;
	
    INTC_ClearVectAddr();    
}

void __irq Isr_TIMER4(void)
{
	u32 uTmp;
	
	uTmp = Inp32(rTINT_CSTAT);
	uTmp = (uTmp & 0x1f) | (1<<9);
	Outp32(rTINT_CSTAT,uTmp);       //  Timer4 Interrupt Clear register

//    UART_Printf("Timer4\n");     
    g_uIntCounter4++;
    
    INTC_ClearVectAddr();    
}
*/
////////////////////////////////Optional Functions////////////////////////////////////
void Clear_Timer(u8 id)
{
	u32 uTmp;
	
	uTmp = Inp32(rTINT_CSTAT);
	uTmp = (uTmp & 0x1f) | (1<<id + 5);
	Outp32(rTINT_CSTAT,uTmp);       //  Timer4 Interrupt Clear register

}

void DelayfrTimer(eDelay_M cMeasure, u32 uDnum)
{
	u32 uCnt;
	
	PWM_stop(2);							// Using PWM timer #2

	if( cMeasure == micro )
	{
		uCnt = PJBoot64.pclk/1000000;
		PWM_Select(2,uCnt,0,0xffffffff,(u32)(0xffffffff/2),0,0,0,1,0,0); // Timer2,Divider =1/1, Prescaler = g_PCLK/1M
		while((0xffffffff-Inp32(rTCNTO2)) != uDnum);
	}
	else if( cMeasure == milli )
	{
		uCnt = PJBoot64.pclk/1000000;
		PWM_Select(2,uCnt,0,0xffffffff,(u32)(0xffffffff/2),0,0,0,1,0,0); // Timer2,Divider =1/1, Prescaler = g_PCLK/1M
		while((0xffffffff-Inp32(rTCNTO2)) != (uDnum*1000));
	}

	PWM_Select(2,uCnt,0,0xffffffff,(u32)(0xffffffff/2),0,0,0,1,0,0); // Timer2,Divider =1/1, Prescaler = g_PCLK/1M
}

void StartTimer(u32 uTsel)
{
	u32 uCnt;

	uCnt = PJBoot64.pclk/1000000;

	if(uTsel == 3) PWM_Select(uTsel,uCnt,5,0xffffffff,(u32)(0xffffffff/2),0,0,0,1,0,0);	//timer0,prescaler=PCLK/1M,divider=1/1,autoreload
	else		   PWM_Select(uTsel,uCnt,0,0xffffffff,(u32)(0xffffffff/2),0,0,0,1,0,0);	//timer0,prescaler=PCLK/1M,divider=1/1,autoreload
}

u32 StopTimer(u32 uTsel)
{
	u32 uVal;
	
	PWM_stop(uTsel);
	uVal = 0xffffffff - (Inp32(rTCNTO0+(uTsel*12)));

	return uVal;
}

u32 GetTickCount(void)
{
	u32 u32Cnt;
	u32Cnt = Inp32(rTCNTO2);
	
	return u32Cnt;
}
