//#include <stdio.h>
//#include <stdlib.h>

//#include "def.h"
#include "common.h"
#include "library.h"
#include "gpio_dev.h"
#include "watchdogtimer.h"
#include "intc.h"
#include "main.h"

volatile u32 g_uIntCounterWT = 0;

void __attribute__ ((interrupt("IRQ"))) Isr_WDT(void)
{

	Outp32(rWTCLRINT ,1);    // Watchdog timer interrupt clear 	
	INTC_Disable(NUM_WDT);
	
	g_uIntCounterWT++;
//	printf("%d ", g_uIntCounterWT);
	 
    	INTC_ClearVectAddr();    
	INTC_Enable(NUM_WDT);
}

void WDT_Init(u32 uPreVal, DIV_FACTOR div_factor, u32 uWdtDat)
{
	u32 div_clk = 0;

	switch (div_factor)
	{
		case DIV16:
			div_clk = CLKSEL_16;
			break;
		case DIV32:
			div_clk = CLKSEL_32;
			break;
		case DIV64:
			div_clk = CLKSEL_64;
			break;
		case DIV128:
			div_clk = CLKSEL_128;
			break;
	}

	Outp32(rWTCON, PREVAL(uPreVal) | div_clk);
	Outp32(rWTDAT, uWdtDat);
	Outp32(rWTCNT, uWdtDat);

}

void WDT_Enable(u8 bData)
{
	u32 uRead;

	uRead = Inp32(rWTCON);
	Outp32(rWTCON, uRead&~(1<<5)|(bData<<5));
}

void WDT_IntEnable(u8 bData)
{
	u32 uRead;

	uRead = Inp32(rWTCON);
	Outp32(rWTCON, uRead&~(1<<2)|(bData<<2));

}

void WDT_EnableReset(u8 bData)
{
	u32 uRead;

	uRead = Inp32(rWTCON);
	Outp32(rWTCON, uRead&~(1<<0)|(bData<<0));
}

extern u32 delayLoopCount;
void CalibrateDelay(void)
{
	u32 i, tmpDelay=100;
	u32 uPCLK_MHz = PJBoot64.pclk/1000000;
	
	delayLoopCount = 5000;
	
	Outp32(rWTCON, (uPCLK_MHz-1)<<8);
	Outp32(rWTDAT, 0xffff);						//for first update
	Outp32(rWTCNT, 0xffff);						//resolution=16us @any PCLK 
	Outp32(rWTCON, ((uPCLK_MHz-1)<<8)|(1<<5));	//Watch-dog timer start
	
	for(;tmpDelay>0;tmpDelay--)
		for(i=0;i<delayLoopCount;i++)
			asm volatile ("nop");
	
	Outp32(rWTCON, (uPCLK_MHz-1)<<8);   		//Watch-dog timer stop
	i = 0xffff - Inp32(rWTCNT);                 // 1count->16us, 100*5000 cycle runtime = 
	delayLoopCount = 50000000/(i*16);         	//100*5000:16*i[us]=1*x:100[us] -> x=50000000/(16*i)
}
