//#include "def.h"
#include "common.h"

#include "main.h"
#include "library.h"
#include "lcd_dev.h"
#include "lcd_dep.h"
#include "intc.h"
#include "glib_dep.h"
#include "timer.h"

extern LCDC oLcdc;

void LCD_GetFrmSz(u32* uFrmHSz, u32* uFrmVSz, LCD_WINDOW eWin)
{
	*uFrmHSz = oLcdc.m_uFrmHSz[eWin];
	*uFrmVSz = oLcdc.m_uFrmVSz[eWin];
}

void LCD_Blend_ColorKeyOn(u8 NonKey_R, u8 NonKey_G, u8 NonKey_B, u8 Key_R, u8 Key_G, u8 Key_B, u32 Key_Color, LCD_WINDOW eWin)
{
	LCD_DEV_SetBlendingType(PER_PLANE, ALPHA0_PLANE, eWin);
	LCD_DEV_SetAlpha(NonKey_R, NonKey_G, NonKey_B, Key_R, Key_G, Key_B, eWin);
	
	LCD_DEV_SetColorKeyOnOff(1, eWin);
	LCD_DEV_SetColorKey(0x0, Key_Color, MATCH_FG_IMAGE, true, eWin);
}

void LCD_Blend_ColorKeyOff(LCD_WINDOW eWin)
{
	LCD_DEV_SetAlpha(0xf, 0xf, 0xf, 0xf, 0xf, 0xf, eWin);
	LCD_DEV_SetColorKeyOnOff(0, eWin);
}

void Switch_GLIBControl(LCD_WINDOW eWin)
{
	if(PJBoot64.lcd_en) GLIB_Init(oLcdc.m_uFbStAddr[eWin], oLcdc.m_uFrmHSz[eWin], oLcdc.m_uFrmVSz[eWin], RGB24);
}

void LCD_Move_Arrow_Win(u32 arrow, LCD_WINDOW eWin)
{
	u32 i;
	u32 value;
	
	if(PJBoot64.lcd_en == FALSE) return;
	
	Switch_GLIBControl(eWin);

	if(arrow == POSITION_RANDOM)
	{
		printf("%d [[\n", GetTickCount());
		arrow = GetTickCount() % 4;
		arrow = 1 << arrow;
		printf("]] %d\n", GetTickCount());
	}

	switch(arrow)
	{
		case 1:	// left
		
			value = PJBoot64.lcd_xsize / 32;
			
			for(i=1;i<=32;i++)
			{
				GLIB_FillRectangle(PJBoot64.lcd_xsize - i*value, 0, value, PJBoot64.lcd_ysize, NO_MASKING_COLOR);
				LCD_Blend_ColorKeyOn(0xf, 0xf, 0xf, 0xf - i/2, 0xf - i/2, 0xf - i/2, NO_MASKING_COLOR, eWin);
//				Delay(1000000);
			}
			break;
		case 2:	// right
			value = PJBoot64.lcd_xsize / 32;
			for(i=0;i<32;i++)
			{
				GLIB_FillRectangle(i*value, 0, value, PJBoot64.lcd_ysize, NO_MASKING_COLOR);
				LCD_Blend_ColorKeyOn(0xf, 0xf, 0xf, 0xf - i/2, 0xf - i/2, 0xf - i/2, NO_MASKING_COLOR, eWin);
//				Delay(1000000);
			}
			break;
		case 4:	// up
			value = PJBoot64.lcd_ysize / 32;
			for(i=1;i<=32;i++)
			{
				GLIB_FillRectangle(0, PJBoot64.lcd_ysize - i*value, PJBoot64.lcd_xsize, value, NO_MASKING_COLOR);
				LCD_Blend_ColorKeyOn(0xf, 0xf, 0xf, 0xf - i/2, 0xf - i/2, 0xf - i/2, NO_MASKING_COLOR, eWin);
//				Delay(1000000);
			}
			break;
		case 8:	// down
			value = PJBoot64.lcd_ysize / 32;
			for(i=0;i<32;i++)
			{
				GLIB_FillRectangle(0, i*value, PJBoot64.lcd_xsize, value, NO_MASKING_COLOR);
				LCD_Blend_ColorKeyOn(0xf, 0xf, 0xf, 0xf - i/2, 0xf - i/2, 0xf - i/2, NO_MASKING_COLOR, eWin);
//				Delay(1000000);
			}
			break;
		default:
			return;
	}

	//switch
	Switch_Win0_Win1();
}

void LCD_GetFbEndAddr(u32* uFbAddr, LCD_WINDOW eWin) 
{ 
	*uFbAddr = oLcdc.m_uFbEndAddr[eWin] + 4;
}

u32 LCD_GetFbStartAddr(LCD_WINDOW eWin) 
{ 
	return oLcdc.m_uFbStAddr[eWin]; 
}

void LCD_Init(u32 start_frame_buffer)
{
	LCD_Dev_Init(start_frame_buffer, PJBoot64.lcd_xsize, PJBoot64.lcd_ysize);
	
	// Background Layer 0
	LCD_InitWin(RGB24, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz, 0, 0, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz,	0, 0, start_frame_buffer, WIN0, false);
	LCD_DEV_SetWinOnOff(1, WIN0);
	// Default Graphic Setting
	GLIB_Init(start_frame_buffer, PJBoot64.lcd_xsize, PJBoot64.lcd_ysize, RGB24);
	GLIB_ClearFrame(WHITE, 0);
	
	// Background Layer 1
	LCD_GetFbEndAddr(&start_frame_buffer, WIN0);
	LCD_InitWin(RGB24, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz, 0, 0, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz,	0, 0, start_frame_buffer, WIN1, false);
	LCD_DEV_SetWinOnOff(1, WIN1);
	LCD_Blend_ColorKeyOn(0xf, 0xf, 0xf, 0, 0, 0, NO_MASKING_COLOR, WIN1);
	Switch_GLIBControl(WIN1);
	GLIB_ClearFrame(NO_MASKING_COLOR, 0);
	
	// Icon Layer
	LCD_GetFbEndAddr(&start_frame_buffer, WIN1);
	LCD_InitWin(RGB24, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz, 0, 0, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz,	0, 0, start_frame_buffer, WIN2, false);
	LCD_DEV_SetWinOnOff(1, WIN2);
	LCD_Blend_ColorKeyOn(0xb, 0xb, 0xb, 0, 0, 0, NO_MASKING_COLOR, WIN2);
	Switch_GLIBControl(WIN2);
	GLIB_ClearFrame(NO_MASKING_COLOR, 0);
	
	LCD_GetFbEndAddr(&start_frame_buffer, WIN2);
	LCD_InitWin(RGB24, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz, 0, 0, oLcdc.m_uLcdHSz, oLcdc.m_uLcdVSz,	0, 0, start_frame_buffer, WIN3, false);
	LCD_DEV_SetWinOnOff(1, WIN3);
	LCD_Blend_ColorKeyOn(0, 0, 0, 0, 0, 0, NO_MASKING_COLOR, WIN3);
//	Switch_GLIBControl(WIN3);
//	GLIB_ClearFrame(NO_MASKING_COLOR, 0);

	LCD_GetFbEndAddr(&start_frame_buffer, WIN3);
	LCD_InitWin(RGB24, oLcdc.m_uLcdHSz / 2, oLcdc.m_uLcdVSz / 2, 0, 0, oLcdc.m_uLcdHSz / 2, oLcdc.m_uLcdVSz / 2, oLcdc.m_uLcdHSz / 4, oLcdc.m_uLcdVSz / 4, start_frame_buffer, WIN4, false);
	LCD_DEV_SetWinOnOff(1, WIN4);
	LCD_Blend_ColorKeyOn(0, 0, 0, 0, 0, 0, NO_MASKING_COLOR, WIN4);
	Switch_GLIBControl(WIN4);
	GLIB_ClearFrame(NO_MASKING_COLOR, 0);

//	PJBoot64.lcd_en = TRUE;
}

void LCD_Start(void)
{
	if(PJBoot64.lcd_en)	LCD_DEV_Start();
}

void LCD_Stop(void)
{
	LCD_DEV_Stop();
	PJBoot64.lcd_en = FALSE;
}
