//#include <stdio.h>

//#include "def.h"
#include "common.h"

#include "main.h"
#include "nand_dep.h"
#include "nand_dev.h"

NAND_oInform NAND_Inform[NAND_CONNUM];

void Board_Nand_Init(u32 id)
{
	u32 NandId;
	
	NandId = Nand_Device_ReadID_NotInit(0);
//	printf("nand id : %x\n", NandId);
	
	// Nand Flash ID Setting..
	switch(NandId)
	{
		case ID_K9S1208V0M:	// Normal SLC, 64M
			NAND_Inform[id].uNandType = NAND_Normal8bit;
			NAND_Inform[id].uBlockNum = 4096;
			
			////////////////// NAND Block ////////////////////
			PJBoot64.nand.Booter = 0;
			PJBoot64.nand.Bootloader = 1;					// 256KB, 2 Block using
			PJBoot64.nand.Boot_CFG = 3;						// 128KB, 1 Block using
			PJBoot64.nand.Boot_Icon = 4;					// 386KB, 3 Block using, 1 Block free
			// Total : 1MB (8 Block)
			
			PJBoot64.nand.Linux_Kernel = 8;					// 4MB,   32 Block using
			PJBoot64.nand.Android_root = 40;				// 64MB,  512 Block using
			PJBoot64.nand.Android_data = 552;				// 64MB,  512 Block using
			
			PJBoot64.nand.Firmware = 8;						// 1MB,	  8 Block using
			PJBoot64.nand.Firmware_etc = 16;				// 1MB,   8 Block using
			
			PJBoot64.nand.Wince_Eboot = 8;					// 128KB, 1 Block using, 1 Block free
			PJBoot64.nand.Wince_Kernel = 10;				// 40MB - 128KB,  318 Block using
			
			PJBoot64.nand.Etc = NAND_Inform[id].uBlockNum;	// 69MB, 552 Block free
			//////////////////////////////////////////////////
			break;
		
		case ID_K9F2G08U0M:	// Normal SLC, 256M
			NAND_Inform[id].uNandType = NAND_Advanced8bit;
			NAND_Inform[id].uBlockNum = 2048;
			
			////////////////// NAND Block ////////////////////
			PJBoot64.nand.Booter = 0;
			PJBoot64.nand.Bootloader = 1;					// 256KB, 2 Block using
			PJBoot64.nand.Boot_CFG = 3;						// 128KB, 1 Block using
			PJBoot64.nand.Boot_Icon = 4;					// 386KB, 3 Block using, 1 Block free
			// Total : 1MB (8 Block)
			
			PJBoot64.nand.Linux_Kernel = 8;					// 4MB,   32 Block using
			PJBoot64.nand.Android_root = 40;				// 64MB,  512 Block using
			PJBoot64.nand.Android_data = 552;				// 64MB,  512 Block using
			// Total : 132MB (1056 Block)
			
			PJBoot64.nand.Firmware = 8;						// 1MB,	  8 Block using
			PJBoot64.nand.Firmware_etc = 16;				// 1MB,   8 Block using
			// Total : 2MB (16 Block)
			
			PJBoot64.nand.Wince_Eboot = 8;					// 128KB, 1 Block using, 1 Block free
			PJBoot64.nand.Wince_Kernel = 10;				// 40MB - 128KB,  318 Block using
			// Total : 40MB (320 Block)
			
			PJBoot64.nand.Etc = NAND_Inform[id].uBlockNum;	// 69MB, 552 Block free
			// Total : 69MB (552 Block)
			//////////////////////////////////////////////////
			break;
			
		case ID_K9G8G08U0M:	// Advanced MLC, 2G
			NAND_Inform[id].uNandType = NAND_MLC8bit;
			NAND_Inform[id].uBlockNum = 4096;
			
			////////////////// NAND Block ////////////////////
			PJBoot64.nand.Booter = 0;
			PJBoot64.nand.Bootloader = 1;					// 256KB, 2 Block using
			PJBoot64.nand.Boot_CFG = 3;						// 128KB, 1 Block using
			PJBoot64.nand.Boot_Icon = 4;					// 386KB, 3 Block using, 1 Block free
			// Total : 1MB (8 Block)
			
			PJBoot64.nand.Linux_Kernel = 8;					// 4MB,   32 Block using
			PJBoot64.nand.Android_root = 40;				// 64MB,  512 Block using
			PJBoot64.nand.Android_data = 552;				// 64MB,  512 Block using
			// Total : 132MB (1056 Block)
			
			PJBoot64.nand.Firmware = 8;						// 1MB,	  8 Block using
			PJBoot64.nand.Firmware_etc = 16;				// 1MB,   8 Block using
			// Total : 2MB (16 Block)
			
			PJBoot64.nand.Wince_Eboot = 8;					// 128KB, 1 Block using, 1 Block free
			PJBoot64.nand.Wince_Kernel = 10;				// 40MB - 128KB,  318 Block using
			// Total : 40MB (320 Block)
			
			PJBoot64.nand.Etc = NAND_Inform[id].uBlockNum;	// 69MB, 552 Block free
			// Total : 69MB (552 Block)
			//////////////////////////////////////////////////
			break;
			
		default:
			printf("Not Support Nand Flash Memory!\n");
			while(1);
	}
	
	// Nand Common Type Setting..
	switch(NAND_Inform[id].uNandType)
	{
		case NAND_Normal8bit:	// Normal SLC
			NAND_Inform[id].uAddrCycle = 4;
			NAND_Inform[id].uPageNum = 32;
			NAND_Inform[id].uPageSize = NAND_PAGE_512;
			NAND_Inform[id].uSpareSize = NAND_SPARE_16;
			NAND_Inform[id].uTacls = 10;
			NAND_Inform[id].uTwrph0 = 25+30;		//Pad delay : about 10ns
			NAND_Inform[id].uTwrph1 = 15;			//tWH : 15ns
			break;
		case NAND_Advanced8bit:	// Advanced SLC
			NAND_Inform[id].uAddrCycle = 5;
			NAND_Inform[id].uPageNum = 64;
			NAND_Inform[id].uPageSize = NAND_PAGE_2048;
			NAND_Inform[id].uSpareSize = NAND_SPARE_64;
			NAND_Inform[id].uTacls = 30;
			NAND_Inform[id].uTwrph0 = 12+30;		//Pad delay : about 10ns
			NAND_Inform[id].uTwrph1 = 5+30;
			break;
		case NAND_MLC8bit:		// Advanced MLC
			NAND_Inform[id].uAddrCycle = 5;
			NAND_Inform[id].uPageNum = 128;
			NAND_Inform[id].uPageSize = NAND_PAGE_2048;
			NAND_Inform[id].uSpareSize = NAND_SPARE_64;
			NAND_Inform[id].uTacls = 0;
			NAND_Inform[id].uTwrph0 = 15+20;		//Pad delay : about 10ns
			NAND_Inform[id].uTwrph1 = 5;			//tWH : 5ns + margin
			break;
	}
	
	Nand_Device_Init(id);
	PJBoot64.nand_id = NandId;
	PJBoot64.nand_type = NAND_Inform[id].uNandType;
	PJBoot64.nand_1_block = NAND_Inform[id].uPageSize * NAND_Inform[id].uPageNum;
	PJBoot64.nand_total_block = NAND_Inform[id].uBlockNum;
}

u32 Nand_Read_Process(u32 id, u32 start_block, u32 destAddress, u32 speed)
{
	u8 *Dest_Pt;
	u32 read_page, read_block, page_size;
	u32 i;
	u32 first = 1;
	
	Dest_Pt = (u8*)destAddress;
	read_block = start_block;

	while(1)	
	{
		// Block Error Check
		if(Nand_Device_CheckInvalidBlock(id, read_block) != eNAND_NoError)
		{
			read_block++;
			start_block++;
			continue;
		}
		
		if(first)
		{
			first = 0;
			Nand_Device_ReadPage(id, read_block, 0, Dest_Pt, NULL);
			page_size = *((u32*)Dest_Pt);
			read_page = 1;
			
			// start_block, 0 page error check;
			if(page_size > 	(NAND_Inform[id].uBlockNum * NAND_Inform[id].uPageNum) || page_size == 0) return 0;
		}
		else
		{
			read_page = 0;
		}

		if(speed) MemoPad_Update(".");
	
		for(i=read_page;i<NAND_Inform[id].uPageNum;i++)
		{
			Nand_Device_ReadPage(id, read_block, i, Dest_Pt, NULL);
			
			Dest_Pt += NAND_Inform[id].uPageSize;
			
			page_size--;
			if(page_size == 0) break;
		}

		if(page_size == 0) break;
		
		if(read_block >= NAND_Inform[id].uBlockNum) 
		{
			if(speed) MemoPad_Update(" : Error!!!\n");
			return 1;	
		}
		read_block++;
	}
	
	if(speed) MemoPad_Update(" : Success!!!\n");
	return ((u32)Dest_Pt - destAddress);
}

u32 Nand_Write_Process(u32 id, u32 start_block, u32 filesize, u32 sourAddress, u32 speed)
{
	u8 *Dest_Pt, *Src_Pt, *Save_Pt;
	u32 no_block, no_page, i, total_page, save_total_page, error = 0;
	u32 write_block, write_page, first = 1;
	
	if(filesize == 0) 
	{
		MemoPad_Update("Not Load Image\n");
		return 1;
	}
	
	Src_Pt = (u8*)sourAddress;
	write_block = start_block;
	
	// Size Setting..
	no_block = (u32)((filesize / NAND_Inform[id].uPageSize) / NAND_Inform[id].uPageNum);
	no_page = (u32)((filesize / NAND_Inform[id].uPageSize) % NAND_Inform[id].uPageNum);
	if(filesize % NAND_Inform[id].uPageSize) no_page++;
	total_page = (no_block * NAND_Inform[id].uPageNum) + no_page;
		
	if(speed) MemoPad_Update("File : %d[%d-block,%d-page].\n", filesize, no_block, no_page);

	Dest_Pt = (u8*)NAND_TEMP_ADDRESS;
	for(i=0;i<NAND_Inform[id].uPageSize;i++) *(Dest_Pt + i) = 0;

	*((u32*)Dest_Pt) = total_page;
	
	while(1)
	{
		// Block Erase
		if(Nand_Device_EraseSingleBlock(id, write_block) != eNAND_NoError)
		{
			write_block++;
			start_block++;
			continue;
		}
		
		// ǵư  .
		Save_Pt = Src_Pt;
		save_total_page = total_page;
		
		// ù°  ũ ..
		if(first)
		{
			if(Nand_Device_WritePage(id, write_block, 0, Dest_Pt, NULL)) 
			{
				write_block++;
				start_block++;
				continue;
			}

			write_page = 1;
			first = 0;
		}
		else
		{
			write_page = 0;
		}

		if(speed) MemoPad_Update(".");
		
		// Nand Flash  ..
		for(i=write_page;i<NAND_Inform[id].uPageNum;i++)
		{
			if(Nand_Device_WritePage(id, write_block, i, Src_Pt, NULL)) 
			{
				error = 1;
				break;
			}
			
			Src_Pt += NAND_Inform[id].uPageSize;
			
			total_page--;
			if(total_page == 0) break;
		}
		
		//  üũ.
		if(error)
		{
			Src_Pt = Save_Pt;
			total_page = save_total_page;
			error = 0;
			if(write_block == start_block) first = 1;
			continue;
		}
		
		if(total_page == 0) break;
		
		//    üũ..
		write_block++;
		if(write_block >= NAND_Inform[id].uBlockNum) 
		{
			if(speed) MemoPad_Update(" : Error!!!\n");
			return 1;	
		}
	}
	
	if(speed) MemoPad_Update(" : Success!!!\n");
	return 0;
}

u32 Nand_NoHead_Write_Process(u32 id, u32 start_block, u32 filesize, u32 sourAddress)
{
	u8 *Dest_Pt, *Src_Pt, *Save_Pt;
	u32 no_block, no_page, i, total_page, save_total_page, error = 0;
	u32 write_block, write_page, first = 1;
	
	if(filesize == 0) 
	{
		MemoPad_Update("Not Load Image\n");
		return 1;
	}
	
	Src_Pt = (u8*)sourAddress;
	write_block = start_block;
	
	// Size Setting..
	no_block = (u32)((filesize / NAND_Inform[id].uPageSize) / NAND_Inform[id].uPageNum);
	no_page = (u32)((filesize / NAND_Inform[id].uPageSize) % NAND_Inform[id].uPageNum);
	if(filesize % NAND_Inform[id].uPageSize) no_page++;
	total_page = (no_block * NAND_Inform[id].uPageNum) + no_page;
		
	MemoPad_Update("File : %d[%d-block,%d-page].\n", filesize, no_block, no_page);

	Dest_Pt = (u8*)NAND_TEMP_ADDRESS;
	for(i=0;i<NAND_Inform[id].uPageSize;i++) *(Dest_Pt + i) = 0;

	*((u32*)Dest_Pt) = total_page;
	
	while(1)
	{
		// Block Erase
		if(Nand_Device_EraseSingleBlock(id, write_block) != eNAND_NoError)
		{
			write_block++;
			continue;
		}
		
		// ǵư  .
		Save_Pt = Src_Pt;
		save_total_page = total_page;
		
		MemoPad_Update(".");
		
		// Nand Flash  ..
		for(i=0;i<NAND_Inform[id].uPageNum;i++)
		{
			if(Nand_Device_WritePage(id, write_block, i, Src_Pt, NULL)) 
			{
				error = 1;
				break;
			}
			
			Src_Pt += NAND_Inform[id].uPageSize;
			
			total_page--;
			if(total_page == 0) break;
		}
		
		//  üũ.
		if(error)
		{
			Src_Pt = Save_Pt;
			total_page = save_total_page;
			error = 0;
			continue;
		}
		
		if(total_page == 0) break;
		
		//    üũ..
		write_block++;
		if(write_block >= NAND_Inform[id].uBlockNum)
		{
			MemoPad_Update(" : Error!!!\n");
			return 1;
		}
	}
	
	MemoPad_Update(" : Success!!!\n");
	return 0;
}

void Nand_Format_Process(u32 id)
{
	u32 i, error = 0;

	Nand_Device_SoftUnLockBlock(id);
	
	for(i=1;i<NAND_Inform[id].uBlockNum;i++)
		if(Nand_Device_EraseSingleBlock(id, i) != eNAND_NoError) error++;
	
	MemoPad_Update("Nand Format Result\n");
	if(error) MemoPad_Update("Error %d Found\n", error);
}

void Nand_ErrorCheck_Process(u32 id)
{
	u32 i, error = 0;
	
	for(i=0;i<NAND_Inform[id].uBlockNum;i++)
	{
		if(Nand_Device_CheckInvalidBlock(id, i) != eNAND_NoError) error++;
	}
	
	MemoPad_Update("Nand Error Check Result\n");
	if(error) MemoPad_Update("Error %d Found\n", error);
}
