#include "common.h"
#include "timer.h"
#include "intc.h"
#include "queue_indep.h"
#include "touch_dep.h"
#include "touch_dev.h"
#include "main.h"

//#define debug_mode
#ifdef debug_mode
#define dprintf		printf
#else
#define dprintf		(void)
#endif

// ġ, ġ xǥ, ġ yǥ, ġ .
static u32 touched, touch_position_x, touch_position_y, touch_arrow;
static u32 x_stat[2], y_stat[2], queue_count;

u32 Read_Touch_arrow(void)
{
	u32 temp;
	if(touched) return 0;
	
	INTC_Disable(NUM_PENDNUP);
	temp = touch_arrow;
	touch_arrow = 0;
	INTC_Enable(NUM_PENDNUP);
	return temp;
}

u32 Get_Touch_Status(void)
{
	return touched;
}

u32 Get_Touch_XY(u32*x, u32*y)
{
	if(!touched) return 0;
	
	*x = touch_position_x;
	*y = touch_position_y;
	
	return 1;
}

void touch_check(void)
{
	u32 i, total_x = 0, total_y = 0;
	
	if(queue_count)
	{
		//  ġ  հ .
		i = queue_count;
		while(queue_count--)
		{
			total_x += queue_delete();
			total_y += queue_delete();
		}

		total_x /= i;
		total_y /= i;
	
		dprintf("%d %d, %d %d : ", total_x,touch_position_x, total_y,touch_position_y);
		
		(total_x > touch_position_x) ? (total_x = total_x - touch_position_x) : (total_x = touch_position_x - total_x);
		(total_y > touch_position_y) ? (total_y = total_y - touch_position_y) : (total_y = touch_position_y - total_y);
		
		dprintf("%d %d\n", total_x, total_y);

		// ּҰ ִ밪     ġ հ .
		if(total_x > 50)
		{
			if((x_stat[1] - x_stat[0]) > 200)
			{
				if(touch_position_x - x_stat[0] < 50 || touch_position_x < x_stat[0]) 
				{
					touch_arrow = 1<<0;
					dprintf("left move\n");
				}
				else if(x_stat[1] - touch_position_x < 50 || touch_position_x > x_stat[1]) 
				{
					touch_arrow = 1<<1;
					dprintf("right move\n");
				}
			}
		}
		else if(total_y > 50)
		{
			if((y_stat[1] - y_stat[0]) > 150)
			{
				if(touch_position_y - y_stat[0] < 50 || touch_position_y < y_stat[0]) 
				{
					touch_arrow |= (1<<2);
					dprintf("up move\n");
				}
				else if(y_stat[1] - touch_position_y < 50 || touch_position_y > y_stat[1]) 
				{
					touch_arrow |= (1<<3);
					dprintf("down move\n");
				}
			}
		}
	}
	queue_free();
}

/*---------------------------------- ISR Routines Definition ---------------------------------*/
void __attribute__ ((interrupt("IRQ")))	ADC_NormalISR(void)
{
	g_oADCTSInform.uADCSampleData 	= ADCTS_GetXPData();
	g_oADCTSInform.ucADCflag		= 1;

	ADCTS_SetADCClearInt();
	INTC_ClearVectAddr();
}

// Timer 4 Interrupt...
void __attribute__ ((interrupt("IRQ"))) Isr_TouchTracking(void)
{
	u32 tmpX, tmpY;
	u32 MIN_X, MIN_Y, MAX_X, MAX_Y;
	
	if(PJBoot64.board_id == 3)
	{
		MIN_X = 886, MIN_Y = 1365;
		MAX_X = 3337, MAX_Y = 2712;
	}
	else if(PJBoot64.board_id == 4)
	{
		MIN_X = 780, MIN_Y = 1250;
		MAX_X = 3430, MAX_Y = 2770;
	}

	Clear_Timer(4);
	INTC_Disable(NUM_PENDNUP);
	
	ADCTS_SetMeasureMode(eADCTS_MEASURE_AUTO_SEQ);
	ADC_InitADCCON();
	ADC_EnableStart(ENABLE);

	while(! (ADC_IsAfterStartup() )) ;	//	Wait for begin sampling
	while (! (ADC_IsEOCFlag()));		//	Wait for the EOC

	tmpX = ADCTS_GetXPData();
	tmpY = ADCTS_GetYPData();
	
	dprintf("X: %04d, Y: %04d\n", tmpX, tmpY);

	if(tmpX > (MIN_X - 10) && tmpY > (MIN_Y - 10)) 
	{
		if(tmpX < MIN_X) tmpX = MIN_X;
		else if(tmpX > MAX_X) tmpX = MAX_X;
		tmpX = (tmpX - MIN_X) * 10000 / ((MAX_X - MIN_X) * 10000 / PJBoot64.lcd_xsize);
		
		if(tmpY < MIN_Y) tmpY = MIN_Y;
		else if(tmpY > MAX_Y) tmpY = MAX_Y;
		if(PJBoot64.board_id == 3)	tmpY = (tmpY - MIN_Y) * 10000 / ((MAX_Y - MIN_Y) * 10000 / PJBoot64.lcd_ysize);
		else tmpY = PJBoot64.lcd_ysize - ((tmpY - MIN_Y) * 10000 / ((MAX_Y - MIN_Y) * 10000 / PJBoot64.lcd_ysize));

		if(touched == 0)
		{
			x_stat[0] = x_stat[1] = tmpX;
			y_stat[0] = y_stat[1] = tmpY;
			touched = 1;
		}
		else
		{
			if(touch_position_x < x_stat[0])		x_stat[0] = touch_position_x;
			else if(touch_position_x > x_stat[1])	x_stat[1] = touch_position_x;
			if(touch_position_y < y_stat[0])		y_stat[0] = touch_position_y;
			else if(touch_position_y > y_stat[1])	y_stat[1] = touch_position_y;

			queue_insert(touch_position_x);
			queue_insert(touch_position_y);
			
			if(queue_count > 4)
			{
				queue_delete();
				queue_delete();
			}
			else
			{
				queue_count++;
			}
		}

		touch_position_x = tmpX;
		touch_position_y = tmpY;
	}
	
	ADCTS_SetMeasureMode(eADCTS_MEASURE_FOR_STYLUS);
	ADCTS_SetStylusDetectMode(STYLUS_UP);
	
	INTC_ClearVectAddr();
	INTC_Enable(NUM_PENDNUP);
}

// ADCTS Interrupt Using.
void __attribute__ ((interrupt("IRQ"))) ADCTS_StylusTrackingISR(void)
{
	if ( ADCTS_GetXPStylusIntState() == STYLUS_DOWN )		// ġ  .
	{
		g_oADCTSInform.ucTouchStatus = ADCTS_TOUCH_DOWN;
		ADCTS_SetStylusDetectMode(STYLUS_UP);

		PWM_Select(4,255,4,600,0,0,0,0,1,0,1);	// 62.1us * 600 = 37.26ms
		INTC_Enable(NUM_TIMER4);
		touched = 0;
		touch_arrow = 0;
		queue_count = 0;
	}
	else 													// ġ  .
	{
		INTC_Disable(NUM_TIMER4);
		Clear_Timer(4);
		PWM_stop(4);
		touch_check();
		
		g_oADCTSInform.ucTouchStatus = ADCTS_TOUCH_UP;
		ADCTS_SetStylusDetectMode(STYLUS_DOWN);
		touched = 0;
	}

	ADCTS_SetRegValue(eADCTS_ADCUPDN, 0x0);
	ADCTS_SetADCClearInt();
	ADCTS_SetADCClearWKInt();
	INTC_ClearVectAddr();
}
/*------------------------------------ Touch Screen Init -----------------------------------*/
void Touch_Init(void)
{
	g_oADCTSInform.ucTouchStatus = ADCTS_TOUCH_INIT;
	ADC_InitADCCON();
//	ADCTS_SetDelay(500);		// ADCDLY[15:0]		
	ADCTS_SetDelayClkSrc(1);
	ADCTS_SetMeasureMode(eADCTS_MEASURE_FOR_STYLUS);
	INTC_SetVectAddr(NUM_PENDNUP, ADCTS_StylusTrackingISR);	
	INTC_Enable(NUM_PENDNUP);
	
	INTC_SetVectAddr(NUM_TIMER4,Isr_TouchTracking);
}

void Touch_Off(void)
{
	INTC_Disable(NUM_PENDNUP);
}
