//#include <stdio.h>
//#include <stdlib.h>
//#include <stdarg.h>
//#include <string.h>
//#include <ctype.h>

//#include "def.h"
#include "common.h"

#include "main.h"
#include "uart_dep.h"
#include "uart_dev.h"
//////////////////////////////////
static u32 debug_port = 0;

void UART_Printf(char *fmt,...)
{
	va_list ap;
	char string[256];

	if(PJBoot64.uart_en == 0) return;
	va_start(ap,fmt);
	vsprintf(string,fmt,ap);
	UART_PutStr(string);
	va_end(ap);
}

u32 Uart_Loader(u32 DownloadAddress)
{
	char buf[4];
	int i;
	u32	FileSize;
	u16	CheckSum=0,dnCS;

	if(!PJBoot64.uart_en) return 0;

	MemoPad_Update("\nDownloadAddress : 0x%08x\n", DownloadAddress);
	MemoPad_Update("STATUS : ");

	//	To get the file size.
	i = 0;
	while(1)
	{
		if(Uart_GetStatus(debug_port))
		{
			buf[i] = Uart_Getchar(debug_port);
			i++;
		}
		
		if(i == 4) break;
		if(Get_Touch_Status()) return 0;
	}
	
	FileSize = (buf[0]) + (buf[1]<<8) + (buf[2]<<16) + (buf[3]<<24);
	FileSize -= 4;
	
	for(i=0;i<FileSize;i++) {
		*((u8*)DownloadAddress + i) = UART_Getc();
		if((i&0x3ff) == 0) MemoPad_Update(".");
	}

	for(i=0;i<(FileSize-2);i++)	CheckSum+=*((u8 *)(DownloadAddress+i));

	dnCS=*((u8 *)(DownloadAddress+FileSize-2)) + (*( (u8 *)(DownloadAddress+FileSize-1) )<<8);

	if(CheckSum!=dnCS) {
		MemoPad_Update("\nChecksum Error!!! MEM : %x  DN : %x\n",CheckSum,dnCS);
		FileSize=0;
	} else {
		FileSize-=2;
		MemoPad_Update("\n%d bytes downloaded OK.\n",FileSize);
	}
    	
	return FileSize;
}

void UART_InitDebug(u8 ch, u32 uBaudRate)
{
	static UART_CON	Uartcon;
	
	Uartcon.uBaudrate = 115200;
	Uartcon.cSelUartIrda = No_IrDaMode;
	Uartcon.cLoopTest = No_LoopTest;
	Uartcon.cAfc = DisableAFCMode;
	Uartcon.cEnableFifo = DisableFIFO;
	Uartcon.cOpClock = UARTPCLK;
	Uartcon.cDataBit = DataLength8bit;
	Uartcon.cParityBit = NoParity;
	Uartcon.cStopBit = OneStopBit;
	Uartcon.cTxMode = TxMode_IRQ;
	Uartcon.cTxTrig = 1;
	Uartcon.cRxMode = RxMode_IRQ;
	Uartcon.cRxTrig = 1;
	Uartcon.cRtsTrig = 1;
	Uartcon.cSendBreakSignal = No_SendBreakSiganl;
	
	PJBoot64.uart_en = TRUE;
	debug_port = 1;

	Uart_Open(ch, Uartcon);
	UART_Putc('\n');
	return;
}

u8 UART_Getc(void)
{
	if(!PJBoot64.uart_en) return 0;
	return Uart_Getc(debug_port);
}

void UART_Putc(u8 data)
{
	if(!PJBoot64.uart_en) return;
	Uart_Putc(debug_port, data);
}

u8 UART_GetStatus(void)
{
	return Uart_GetKey(debug_port);
}

void UART_GetStr(s8 *pStr)
{
    u8 *pStrOrg;
    u8 c;

	if(!PJBoot64.uart_en) return;
    pStrOrg = (u8 *)pStr;
	
    while ((c= UART_Getc())!='\r')
    {
 		if (c=='\b') {
 		    if ((int)pStrOrg < (int)pStr) {
 				UART_PutStr("\b \b");
 				pStr--;
			}
 		}
 		else
 		{
			*pStr++ = c;
			UART_Putc(c);
 		}
    }
	
    *pStr = '\0';
	UART_Putc('\n');
}

void UART_PutStr(char *pt)
{
	if(!PJBoot64.uart_en) return;
    while(*pt) UART_Putc(*pt++);
}

static s32 Str2Int(s8 *string)
{
    s32 base     = 10;
    s32 minus    = 0;
    s32 result   = 0;
    s32 lastIndex;
    s32 i;

    if(string[0]=='-')
    {
        minus = 1;
        string++;
    }

    if(string[0]=='0' && (string[1]=='x' || string[1]=='X'))
    {
        base    = 16;
        string += 2;
    }

    lastIndex = strlen((const char *)string) - 1;

    if(lastIndex<0)
        return -1;

    if(string[lastIndex]=='h' || string[lastIndex]=='H' )
    {
        base = 16;
        string[lastIndex] = 0;
        lastIndex--;
    }

    if(base==10)
    {
        //result = atoi(string); // atoi conversion not working properly //trb 051011
		/* Alternate implementation for atoi */
        result = 0;
        for(i = 0; i<=lastIndex; i++){
			result = result * 10 + string[i] - 0x30;
       	}
        result = minus ? (-1*result):result;
    }
    else
    {
        for(i=0;i<=lastIndex;i++)
        {
            if(isalpha(string[i]))
            {
                if(isupper(string[i]))
                    result = (result<<4) + string[i] - 'A' + 10;
                else
                    result = (result<<4) + string[i] - 'a' + 10;
            }
            else
                result = (result<<4) + string[i] - '0';
        }
        result = minus ? (-1*result):result;
    }
    return result;
}

s32 UART_GetIntNum(void)
{
    u8 str[32];
	
    UART_GetStr((s8 *)str);
    return Str2Int((s8 *)str);
}

u8 UART_GetKey(void)
{
	return Uart_GetKey(debug_port);
}
