#include "common.h"
#include "intc.h"
#include "gpio_dev.h"
#include "otg_dev.h"
#include "usb_dep.h"

static u32 Download_Only(u8 *DownloadAddress)
{
	u32 uDownAddr, uDownFileSize, pDownPt;

	OTGDEV_ClearDownFileInfo();
	OTGDEV_SetOpMode(USB_DMA);
	OTGDEV_SetDownloadAddress((u32)DownloadAddress);
	
	MemoPad_Update("Select a file to download in DNW\n");
	do
	{
		OTGDEV_GetDownFileInfo(&uDownAddr, &uDownFileSize, &pDownPt);
		if(Response_Key())	return 0;
	}while((uDownAddr == 0) ||(uDownFileSize == 0));

	MemoPad_Update("\n[ADDRESS:%xh,TOTAL:%d(0x%x)]\n", uDownAddr, uDownFileSize, (uDownFileSize-10));

	do
	{
		OTGDEV_GetDownFileInfo(&uDownAddr, &uDownFileSize, &pDownPt);
		if(UART_GetKey() == 'x') return 0;
	}while((pDownPt - uDownAddr)<(uDownFileSize - 8));

	if(OTGDEV_VerifyChecksum()) return uDownFileSize - 10;
	else						return 0;
}

void Upload_Only(void)
{
	u32 uUpAddr, uUpFileSize, pUpPt;

	OTGDEV_ClearDownFileInfo();
	OTGDEV_ClearUpFileInfo();

	OTGDEV_SetOpMode(USB_DMA); 

	MemoPad_Update("\nConfigure the address and the size of the file to upload in DNW\n");

	do
	{
		OTGDEV_GetUpFileInfo(&uUpAddr, &uUpFileSize, &pUpPt);
		if(UART_GetKey() != 0) return;
	}while((uUpAddr == 0) || (uUpFileSize == 0));
	
	MemoPad_Update("\n[ADDRESS:%xh,TOTAL:%d]\n", uUpAddr, uUpFileSize);

	do
	{
		OTGDEV_GetUpFileInfo(&uUpAddr, &uUpFileSize, &pUpPt);
		if(UART_GetKey() == 'x') return;
	}while((pUpPt - uUpAddr)<uUpFileSize);

	MemoPad_Update("Success!!\n");
}

u32 Usb_Loader(u32 DownloadAddress, u8 option)
{
	u32 filesize;
	
	if(DownloadAddress < 0x50000000 && DownloadAddress >= 0x58000000) return;
	
	MemoPad_Update("Waiting for USB host connection.\n");

	rGPNPUD = rGPNPUD & ~(0x3<<12 | 0x3<<14);
	rGPNCON = rGPNCON & ~(0x3<<12 | 0x3<<14) | (1<<12);
	rGPNDAT = rGPNDAT | (1<<6);
	
	rGPQPUD = rGPQPUD & ~(0x3<<8);
	rGPQCON = rGPQCON & ~(0x3<<8);

	// USB  ..
	while(!(rGPNDAT & (1<<7)))
	{
		if(Response_Key()) 
		{
			MemoPad_Update("USB Loader Cancle\n");
			return 0;
		}
	}
	
	// Ŭ   ...
	Delay(10000);
	
	OTGDEV_InitOtg(USB_HIGH);
		
	while(!OTGDEV_IsUsbOtgSetConfiguration());
	if(option == UPLOAD_MODE)	Upload_Only();
	else						filesize = Download_Only((u8*)DownloadAddress);

	OTGDEV_SetSoftDisconnect();
	OTGDEV_ClearAllOutEpNak();

	INTC_Disable(NUM_OTG);
	
	rGPNCON = rGPNCON & ~(0x3<<12);
	rGPNDAT = rGPNDAT & ~(1<<6);
	rGPQCON = rGPQCON & ~(0x3<<8);

	return filesize;
}

void Usb_Loader_Init(void)
{


}