#ifndef __GPIO_DEV_H__
#define __GPIO_DEV_H__
#ifdef __cplusplus
extern "C" {
#endif

//#include "def.h"

#define GPIO_BASE						(0x7F008000)
#define rGPACON							(*(volatile unsigned *)(GPIO_BASE+0x00))
#define rGPBCON							(*(volatile unsigned *)(GPIO_BASE+0x20))
#define rGPNCON							(*(volatile unsigned *)(GPIO_BASE+0x830))
#define rGPNDAT							(*(volatile unsigned *)(GPIO_BASE+0x834))
#define rGPNPUD							(*(volatile unsigned *)(GPIO_BASE+0x838))
#define rGPQCON							(*(volatile unsigned *)(GPIO_BASE+0x180))
#define rGPQDAT							(*(volatile unsigned *)(GPIO_BASE+0x184))
#define rGPQPUD							(*(volatile unsigned *)(GPIO_BASE+0x188))

#define Outp32(addr, data)				(*(volatile u32 *)(addr) = (data))
#define Outp16(addr, data)				(*(volatile u16 *)(addr) = (data))
#define Outp8(addr, data)				(*(volatile u8 *)(addr) = (data))
#define Inp32(addr)						(*(volatile u32 *)(addr))
#define Inp16(addr)						(*(volatile u16 *)(addr))
#define Inp8(addr)						(*(volatile u8 *)(addr))

typedef enum GPIO_BitPos
{
	eGPIO_0		= (u32)(0),
	eGPIO_1		= (u32)(1),
	eGPIO_2		= (u32)(2),
	eGPIO_3		= (u32)(3),
	eGPIO_4		= (u32)(4),
	eGPIO_5		= (u32)(5),
	eGPIO_6		= (u32)(6),
	eGPIO_7		= (u32)(7),
	eGPIO_8		= (u32)(8),
	eGPIO_9		= (u32)(9),
	eGPIO_10	= (u32)(10),
	eGPIO_11	= (u32)(11),
	eGPIO_12	= (u32)(12),
	eGPIO_13	= (u32)(13),
	eGPIO_14	= (u32)(14),
	eGPIO_15	= (u32)(15)
}GPIO_eBitPos;

typedef enum GPIO_Id
{
	eGPIO_A = 0x4100,    		// muxing bit : 4, 	control register : 1,	offset : 0, 		
	eGPIO_B = 0x4102,    		// muxing bit : 4, 	control register : 1,	offset : 0x02*0x10
	eGPIO_C = 0x4104,    		// muxing bit : 4, 	control register : 1,	offset : 0x04*0x10
	eGPIO_D = 0x4106,    		// muxing bit : 4, 	control register : 1,	offset : 0x06*0x10
	eGPIO_E = 0x4108,    		// muxing bit : 4, 	control register : 1,	offset : 0x08*0x10
	eGPIO_F = 0x210A,    		// muxing bit : 2, 	control register : 1,	offset : 0x0A*0x10
	eGPIO_G = 0x410C,    		// muxing bit : 4, 	control register : 1,	offset : 0x0C*0x10
	eGPIO_H = 0x420E,    		// muxing bit : 4, 	control register : 2,	offset : 0x0E*0x10
	eGPIO_I = 0x2110,    		// muxing bit : 2, 	control register : 1,	offset : 0x10*0x10
	eGPIO_J = 0x2112,    		// muxing bit : 2, 	control register : 1,	offset : 0x12*0x10
	
	eGPIO_K = 0x4280,    		// muxing bit : 4, 	control register : 2,	offset : 0x80*0x10
	eGPIO_L = 0x4281,    		// muxing bit : 4, 	control register : 2,	offset : 0x81*0x10
	eGPIO_M = 0x4182,    		// muxing bit : 4, 	control register : 1,	offset : 0x82*0x10
	eGPIO_N = 0x2183,    		// muxing bit : 2, 	control register : 1,	offset : 0x83*0x10
	
	eGPIO_O = 0x2114,    		// muxing bit : 2, 	control register : 1,	offset : 0x14*0x10
	eGPIO_P = 0x2116,    		// muxing bit : 2, 	control register : 1,	offset : 0x16*0x10
	eGPIO_Q = 0x2118   		// muxing bit : 2, 	control register : 1,	offset : 0x18*0x10
}
GPIO_eId;

#define		Low_Level		0x0
#define		High_Level		0x1
#define		Falling_Edge	0x2
#define		Rising_Edge		0x4
#define		Both_Edge		0x6

typedef enum LCD_TYPE
{
	eHOSTIF	= 	0,
	eRGBIF		=	1,
	eYCbCr		=	2,
	eHidden		=	3
}LCD_eTYPE;

void GPIO_Init(void);
void GPIO_SetFunctionEach(GPIO_eId Id, GPIO_eBitPos eBitPos, u32 uFunction);
void GPIO_SetFunctionAll(GPIO_eId Id, u32 uValue0, u32 uValue1);
void GPIO_SetDataEach(GPIO_eId Id, GPIO_eBitPos eBitPos, u32 uValue);
void GPIO_SetDataAll(GPIO_eId Id, u32 uValue);
u32 GPIO_GetDataAll(GPIO_eId Id);
void GPIO_SetPullUpDownEach(GPIO_eId Id, GPIO_eBitPos eBitPos, u32 uValue);
void GPIO_SetPullUpDownAll(GPIO_eId Id, u32 uValue);
void GPIO_SetLCDType(LCD_eTYPE eLCDType);

#ifdef __cplusplus
}
#endif
#endif
