//#include "def.h"
#include "common.h"

#include "lcd_dep.h"
#include "library.h"
#include "lcd_dev.h"
#include "gpio_dev.h"
#include "library.h"
#include "main.h"

/////////////////////////////////////////////////////
// LCD SFR Write  
#define LCD_BASE				(0x77100000)
#define LcdcOutp32(offset, x)	Outp32(LCD_BASE+offset, x)
#define LcdcOutp16(offset, x)	Outp16(LCD_BASE+offset, x)
#define LcdcInp32(offset)		Inp32(LCD_BASE+offset)

LCDC oLcdc;

u32 uFrameCount;

//////////////////////////////////////////////////////////////////////////////////////
// LCD API Functions
//////////////////////////////////////////////////////////////////////////////////////

void LCD_DEV_SetPort (void)
{
	///////////////////////////////////////////////////////////////////////////////
	(*(volatile unsigned *)0x7410800c)=0;	//Must be '0' for Normal-path instead of By-pass
	///////////////////////////////////////////////////////////////////////////////
	
	//Select LCD IF Pin Configure
	GPIO_SetLCDType(eRGBIF);
	
	//Set LCD GPIO Port
	GPIO_SetFunctionAll(eGPIO_I, 0xaaaaaaaa, 2); //GPI[15..0]-> RGB VD[15..0]
	GPIO_SetFunctionAll(eGPIO_J, 0xaaaaaaaa, 2); //GPJ[7..0]-> RGB VD[23..16], GPJ[11..8]-> VCLK, VDEN, VSYNC, HSYNC

	//Set PWM GPIO to control Back-light  Regulator  Shotdown Pin
	GPIO_SetFunctionEach(eGPIO_F, eGPIO_15, 1); //GPF[15] -> Output

	//Set Display Signal
	GPIO_SetFunctionEach(eGPIO_N, eGPIO_9, 1);	//GPN[9] -> Output

	Delay(100);
}

void LCD_Dev_Init(u32 start_frame_buffer, u32 xsize, u32 ysize)
{
	u32 temp;
	u32 uClkVal;

	LCD_DEV_SetPort();
	
	oLcdc.m_uLcdHSz = xsize;
	oLcdc.m_uLcdVSz = ysize;
	
	oLcdc.m_uScanMode = 0; // progressive mode
	oLcdc.m_bIsLocalCalled[0] = false;
	oLcdc.m_bIsLocalCalled[1] = false;
	oLcdc.m_bIsLocalCalled[2] = false;

	if(PJBoot64.board_id == 4) 
	{
		LcdcOutp32(rVIDCON1, IVCLK_FALL_EDGE | IHSYNC_INVERT | IVSYNC_INVERT |IVDEN_NORMAL);
		LcdcOutp32(rVIDTCON0, VBPDE(VBPD_LTP700WV) | VBPD(VBPD_LTP700WV) | VFPD(VFPD_LTP700WV) | VSPW(VSPW_LTP700WV));
		LcdcOutp32(rVIDTCON1, VFPDE(VFPD_LTP700WV) | HBPD(HBPD_LTP700WV) | HFPD(HFPD_LTP700WV) | HSPW(HSPW_LTP700WV));

		uClkVal = (int)((((double)PJBoot64.hclk/(double)((oLcdc.m_uLcdHSz + HBPD_LTP700WV + HFPD_LTP700WV + HSPW_LTP700WV + 3) * 
				(oLcdc.m_uLcdVSz + VBPD_LTP700WV + VFPD_LTP700WV + VSPW_LTP700WV + 3) * LTP700WV_FRM_RATE) - 1) * 10 + 5)/10.0);

	}
	else 
	{
		LcdcOutp32(rVIDCON1, IVCLK_RISE_EDGE | IHSYNC_INVERT | IVSYNC_INVERT |IVDEN_NORMAL);
		LcdcOutp32(rVIDTCON0, VBPDE(VBPD_FL500WVR) | VBPD(VBPD_FL500WVR) | VFPD(VFPD_FL500WVR) | VSPW(VSPW_FL500WVR));
		LcdcOutp32(rVIDTCON1, VFPDE(VFPD_FL500WVR) | HBPD(HBPD_FL500WVR) | HFPD(HFPD_FL500WVR) | HSPW(HSPW_FL500WVR));

		uClkVal = (int)((((double)PJBoot64.hclk/(double)((oLcdc.m_uLcdHSz + HBPD_FL500WVR + HFPD_FL500WVR + HSPW_FL500WVR + 3) * 
				(oLcdc.m_uLcdVSz + VBPD_FL500WVR + VFPD_FL500WVR + VSPW_FL500WVR + 3) * FL500WVR_FRM_RATE) - 1) * 10 + 5)/10.0);

	}
	
	LcdcOutp32(rVIDTCON2, LINEVAL(oLcdc.m_uLcdVSz-1) | HOZVAL(oLcdc.m_uLcdHSz-1));
	LcdcOutp32(rDITHMODE, RDITHPOS_8BIT|GDITHPOS_8BIT|BDITHPOS_8BIT|DITHERING_DISABLE); // Fixed Dithering Matrix
	
	LCD_DEV_Stop();
	while ((LcdcInp32(rVIDCON0)&0x03));
	// checking whether disable the video output and the Display control signal or not.
		
	temp = PROGRESSIVE|VIDOUT_RGBIF|SUB_16_MODE|MAIN_16_MODE|PNRMODE_BGR_P|CLKVALUP_ALWAYS|CLKVAL_F(uClkVal - 1)|
				 VCLKEN_DISABLE | CLKDIR_F((uClkVal<1)?0:1) | CLKSEL_F_HCLK|	ENVID_DISABLE | ENVID_F_DISABLE;
	LcdcOutp32(rVIDCON0, temp);

	temp =LcdcInp32(rVIDINTCON0);
	LcdcOutp32(rVIDINTCON0, temp|FRAMESEL0_BACK|FRAMESEL1_NONE|INTFRMEN_DISABLE|FIFOSEL_WIN0|FIFOLEVEL_25|INTFIFOEN_DISABLE|INTEN_DISABLE);

	LCD_DEV_SetAllWinOnOff(0);				// Turn all windows off
	LCD_DEV_SetAllWinColorMapOnOff(0);		// Turn all windows color map off
	LCD_DEV_SetAllWinColorKeyOnOff(0);		// Turn all windows Color Key off

	LCD_DEV_SetAlpha(0x0, 0x0, 0x0, 0x0, 0x0, 0x0, WIN1);
	LCD_DEV_SetAlpha(0x0, 0x0, 0x0, 0x0, 0x0, 0x0, WIN2);
	LCD_DEV_SetAlpha(0x0, 0x0, 0x0, 0x0, 0x0, 0x0, WIN3);
	LCD_DEV_SetAlpha(0x0, 0x0, 0x0, 0x0, 0x0, 0x0, WIN4);
}

void LCD_DEV_Start(void)
{
	u32 uLcdCon;

	GPIO_SetDataEach(eGPIO_F, eGPIO_15 , 1);	// back_en
	GPIO_SetDataEach(eGPIO_N, eGPIO_9 , 1);		// dis_en

	uLcdCon=Inp32(LCD_BASE+rVIDCON0);
	uLcdCon |= 0x3<<0;
	LcdcOutp32(rVIDCON0, uLcdCon);
}


void LCD_DEV_Stop(void)
{
	u32 uLcdCon;

	GPIO_SetDataEach(eGPIO_F, eGPIO_15 , 0);	// back_en
	GPIO_SetDataEach(eGPIO_N, eGPIO_9 , 0);		// dis_en

	uLcdCon=Inp32(LCD_BASE+rVIDCON0);
	uLcdCon &= ~(0x3<<0);
	LcdcOutp32(rVIDCON0, uLcdCon);
}
 

///////////////////////////////////////////////////////////////////
// Middle Layer
///////////////////////////////////////////////////////////////////

void LCD_DEV_SetColorMapOnOff(u32 uOnOff, LCD_WINDOW eWin)
{

	u32 uWinMapReg, uWinMapVal;

	uWinMapReg =
		(eWin == WIN0) ? rWIN0MAP :
		(eWin == WIN1) ? rWIN1MAP :
		(eWin == WIN2) ? rWIN2MAP :
		(eWin == WIN3) ? rWIN3MAP : rWIN4MAP;

	uWinMapVal=Inp32(LCD_BASE+uWinMapReg);

	if (uOnOff)
		LcdcOutp32(uWinMapReg, COLOR_MAP_ENABLE|uWinMapVal);
	else
		LcdcOutp32(uWinMapReg, ~COLOR_MAP_ENABLE&uWinMapVal);
}

void LCD_DEV_SetColorMapValue(u32 uVal, LCD_WINDOW eWin)
{
	u32 uColVal = uVal&0x00ffffff;
	u32 uWinMapReg;

	uWinMapReg =
		(eWin == WIN0) ? rWIN0MAP :
		(eWin == WIN1) ? rWIN1MAP :
		(eWin == WIN2) ? rWIN2MAP :
		(eWin == WIN3) ? rWIN3MAP : rWIN4MAP;

	LcdcOutp32(uWinMapReg, COLOR_MAP_ENABLE|uColVal);
}

void LCD_DEV_Trigger(void)
{
	u32 uTrigCtrl;

	uTrigCtrl=Inp32(LCD_BASE+rTRIGCON);

	if (oLcdc.m_bIsAutoBuf == true)
		LcdcOutp32(rTRIGCON, uTrigCtrl | SW_TRIG_MODE | SW_TRIGGER);
}


///////////////////////////////////////////////////////////////////
// Bottom Layer
///////////////////////////////////////////////////////////////////
void LCD_DEV_InitWinRegs(LCD_WINDOW eWin)
{
	u32 uWinConXReg;
	u32 uAlphaValReg;
	u32 uFrmBufStAddrReg;
	u32 uFrmBufEndAddrReg;
	u32 uOffsetPageWidthReg;
	u32 uOsdLeftPosReg;
	u32 uOsdRightPosReg;
	u32 uWinConXRegVal;
	u32 uLcdEndX, uLcdEndY;
	
	uWinConXReg =
		(eWin == WIN0) ? rWINCON0 :
		(eWin == WIN1) ? rWINCON1 :
		(eWin == WIN2) ? rWINCON2 :
		(eWin == WIN3) ? rWINCON3 : rWINCON4;

	uAlphaValReg =
		(eWin == WIN1) ? rVIDOSD1C :
		(eWin == WIN2) ? rVIDOSD2C :
		(eWin == WIN3) ? rVIDOSD3C :
		(eWin == WIN4) ? rVIDOSD4C : rVIDOSD1C;

	uFrmBufStAddrReg =
		(eWin == WIN0) ? rVIDW00ADD0B0 :
		(eWin == WIN1) ? rVIDW01ADD0B0:
		(eWin == WIN2) ? rVIDW02ADD0 :
		(eWin == WIN3) ? rVIDW03ADD0 : rVIDW04ADD0;

	uFrmBufEndAddrReg =
		(eWin == WIN0) ? rVIDW00ADD1B0 :
		(eWin == WIN1) ? rVIDW01ADD1B0 :
		(eWin == WIN2) ? rVIDW02ADD1 :
		(eWin == WIN3) ? rVIDW03ADD1 : rVIDW04ADD1;

	uOffsetPageWidthReg =
		(eWin == WIN0) ? rVIDW00ADD2 :
		(eWin == WIN1) ? rVIDW01ADD2 :
		(eWin == WIN2) ? rVIDW02ADD2 :
		(eWin == WIN3) ? rVIDW03ADD2 : rVIDW04ADD2;

	uOsdLeftPosReg =
		(eWin == WIN0) ? rVIDOSD0A :
		(eWin == WIN1) ? rVIDOSD1A :
		(eWin == WIN2) ? rVIDOSD2A :
		(eWin == WIN3) ? rVIDOSD3A : rVIDOSD4A;

	uOsdRightPosReg =
		(eWin == WIN0) ? rVIDOSD0B :
		(eWin == WIN1) ? rVIDOSD1B :
		(eWin == WIN2) ? rVIDOSD2B :
		(eWin == WIN3) ? rVIDOSD3B : rVIDOSD4B;

	uWinConXRegVal = 0;
	
	if (eWin == WIN0)
	{
		if (!oLcdc.m_bIsLocalCalled[0]) // Input path is DMA
		{
			uWinConXRegVal =
				W0DMA | W0BUF0 | W0BUFAUTO_DISABLE | BITSWP_DISABLE |
				oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode|
				oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | oLcdc.m_uBppMode |
				ENWIN_F_DISABLE;
			
		}
		else // Input path is Local
		{
			Assert(oLcdc.m_eLocalIn[0] == IN_POST);

			if(oLcdc.m_uLocalInColorSpace[0] == LOCALIN_YCbCr)
			{
				uWinConXRegVal =
				W0LOCAL_POST | W0BUF0 | W0BUFAUTO_DISABLE | BITSWP_DISABLE |
				oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode| IN_LOCAL_YUV |
				oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | oLcdc.m_uBppMode |
				ENWIN_F_DISABLE;
			}
			else
			{
				uWinConXRegVal =
				W0LOCAL_POST | W0BUF0 | W0BUFAUTO_DISABLE | BITSWP_DISABLE |
				oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode| IN_LOCAL_RGB |
				oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | oLcdc.m_uBppMode |
				ENWIN_F_DISABLE;
			}
			
		}

		LcdcOutp32(rVIDW00ADD0B1, oLcdc.m_uDoubleBufStAddr[0]);
		LcdcOutp32(rVIDW00ADD1B1, oLcdc.m_uDoubleBufEndAddr[0]);
		LcdcOutp32(rVIDOSD0C, OSDSIZE(oLcdc.m_uViewHSz[0]*oLcdc.m_uViewVSz[0]));
	}
	else
	{
		if (eWin == WIN1)
		{
			if (!oLcdc.m_bIsLocalCalled[1]) // Input path is DMA
			{
				uWinConXRegVal =
					W0DMA | W0BUF0 | W0BUFAUTO_DISABLE | BITSWP_DISABLE |
					oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode|
					oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | BLD_PIX_PLANE | oLcdc.m_uBppMode |
					BLEND_ALPHA0_PLANE | ENWIN_F_DISABLE;
			}
			else // Input path is Local
			{
				Assert(oLcdc.m_eLocalIn[1] == IN_POST || oLcdc.m_eLocalIn[1] == IN_CIM);

				if(oLcdc.m_uLocalInColorSpace[1] == LOCALIN_YCbCr)
				{
					uWinConXRegVal =
					((oLcdc.m_eLocalIn[1] == IN_CIM) ? W1ENLOCAL_CIM : W1ENLOCAL_POST) |
					W1LOCAL| W0BUF0 | W0BUFAUTO_DISABLE | BITSWP_DISABLE |
					oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode| IN_LOCAL_YUV |
					oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | BLD_PIX_PLANE | oLcdc.m_uBppMode |
					BLEND_ALPHA0_PLANE | ENWIN_F_DISABLE;
				}
				else
				{
					uWinConXRegVal =
					((oLcdc.m_eLocalIn[1] == IN_CIM) ? W1ENLOCAL_CIM : W1ENLOCAL_POST) |
					W1LOCAL| W0BUF0 | W0BUFAUTO_DISABLE | BITSWP_DISABLE |
					oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode| IN_LOCAL_RGB |
					oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | BLD_PIX_PLANE | oLcdc.m_uBppMode |
					BLEND_ALPHA0_PLANE | ENWIN_F_DISABLE;
				}	
			}
			
			// set double buf. addr. and offset size
			LcdcOutp32(rVIDW01ADD0B1, oLcdc.m_uDoubleBufStAddr[1]);
			LcdcOutp32(rVIDW01ADD1B1, oLcdc.m_uDoubleBufEndAddr[1]);

			LcdcOutp32(rVIDOSD1D, OSDSIZE(oLcdc.m_uViewHSz[1]*oLcdc.m_uViewVSz[1]));			
		}
		else // eWin == WIN2 OR WIN3 OR WIN4
		{
			if (eWin == WIN2)
			{
				if (!oLcdc.m_bIsLocalCalled[2]) // Input path is DMA
				{
					uWinConXRegVal =
						W0DMA | BITSWP_DISABLE |
						oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode|
						oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | BLD_PIX_PLANE | oLcdc.m_uBppMode |
						BLEND_ALPHA0_PLANE | ENWIN_F_DISABLE;
				}
				else // Input path is Local
				{
					Assert(oLcdc.m_eLocalIn[2] == IN_POST || oLcdc.m_eLocalIn[2] == IN_CIM);

					if(oLcdc.m_uLocalInColorSpace[2] == LOCALIN_YCbCr)
					{
					
						uWinConXRegVal =
						((oLcdc.m_eLocalIn[2] == IN_CIM) ? W1ENLOCAL_CIM : W1ENLOCAL_POST) | 
						W1LOCAL | BITSWP_DISABLE |oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode| IN_LOCAL_YUV |
						oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | BLD_PIX_PLANE | oLcdc.m_uBppMode |
						BLEND_ALPHA0_PLANE | ENWIN_F_DISABLE;
					}
					else 
					{
						
						uWinConXRegVal =
						((oLcdc.m_eLocalIn[2] == IN_CIM) ? W1ENLOCAL_CIM : W1ENLOCAL_POST) | 
						W1LOCAL | BITSWP_DISABLE |oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode| IN_LOCAL_RGB |
						oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen | BLD_PIX_PLANE | oLcdc.m_uBppMode |
						BLEND_ALPHA0_PLANE | ENWIN_F_DISABLE;
					}
					
				}

				LcdcOutp32(rVIDOSD2D, OSDSIZE(oLcdc.m_uViewHSz[2]*oLcdc.m_uViewVSz[2]));				
			}
			else // eWin == WIN3 || eWin == WIN4
			{
				uWinConXRegVal = 
					BITSWP_DISABLE | oLcdc.m_uBytSwpMode | oLcdc.m_uHawSwpMode |
					oLcdc.m_uMinBurstLen | oLcdc.m_uMaxBurstLen |
					BLD_PIX_PLANE | oLcdc.m_uBppMode | BLEND_ALPHA0_PLANE | ENWIN_F_DISABLE;
			}
		}

		LcdcOutp32(uAlphaValReg,
			ALPHA0_R(0xF) | ALPHA0_G(0xF) | ALPHA0_B(0xF) |
			ALPHA1_R(0xF) | ALPHA1_G(0xF)| ALPHA1_B(0xF));
	}

	LcdcOutp32(uWinConXReg, uWinConXRegVal);		
	LcdcOutp32(uFrmBufStAddrReg, oLcdc.m_uFbStAddr[eWin]);
	LcdcOutp32(uFrmBufEndAddrReg, oLcdc.m_uFbEndAddr[eWin]);
	LcdcOutp32(uOffsetPageWidthReg, (oLcdc.m_uOffsetSz[eWin]<<13) | oLcdc.m_uPageWidth[eWin]);

	uLcdEndX = oLcdc.m_uLcdStX + oLcdc.m_uViewHSz[eWin] - 1;
	uLcdEndY = oLcdc.m_uLcdStY + oLcdc.m_uViewVSz[eWin] - 1;

	if (oLcdc.m_uScanMode == 1) // Interlace mode
	{
		Assert(!(oLcdc.m_uLcdStX%2));	// Left top -> Y coordinate must be even number in interlace mode
		Assert(uLcdEndY%2); 			// Rigth bottom -> Y coordinate must be odd number in interlace mode

		// In interlace mode, Left Top Y(oLcdc.m_uLcdStY) and Right Bottom Y(uLcdEndY) must be divided by 2.
		// And, Right Bottom Y must be rounded down
		LcdcOutp32(uOsdLeftPosReg, OSD_LTX_F(oLcdc.m_uLcdStX) | (OSD_LTY_F(oLcdc.m_uLcdStY)>>1));
		LcdcOutp32(uOsdRightPosReg, OSD_RBX_F(uLcdEndX) | (OSD_RBY_F(uLcdEndY)>>1));
	}
	else // Progressive mode
	{
		LcdcOutp32(uOsdLeftPosReg, OSD_LTX_F(oLcdc.m_uLcdStX) | OSD_LTY_F(oLcdc.m_uLcdStY));
		LcdcOutp32(uOsdRightPosReg, OSD_RBX_F(uLcdEndX) | OSD_RBY_F(uLcdEndY));
	}
}

void LCD_DEV_SetWinOnOff(u32 uOnOff, LCD_WINDOW eWin)
{
	u32 uWinTemp;
	u32 uWinConReg;

	uWinConReg =
		(eWin == WIN0) ? rWINCON0 :
		(eWin == WIN1) ? rWINCON1 :
		(eWin == WIN2) ? rWINCON2 :
		(eWin == WIN3) ? rWINCON3 : rWINCON4;

	uWinTemp=Inp32(LCD_BASE+uWinConReg);

	if (uOnOff) // Turn OSD on
		uWinTemp |= ENWIN_F_ENABLE;
	else // Turn OSD off
		uWinTemp &= ~ENWIN_F_ENABLE;

	LcdcOutp32(uWinConReg, uWinTemp);
}

void LCD_DEV_SetAllWinOnOff(u32 uOnOff)
{
	u32 uWinTemp[5];

	uWinTemp[0]=Inp32(LCD_BASE+rWINCON0);
	uWinTemp[1]=Inp32(LCD_BASE+rWINCON1);
	uWinTemp[2]=Inp32(LCD_BASE+rWINCON2);
	uWinTemp[3]=Inp32(LCD_BASE+rWINCON3);
	uWinTemp[4]=Inp32(LCD_BASE+rWINCON4);

	if (uOnOff)
	{
		LcdcOutp32(rWINCON0, uWinTemp[0]|ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON1, uWinTemp[1]|ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON2, uWinTemp[2]|ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON3, uWinTemp[3]|ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON4, uWinTemp[4]|ENWIN_F_ENABLE);
	}
	else
	{
		LcdcOutp32(rWINCON0, uWinTemp[0]&~ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON1, uWinTemp[1]&~ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON2, uWinTemp[2]&~ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON3, uWinTemp[3]&~ENWIN_F_ENABLE);
		LcdcOutp32(rWINCON4, uWinTemp[4]&~ENWIN_F_ENABLE);
	}
}

void LCD_DEV_SetAllWinColorMapOnOff(u32 uOnOff)
{
	u32 uColorMapVal;
	u32 uColorMapReg;
	int i;

	for (i=0; i<5; i++)
	{
		uColorMapReg =
			(i == 0) ? rWIN0MAP :
			(i == 1) ? rWIN1MAP :
			(i == 2) ? rWIN2MAP :
			(i == 3) ? rWIN3MAP : rWIN4MAP;

		uColorMapVal=Inp32(LCD_BASE+uColorMapReg);

		if (uOnOff)
			LcdcOutp32(uColorMapReg, COLOR_MAP_ENABLE|uColorMapVal);
		else
			LcdcOutp32(uColorMapReg, ~COLOR_MAP_ENABLE&uColorMapVal);
	}

}

 void LCD_DEV_SetAllWinColorKeyOnOff(u32 uOnOff)
{
	u32 uColorKeyReg, uColorKeyVal;
	int i;

	for (i=1; i<5; i++)
	{
		uColorKeyReg = (i == 1) ? rW1KEYCON0 : (i == 2) ? rW2KEYCON0 : (i == 3) ? rW3KEYCON0 : rW4KEYCON0;
		uColorKeyVal = Inp32(LCD_BASE+uColorKeyReg);

		if (uOnOff) LcdcOutp32(uColorKeyReg, uColorKeyVal | KEYEN_F_ENABLE);
		else		LcdcOutp32(uColorKeyReg, uColorKeyVal & KEYBLEN_DISABLE & KEYEN_F_DISABLE);
	}
}

void LCD_DEV_SetAlpha(u8 ucA0R, u8 ucA0G, u8 ucA0B, u8 ucA1R, u8 ucA1G, u8 ucA1B, LCD_WINDOW eWin)
{
	u8 ucA0RVal = ucA0R&0xf;
	u8 ucA0GVal = ucA0G&0xf;
	u8 ucA0BVal = ucA0B&0xf;
	u8 ucA1RVal = ucA1R&0xf;
	u8 ucA1GVal = ucA1G&0xf;
	u8 ucA1BVal = ucA1B&0xf;
	u32 uAlphaReg;

	// Can't support Alpha value to window 0
	Assert(eWin != WIN0);

	uAlphaReg = (eWin == WIN1) ? rVIDOSD1C : (eWin == WIN2) ? rVIDOSD2C : (eWin == WIN3) ? rVIDOSD3C : rVIDOSD4C;
	LcdcOutp32(uAlphaReg, ALPHA0_R(ucA0RVal) | ALPHA0_G(ucA0GVal) | ALPHA0_B(ucA0BVal) |	// Non-Key Area
						ALPHA1_R(ucA1RVal) | ALPHA1_G(ucA1GVal) | ALPHA1_B(ucA1BVal));		// Key Area
}

void LCD_DEV_SetColorKeyOnOff(u32 uOnOff, LCD_WINDOW eWin)
{
	u32 uColorKeyReg, uColorKeyVal;
	
	Assert(eWin != WIN0);

	uColorKeyReg = (eWin == WIN1) ? rW1KEYCON0 : (eWin == WIN2) ? rW2KEYCON0 : (eWin == WIN3) ? rW3KEYCON0 : rW4KEYCON0;
	uColorKeyVal = LcdcInp32(uColorKeyReg);

	if (uOnOff) LcdcOutp32(uColorKeyReg, uColorKeyVal | KEYEN_F_ENABLE);
	else		LcdcOutp32(uColorKeyReg, uColorKeyVal & KEYBLEN_DISABLE & KEYEN_F_DISABLE);
}

void LCD_DEV_SetColorKey(u32 uMaskKey, u32 uColorKey, COLOR_KEY_DIRECTION eDir, u8 bIsBlending, LCD_WINDOW eWin)
{
	u32 uColorKeyReg_0, uColorKeyReg_1;
	u32 uColorKeyVal_0;

	u32 uBlending = KEYBLEN_DISABLE;
	u32 uDir;

	Assert(eWin != WIN0);

	if (bIsBlending == true) // To use Key blending, BLD_PIX must be set 1 and ALPHA_SEL be set 0
	{
		u32 uWinConReg, uWinConVal;
		uBlending = KEYBLEN_ENABLE;
		
		uWinConReg =
			(eWin == WIN1) ? rWINCON1 :
			(eWin == WIN2) ? rWINCON2 :
			(eWin == WIN3) ? rWINCON3 : rWINCON4;

		uWinConVal = Inp32(LCD_BASE+uWinConReg);
		// Set BLD_PIX is set 1 and ALPHA_SEL is set 0
		LcdcOutp32(uWinConReg, uWinConVal | BLD_PIX_PIXEL & ALPHA_SEL_0);
	}

	uColorKeyReg_0 =
		(eWin == WIN1) ? rW1KEYCON0 :
		(eWin == WIN2) ? rW2KEYCON0 :
		(eWin == WIN3) ? rW3KEYCON0 : rW4KEYCON0;

	uColorKeyReg_1 =
		(eWin == WIN1) ? rW1KEYCON1 :
		(eWin == WIN2) ? rW2KEYCON1 :
		(eWin == WIN3) ? rW3KEYCON1 : rW4KEYCON1;

	uDir = (eDir == MATCH_FG_IMAGE) ? DIRCON_MATCH_FG_IMAGE : DIRCON_MATCH_BG_IMAGE;

	uColorKeyVal_0 = Inp32(LCD_BASE+uColorKeyReg_0);

	uColorKeyVal_0 &= ~(1<<26);
	uColorKeyVal_0 |= uBlending;

	uColorKeyVal_0 &= ~(1<<24);
	uColorKeyVal_0 |= uDir;

	uColorKeyVal_0 |= ( (uMaskKey&0xffffff)<<0 ) ;
	
	LcdcOutp32(uColorKeyReg_0, uColorKeyVal_0);
	LcdcOutp32(uColorKeyReg_1, uColorKey);
	
}

void LCD_DEV_SetBlendingType(BLENDING_APPLIED_UNIT eBldPix, BLENDING_SELECT_MODE eBldMode, LCD_WINDOW eWin)
{
	u32 uWinConReg, uWinConVal;
	u32 uBlendUnit, uBlendSel;

	Assert(eWin != WIN0); // Blending feature supports window 1 ~ 4

	uWinConReg =
		(eWin == WIN1) ? rWINCON1 :
		(eWin == WIN2) ? rWINCON2 :
		(eWin == WIN3) ? rWINCON3 : rWINCON4;

	uBlendUnit = (eBldPix == PER_PLANE) ? BLD_PIX_PLANE : BLD_PIX_PIXEL;

	uBlendSel =
		(eBldMode == ALPHA0_PLANE) ? BLEND_ALPHA0_PLANE :
		(eBldMode == ALPHA1_PLANE) ? BLEND_ALPHA1_PLANE :
		(eBldMode == DATA_PIXEL) ? BLEND_DATA_PIXEL :
		(eBldMode == AEN_PIXEL) ? BLEND_AEN_PIXEL : BLEND_COLORKEY_AREA;

	uWinConVal = Inp32(LCD_BASE+uWinConReg);
	LcdcOutp32(uWinConReg, uWinConVal & ~BLD_PIX_MASK|uBlendUnit & ~ BLEND_SEL_MASK | uBlendSel);
}

void LCD_InitWin(CSPACE eBpp, u32 uFrameH, u32 uFrameV, u32 uX_Frame, u32 uY_Frame, u32 uViewH, u32 uViewV,
				u32 uX_Lcd, u32 uY_Lcd, u32 uFbAddr, LCD_WINDOW eWin, u8 bIsDoubleBuf)
{
	u32 uOffset;
	u32 uBurstSize;

	if ( (eWin != WIN0 && eWin != WIN1) && bIsDoubleBuf == true )
		Assert(0); // In WIN2, WIN3 or WIN4, Double buffering can't be supported

	// Viewport size must be less than LCD size
	Assert(uViewH <= oLcdc.m_uLcdHSz);
	Assert(uViewV <= oLcdc.m_uLcdVSz);

	// Check (X,Y) coordinate is valid in LCD and Frame?
	Assert( ((uX_Frame + uViewH) <= uFrameH) && ((uY_Frame + uViewV) <= uFrameV) );
	Assert( (uX_Lcd + uViewH <= oLcdc.m_uLcdHSz) && (uY_Lcd + uViewV <= oLcdc.m_uLcdVSz) );

	// Double buffering is supported by only window 0 and window 1
	if ( ((eWin != WIN0) && (eWin != WIN1)) && (bIsDoubleBuf == true) )
		Assert(0);

	oLcdc.m_uFrmHSz[eWin] = uFrameH;
	oLcdc.m_uFrmVSz[eWin] = uFrameV;

	oLcdc.m_uViewHSz[eWin] = uViewH;
	oLcdc.m_uViewVSz[eWin] = uViewV;

	oLcdc.m_uLcdStX = uX_Lcd;
	oLcdc.m_uLcdStY = uY_Lcd;

	uOffset = oLcdc.m_uFrmHSz[eWin] - oLcdc.m_uViewHSz[eWin];

	oLcdc.m_uBytSwpMode = BYTSWP_DISABLE;		// BYTE swap disable
	oLcdc.m_uHawSwpMode = HAWSWP_DISABLE;		// Half-Word swap disable

	oLcdc.m_uMaxBurstLen = MAX_BURSTLEN_16WORD;		// DMA'burst max 16word burst
	oLcdc.m_uMinBurstLen = MIN_BURSTLEN_16WORD;		// DMA'burst min 16 word burst


	//Setting BPP, HW Swap, and Byte Swap
	if (eBpp == PAL1)
	{
		oLcdc.m_uBytes = 1;
		oLcdc.m_uBits = 4;
		Assert( !(uOffset%32) ); // Must keep to word-alignment
		oLcdc.m_uBppMode = BPPMODE_F_1BPP;

		oLcdc.m_uMaxBurstLen = MAX_BURSTLEN_4WORD;
		oLcdc.m_uMinBurstLen = MIN_BURSTLEN_16WORD;
	}
	else if (eBpp == PAL2)
	{
		oLcdc.m_uBytes = 1;
		oLcdc.m_uBits = 4;
		Assert( !(uOffset%16) ); // Must keep to word-alignment
		oLcdc.m_uBppMode = BPPMODE_F_2BPP;

		oLcdc.m_uMaxBurstLen = MAX_BURSTLEN_4WORD;
		oLcdc.m_uMinBurstLen = MIN_BURSTLEN_16WORD;
	}
	else if ( (eBpp == PAL4) && (eWin != WIN4) )
	{
		oLcdc.m_uBytes = 1;
		oLcdc.m_uBits = 2;

		Assert( !(uOffset%8) ); // Must keep to word-alignment
		oLcdc.m_uBppMode = BPPMODE_F_4BPP;

		oLcdc.m_uMaxBurstLen = MAX_BURSTLEN_4WORD;
		oLcdc.m_uMinBurstLen = MIN_BURSTLEN_16WORD;
	}
	else if ( (eBpp == PAL8) && ((eWin == WIN0) || (eWin == WIN1)) )
	{
		oLcdc.m_uBytes = 1;
		oLcdc.m_uBits = 1;
		Assert( !(uOffset%4) ); // Must keep to word-alignment
		oLcdc.m_uBppMode = BPPMODE_F_8BPP_PAL;
		oLcdc.m_uBytSwpMode = BYTSWP_ENABLE;
	}
	else if ( (eBpp == RGB8) && ((eWin == WIN1) || (eWin == WIN2)) )
	{
		oLcdc.m_uBytes = 1;
		oLcdc.m_uBits = 1;
		Assert( !(uOffset%4) ); // Must keep to word-alignment
		oLcdc.m_uBppMode = BPPMODE_F_8BPP_NOPAL;

		oLcdc.m_uBytSwpMode = BYTSWP_ENABLE;
	}
	else if (eBpp == RGB16)
	{
		oLcdc.m_uBytes = 2;
		oLcdc.m_uBits = 1;
		Assert( !(uOffset%2) ); // Must keep to word-alignment

		oLcdc.m_uBppMode = BPPMODE_F_16BPP_565;
		oLcdc.m_uHawSwpMode = HAWSWP_ENABLE;
	}
	else if (eBpp == ARGB16)
	{
		oLcdc.m_uBytes = 2;
		oLcdc.m_uBits = 1;
		Assert( !(uOffset%2) ); // Must keep to word-alignment

		oLcdc.m_uBppMode = BPPMODE_F_16BPP_A555;
		oLcdc.m_uHawSwpMode = HAWSWP_ENABLE;
	}
	else if (eBpp == RGB18)
	{
		oLcdc.m_uBytes = 4;
		oLcdc.m_uBits = 1;

		oLcdc.m_uBppMode = BPPMODE_F_18BPP_666;
	}
	else if (eBpp == RGB24)
	{
		oLcdc.m_uBytes = 4;
		oLcdc.m_uBits = 1;

		oLcdc.m_uBppMode = BPPMODE_F_24BPP_888;
	}
	else if (eBpp == ARGB24)
	{
		oLcdc.m_uBytes = 4;
		oLcdc.m_uBits = 1;

		oLcdc.m_uBppMode = BPPMODE_F_24BPP_A887;
	}
	else
	{
		Assert(0);
	}

	// Get offsetsize, pagewidth and lineVal
	if (oLcdc.m_uFrmHSz[eWin] >= oLcdc.m_uViewHSz[eWin])
	{
		oLcdc.m_uOffsetSz[eWin] = uOffset*oLcdc.m_uBytes/oLcdc.m_uBits;
		oLcdc.m_uPageWidth[eWin] = oLcdc.m_uViewHSz[eWin]*oLcdc.m_uBytes/oLcdc.m_uBits;

		// Offset size must be more than the burst size
		uBurstSize =
			(oLcdc.m_uMaxBurstLen == MAX_BURSTLEN_16WORD) ? 16*4 :
			(oLcdc.m_uMaxBurstLen == MAX_BURSTLEN_8WORD) ? 8*4 : 4*4;

		// Offset size must be more than the burst size
		Assert( oLcdc.m_uOffsetSz[eWin] == 0 || !(oLcdc.m_uOffsetSz[eWin]%4) );
		Assert(oLcdc.m_uPageWidth[eWin] > uBurstSize); // Page width must be more than burst size and be word-aligned

		//uLineVal = oLcdc.m_uViewVSz[eWin] - 1;
	} 
	else
	{
		Assert(0);
	}

	oLcdc.m_uImgStAddr[eWin] = uFbAddr;

	oLcdc.m_uFbStAddr[eWin] = uFbAddr + (oLcdc.m_uFrmHSz[eWin]*uY_Frame + uX_Frame)*oLcdc.m_uBytes/oLcdc.m_uBits;
	oLcdc.m_uFbEndAddr[eWin] = oLcdc.m_uFbStAddr[eWin] + (oLcdc.m_uOffsetSz[eWin]+oLcdc.m_uPageWidth[eWin])*(uFrameV);

	if ( (bIsDoubleBuf == true) && ((eWin == WIN0) || (eWin == WIN1)) )
	{
		oLcdc.m_uDoubleBufImgStAddr[eWin] = oLcdc.m_uImgStAddr[eWin] + oLcdc.m_uFrmHSz[eWin]*oLcdc.m_uFrmVSz[eWin]*oLcdc.m_uBytes/oLcdc.m_uBits;
		oLcdc.m_uDoubleBufStAddr[eWin] = oLcdc.m_uFbStAddr[eWin] + oLcdc.m_uFrmHSz[eWin]*oLcdc.m_uFrmVSz[eWin]*oLcdc.m_uBytes/oLcdc.m_uBits;
		oLcdc.m_uDoubleBufEndAddr[eWin] = oLcdc.m_uDoubleBufStAddr[eWin] + (oLcdc.m_uOffsetSz[eWin]+oLcdc.m_uPageWidth[eWin])*(uFrameV);
	}
	else if ( (bIsDoubleBuf == false) && ((eWin == WIN0) || (eWin == WIN1)) )
	{
		oLcdc.m_uDoubleBufStAddr[eWin] = oLcdc.m_uFbStAddr[eWin];
		oLcdc.m_uDoubleBufEndAddr[eWin] = oLcdc.m_uFbEndAddr[eWin];
	}

	LCD_DEV_InitWinRegs(eWin);

	// For back-ward compatibility LCDC V2.0
	if(eWin == WIN0)
		LCD_DEV_SetWinOnOff(1, WIN0); // first arguement == 1 -> turn on window
}

void Switch_Win0_Win1(void)
{
	u32 temp;
	
	LCD_DEV_SetWinOnOff(0, WIN1);
	LcdcOutp32(rVIDW01ADD0B0, oLcdc.m_uFbStAddr[WIN0]);
	LcdcOutp32(rVIDW01ADD1B0, oLcdc.m_uFbEndAddr[WIN0]);
	LCD_DEV_SetWinOnOff(1, WIN1);

	LCD_Blend_ColorKeyOn(0x0, 0x0, 0x0, 0, 0, 0, NO_MASKING_COLOR, WIN1);
	
	LCD_DEV_SetWinOnOff(0, WIN0);
	LcdcOutp32(rVIDW00ADD0B0, oLcdc.m_uFbStAddr[WIN1]);
	LcdcOutp32(rVIDW00ADD1B0, oLcdc.m_uFbEndAddr[WIN1]);
	LCD_DEV_SetWinOnOff(1, WIN0);

	LCD_Blend_ColorKeyOn(0xf, 0xf, 0xf, 0, 0, 0, NO_MASKING_COLOR, WIN1);
	
	temp = oLcdc.m_uFbStAddr[WIN0];
	oLcdc.m_uFbStAddr[WIN0] = oLcdc.m_uFbStAddr[WIN1];
	oLcdc.m_uFbStAddr[WIN1] = temp;
	
	temp = oLcdc.m_uFbEndAddr[WIN0];
	oLcdc.m_uFbEndAddr[WIN0] = oLcdc.m_uFbEndAddr[WIN1];
	oLcdc.m_uFbEndAddr[WIN1] = temp;
}
