//#include "def.h"
#include "common.h"

#include "intc.h"
#include "gpio_dev.h"
#include "dma.h"
#include "main.h"
#include "nand_dep.h"
#include "nand_dev.h"

/******************************************/
/*          Function Define               */
/******************************************/
#define NF_MECC_UnLock(Controller)         	{NAND(Controller)->rNFCONT&=~(1<<7);}
#define NF_MECC_Lock(Controller)         	{NAND(Controller)->rNFCONT|=(1<<7);}
#define NF_SECC_UnLock(Controller)         	{NAND(Controller)->rNFCONT&=~(1<<6);}
#define NF_SECC_Lock(Controller)         	{NAND(Controller)->rNFCONT|=(1<<6);}
#define NF_CMD(Controller, cmd)				{NAND(Controller)->rNFCMMD=cmd;}
#define NF_ADDR(Controller, addr)			{NAND(Controller)->rNFADDR=addr;}	
#define NF_nFCE_L(Controller)				{NAND(Controller)->rNFCONT&=~(1<<1);}
#define NF_nFCE_H(Controller)				{NAND(Controller)->rNFCONT|=(1<<1);}
#define NF_RSTECC(Controller)				{NAND(Controller)->rNFCONT|=(1<<5)|(1<<4);}
#define NF_RDDATA(Controller) 				(NAND(Controller)->rNFDATA)
#define NF_RDDATA8(Controller) 				((u8)(*(u8*)(&NAND(Controller)->rNFDATA)) )
#define NF_RDDATA16(Controller) 			((u16)(*(u16*)(&NAND(Controller)->rNFDATA)) )
#define NF_WRDATA(Controller, data) 		{NAND(Controller)->rNFDATA=data;}
#define NF_WRDATA8(Controller, data) 		{*(u8*)(&NAND(Controller)->rNFDATA)=data;}

// RnB Signal
#define NF_CLEAR_RnB(Controller)    		{NAND(Controller)->rNFSTAT |= (1<<4);}	// Have write '1' to clear this bit.
#define NF_DETECT_RnB(Controller)    		{while(!(NAND(Controller)->rNFSTAT&(1<<4)));}

#define NAND(__n) 					( ( volatile oNAND_REGS * ) ( NAND_pBase[__n] ) )

typedef struct tag_NAND_REGS
{
	u32 rNFCONF;
	u32 rNFCONT;
	u32 rNFCMMD;
	u32 rNFADDR;
	u32 rNFDATA;
	u32 rNFMECCD0;
	u32 rNFMECCD1;
	u32 rNFSECCD;
	u32 rNFSBLK;
	u32 rNFEBLK;
	u32 rNFSTAT;
	u32 rNFECCERR0;
	u32 rNFECCERR1;
	u32 rNFMECC0;
	u32 rNFMECC1;
	u32 rNFSECC;
	u32 rNFMLCBITPT;
	u32 rNF8ECCERR0;
	u32 rNF8ECCERR1;
	u32 rNF8ECCERR2;
	u32 rNFM8ECC0;
	u32 rNFM8ECC1;
	u32 rNFM8ECC2;
	u32 rNFM8ECC3;
	u32 rNFMLC8BITPT0;
	u32 rNFMLC8BITPT1;
} 
oNAND_REGS;

#if (NAND_TRANSFER_MODE == DMA_TRANSFER)
static volatile u32 Nand_DmaDone;
DMAC g_oNandDmac1;
#endif

static void *NAND_pBase[NAND_CONNUM] = 
{
	(void *)(0x70200000)
};
static volatile u32 g_Nand_RnBTransition, g_Nand_4bitEccEncDone, g_Nand_4bitEccDecDone, g_Nand_IllegalAccError;
static u8 current_interrupt_id;
static u8 aNand_Spare_Data[NAND_SPARE_MAX] = 
{	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
};

static void __attribute__ ((interrupt("IRQ"))) Nand_ISR0(void);
static void __attribute__ ((interrupt("IRQ"))) Nand_DmaISR(void);
static void Nand_GetDurationValue(u32 Controller, u32 *uTacls, u32 *uTwrph0, u32 *uTwrph1);
static void Nand_Reset(u32 Controller);
static NAND_eERROR Nand_CheckECCError(u32 Controller);
static NAND_eERROR Nand_ReadPageSLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *pSpareBuffer);
static NAND_eERROR Nand_ReadPageMLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *pSpareBuffer);
static NAND_eERROR Nand_WritePageSLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *aSpareBuffer);
static NAND_eERROR Nand_WritePageMLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *aSpareBuffer);
static u32 Nand_ReadNFCONRegister(u32 Controller);

static void __attribute__ ((interrupt("IRQ"))) Nand_ISR0(void)
{
	u32 uIntSource;

	INTC_Disable(NUM_NFC);

	uIntSource = Inp32(&NAND(current_interrupt_id)->rNFSTAT);
	if(uIntSource & NAND_4BITECC_ENC)
	{
		Outp32(&NAND(current_interrupt_id)->rNFSTAT, NAND_4BITECC_ENC);
		g_Nand_4bitEccEncDone = 1;
	}
	else if(uIntSource & NAND_4BITECC_DEC)
	{
		Outp32(&NAND(current_interrupt_id)->rNFSTAT, NAND_4BITECC_DEC);
		g_Nand_4bitEccDecDone = 1;
	}
	else if(uIntSource & NAND_ILLEGAL_ACCESS)
	{
		Outp32(&NAND(current_interrupt_id)->rNFSTAT, NAND_ILLEGAL_ACCESS);
		g_Nand_IllegalAccError = 1;
	}
	else if(uIntSource & NAND_RnB_TRANS)
	{
		Outp32(&NAND(current_interrupt_id)->rNFSTAT, NAND_RnB_TRANS);
		g_Nand_RnBTransition = 1;
	}
	
	INTC_Enable(NUM_NFC);
	INTC_ClearVectAddr();
}

#if (NAND_TRANSFER_MODE == DMA_TRANSFER)
static void __attribute__ ((interrupt("IRQ"))) Nand_DmaISR(void) // NonVIC
{
  	DMACH_ClearIntPending(&g_oNandDmac1);
	Nand_DmaDone = 1;
	INTC_ClearVectAddr();
}
#endif

static u32 Nand_ReadNFCONRegister(u32 Controller)
{
	u32 uTemp, uErrorBit;
	
	uTemp = Inp32(&(NAND(Controller)->rNFCONT));
	uErrorBit = (uTemp & (3<<13)) >> 1;
	uTemp &= ~(3<<13);

	return (uTemp | uErrorBit);
}

static void Nand_GetDurationValue(u32 Controller, u32 *uTacls, u32 *uTwrph0, u32 *uTwrph1)
{
	u32 uTemp;
	float fTemp;

	uTemp = (NAND_Inform[Controller].uTacls * (PJBoot64.hclk/1000))/1000000;
	fTemp = (((float)NAND_Inform[Controller].uTacls * (float)(PJBoot64.hclk/1000))/(float)1000000);
	fTemp -= uTemp;

	if(fTemp != (float)0)	*uTacls = uTemp+1;
	else					*uTacls = uTemp;

	uTemp = (NAND_Inform[Controller].uTwrph0 * (PJBoot64.hclk/1000))/1000000;
	fTemp = (((float)NAND_Inform[Controller].uTwrph0 * (float)(PJBoot64.hclk/1000))/(float)1000000);
	fTemp -= uTemp;

	if(fTemp != (float)0)	*uTwrph0 = uTemp;
	else
	{
		if(uTemp > 0)		*uTwrph0 = uTemp-1;
		else				*uTwrph0 = 0;
	}
	
	uTemp = (NAND_Inform[Controller].uTwrph1 * (PJBoot64.hclk/1000))/1000000;
	fTemp = (((float)NAND_Inform[Controller].uTwrph1 * (float)(PJBoot64.hclk/1000))/(float)1000000);
	fTemp -= uTemp;

	if(fTemp != (float)0)	*uTwrph1 = uTemp;
	else
	{
		if(uTemp > 0)		*uTwrph1 = uTemp-1;
		else				*uTwrph1 = 0;
	}
}

static void Nand_Reset(u32 Controller)
{
	u32 i;
   
	NF_nFCE_L(Controller);

	NF_CLEAR_RnB(Controller);
	NF_CMD(Controller, 0xFF);	//reset command

	if(NAND_Inform[Controller].uNandType == NAND_Normal8bit)	for(i=0;i<1000;i++);
	else
	{
		for(i=0;i<10;i++);  //tWB = 100ns. //??????
		NF_DETECT_RnB(Controller);
	}
	NF_nFCE_H(Controller);
}

static NAND_eERROR Nand_CheckECCError(u32 Controller)
{
	u32 uEccError0;
	NAND_eERROR eError;	

	eError = eNAND_NoError;

	if((NAND_Inform[Controller].uNandType == NAND_Normal8bit) || (NAND_Inform[Controller].uNandType == NAND_Advanced8bit) )
	{
		uEccError0 = Inp32(&NAND(Controller)->rNFECCERR0);
		switch (uEccError0 & 0x03)
		{
			case 0x00 : eError = eNAND_NoError;			break;
			case 0x01 : eError = eNAND_1bitEccError;	break;
			case 0x02 : eError = eNAND_MultiError;		break;
			case 0x03 : eError = eNAND_EccAreaError;	break;
		}

		switch ((uEccError0 & 0x0C)>>2)
		{
			case 0x00 : eError |= eNAND_NoError;			break;
			case 0x01 : eError |= eNAND_Spare1bitEccError;	break;
			case 0x02 : eError |= eNAND_SpareMultiError;	break;
			case 0x03 : eError |= eNAND_SpareEccAreaError;	break;
		}	
	}
	else if(NAND_Inform[Controller].uNandType == NAND_MLC8bit)
	{	
		uEccError0 = Inp32(&NAND(Controller)->rNFECCERR0);
		switch ((uEccError0>>26) & 0x07)
		{
			case 0x00 : eError = eNAND_NoError;				break;
			case 0x01 : eError = eNAND_1bitEccError;		break;
			case 0x02 : eError = eNAND_2bitEccError;		break;
			case 0x03 : eError = eNAND_3bitEccError;		break;
			case 0x04 : eError = eNAND_4bitEccError;		break;
			case 0x05 : eError = eNAND_UncorrectableError;	break;
		}
	}

	return eError;
}

static NAND_eERROR Nand_ReadPageSLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *pSpareBuffer)
{
	u32 i, uBlockPage, uPageSize;
	u32 uMecc, uSecc;
	u32 *pBufferW;
	u32 uNFCONFreg, uNFCONTreg;
			
	pBufferW = (u32 *)pBuffer;			
	g_Nand_RnBTransition = 0;

	uNFCONFreg = Inp32(&(NAND(Controller)->rNFCONF)) & (~(0x1<<30)) & (~(0x3<<23)) | ((7<<12)|(7<<8)|(7<<4));
	// HCLK is more than 66Mhz., 1bit ECC Select, Timing 7,7,7 setting.
	Outp32(&(NAND(Controller)->rNFCONF), uNFCONFreg );	// NFCONF Register setting. 

	uNFCONTreg = Inp32(&(NAND(Controller)->rNFCONT)) & ~((1<<13)|(1<<12)) | (1<<10)|(1<<9);
	// 4bit ECC encoding/decoding completion Interrupt disable, Illegal Access & RnB Transition Interrupt enable
	Outp32(&(NAND(Controller)->rNFCONT), uNFCONTreg);	// write setting. NFCONT Register
	Outp32(&(NAND(Controller)->rNFSTAT), (1<<5)|(1<<4));// Illegal Access & RnB Interrupt Pending bit clear

	current_interrupt_id = Controller;
	INTC_Enable(NUM_NFC);								// Interrupt Enable.

	uBlockPage=(uBlock<<(5 + NAND_Inform[Controller].uNandType)) + uPage;
	uPageSize = NAND_Inform[Controller].uPageSize;

	NF_RSTECC(Controller);    							// Initialize ECC
	NF_MECC_UnLock(Controller);							// Main Data ECC Unlock
	NF_nFCE_L(Controller);								// CS Low
	NF_CLEAR_RnB(Controller);

	NF_CMD(Controller, 0x00);							// 1st Read command
	NF_ADDR(Controller, 0); 	
	if(NAND_Inform[Controller].uNandType == NAND_Advanced8bit)	NF_ADDR(Controller, 0);
	
	NF_ADDR(Controller, uBlockPage&0xff);				//
	NF_ADDR(Controller, (uBlockPage>>8)&0xff);			// Block & Page num.
	NF_ADDR(Controller, (uBlockPage>>16)&0xff);			//
	if(NAND_Inform[Controller].uNandType == NAND_Advanced8bit)	NF_CMD(Controller, 0x30);	// 2'nd command
	NF_DETECT_RnB(Controller);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<uPageSize/4 ; i++) *pBufferW++ = NF_RDDATA(Controller);	// Read one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)(&(NAND(Controller)->rNFDATA)), 1, (u32)pBufferW, 0, WORD, uPageSize/4, DEMAND, DMA1_NAND_TX, MEM, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
#endif

	NF_MECC_Lock(Controller);							// Main Data ECC Stop. ECC Lock

	if(NAND_Inform[Controller].uNandType == NAND_Advanced8bit)
	{
		if(pSpareBuffer) *pSpareBuffer++ = NF_RDDATA8(Controller);
		else 			 uMecc = NF_RDDATA8(Controller);
	}
	
	NF_SECC_UnLock(Controller);							// Spare ECC Unlock
	uMecc = NF_RDDATA(Controller);
	NF_SECC_Lock(Controller);							// Spare ECC Lock
	
	Outp32(&NAND(Controller)->rNFMECCD0, ((uMecc&0xff00)<<8)|(uMecc&0xff) );
	Outp32(&NAND(Controller)->rNFMECCD1, ((uMecc&0xff000000)>>8)|((uMecc&0xff0000)>>16) );

	if(NAND_Inform[Controller].uNandType == NAND_Normal8bit)
	{
		if(pSpareBuffer) *pSpareBuffer++ = NF_RDDATA8(Controller);
		else 			 uMecc = NF_RDDATA8(Controller);
	}

	for(i=5;i<8;i++) 
	{
		if(pSpareBuffer)	*pSpareBuffer++ = NF_RDDATA8(Controller);
		else 				uSecc = NF_RDDATA8(Controller);
	}
	
	uSecc = NF_RDDATA8(Controller);
	uSecc |= NF_RDDATA8(Controller) << 8;
	
	if(pSpareBuffer)
	{
		*pSpareBuffer++ = (uSecc & 0xff);
		*pSpareBuffer++ = (uSecc & 0xff00) >> 8;
	}
	
	for(i=10; i<NAND_Inform[Controller].uSpareSize ; i++) *pSpareBuffer++ = NF_RDDATA8(Controller);
	Outp32(&NAND(Controller)->rNFSECCD, ((uSecc&0xff00)<<8)|(uSecc&0xff));

	NF_nFCE_H(Controller); 

	return Nand_CheckECCError(Controller);
}

static NAND_eERROR Nand_ReadPageMLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *pSpareBuffer)
{
	u32 i, uBlockPage, uPageSize, uTemp;
	NAND_eERROR eError;
	u32 *pBufferW;
			
	pBufferW = (u32 *)pBuffer;
	g_Nand_4bitEccDecDone = 0;
	g_Nand_RnBTransition = 0;
	
	uTemp = Nand_ReadNFCONRegister(Controller) | (1<<12)|(1<<13) | (1<<9)|(1<<10) & ~(1<<18);
	// 4bit ECC decoding/encoding Interrupt enable, RnB, Illegal Acess Interupt Enable, Decoding 4bit ECC selection
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);
	Outp32(&(NAND(Controller)->rNFSTAT), (1<<6));		// 4bit ECC Decoding finish bit clear
	Outp32(&(NAND(Controller)->rNFSTAT), (1<<5));		// Illegal access Pending clear.
	Outp32(&(NAND(Controller)->rNFSTAT), (1<<4));		// RnB pending clear

	current_interrupt_id = Controller;
	INTC_Enable(NUM_NFC);

	uBlockPage = (uBlock<<7) + uPage;
	uPageSize = NAND_Inform[Controller].uPageSize;

	//////////////////////////
	// Read 0 ~ 511
	//////////////////////////
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);
	NF_nFCE_L(Controller);   
	NF_CLEAR_RnB(Controller);

	NF_CMD(Controller, 0x00);	// 1st Read command
	NF_ADDR(Controller, 0); 	
	NF_ADDR(Controller, 0); 	
	NF_ADDR(Controller, uBlockPage&0xff);		//
	NF_ADDR(Controller, (uBlockPage>>8)&0xff);	// Block & Page num.
	NF_ADDR(Controller, (uBlockPage>>16)&0xff);	//
	NF_CMD(Controller, 0x30);	// 2'nd command
	while(!g_Nand_RnBTransition);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) *pBufferW++ = NF_RDDATA(Controller);	// Read one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)(&(NAND(Controller)->rNFDATA)), 1, (u32)pBufferW, 0, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, DMA1_NAND_TX, MEM, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_CMD(Controller, 0x05);
	NF_ADDR(Controller, (uPageSize)&0xFF);
	NF_ADDR(Controller, ((uPageSize)>>8)&0xFF);
	NF_CMD(Controller, 0xE0);

	for(i=0 ; i<7 ; i++) 
	{
		if(pSpareBuffer)	*pSpareBuffer++ = NF_RDDATA8(Controller);
		else				uTemp = NF_RDDATA8(Controller);
	}

	while(!g_Nand_4bitEccDecDone); // 6400 Code ISR Using. 
	NF_MECC_Lock(Controller);

	eError = Nand_CheckECCError(Controller);
	if(eError != eNAND_NoError)
	{
		NF_MECC_Lock(Controller);	
		NF_nFCE_H(Controller);
		uTemp = Nand_ReadNFCONRegister(Controller) & ~((1<<12)|(1<<13));	// 4bit ECC decoding Interrupt disable
		Outp32(&(NAND(Controller)->rNFCONT), uTemp);	
		return eError;
	}

	//////////////////////////
	// Read 512 ~ 1023
	//////////////////////////
	g_Nand_4bitEccDecDone = 0;
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);

	NF_CMD(Controller, 0x05);
	NF_ADDR(Controller, (512)&0xFF);
	NF_ADDR(Controller, (512>>8)&0xFF);
	NF_CMD(Controller, 0xE0);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) *pBufferW++ = NF_RDDATA(Controller);	// Read one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)(&(NAND(Controller)->rNFDATA)), 1, (u32)pBufferW, 0, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, DMA1_NAND_TX, MEM, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_CMD(Controller, 0x05);
	NF_ADDR(Controller, (uPageSize+7)&0xFF);
	NF_ADDR(Controller, ((uPageSize+7)>>8)&0xFF);
	NF_CMD(Controller, 0xE0);

	for(i=7;i<14;i++) 
	{
		if(pSpareBuffer)	*pSpareBuffer++ = NF_RDDATA8(Controller);
		else				uTemp = NF_RDDATA8(Controller);
	}	

	while(!g_Nand_4bitEccDecDone); // 6400 Code ISR Using. 
	NF_MECC_Lock(Controller);

	eError = Nand_CheckECCError(Controller);
	if(eError != eNAND_NoError)
	{
		NF_MECC_Lock(Controller);	
		NF_nFCE_H(Controller);
		uTemp = Nand_ReadNFCONRegister(Controller) & ~((1<<12)|(1<<13));	// 4bit ECC decoding Interrupt disable
		Outp32(&(NAND(Controller)->rNFCONT), uTemp);			
		return eError;
	}
	
	//////////////////////////
	// Read 1024 ~ 1535
	//////////////////////////
	g_Nand_4bitEccDecDone = 0;
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);

	NF_CMD(Controller, 0x05);
	NF_ADDR(Controller, (512*2)&0xFF);
	NF_ADDR(Controller, ((512*2)>>8)&0xFF);
	NF_CMD(Controller, 0xE0);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) *pBufferW++ = NF_RDDATA(Controller);	// Read one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
//		DMACH_Setup(DMA_A, 0x0, (u32)(&(NAND(Controller)->rNFDATA)), 1, (u32)pBufferW, 0, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, DMA1_NAND_TX, MEM, BURST128, &g_oNandDmac1);
//		DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_CMD(Controller, 0x05);
	NF_ADDR(Controller, (uPageSize+14)&0xFF);
	NF_ADDR(Controller, ((uPageSize+14)>>8)&0xFF);
	NF_CMD(Controller, 0xE0);

	for(i=14;i<21;i++) 
	{
		if(pSpareBuffer)	*pSpareBuffer++ = NF_RDDATA8(Controller);
		else				uTemp = NF_RDDATA8(Controller);
	}
	
	while(!g_Nand_4bitEccDecDone); // 6400 Code ISR Using. 
	NF_MECC_Lock(Controller);

	eError = Nand_CheckECCError(Controller);
	if(eError != eNAND_NoError)
	{
		NF_MECC_Lock(Controller);		
		NF_nFCE_H(Controller);
		uTemp = Nand_ReadNFCONRegister(Controller) & ~(1<<12);	// 4bit ECC decoding Interrupt enable
		Outp32(&(NAND(Controller)->rNFCONT), uTemp);	
		return eError;
	}
	
	//////////////////////////
	// Read 1536 ~ 2047
	//////////////////////////
	g_Nand_4bitEccDecDone = 0;
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);

	NF_CMD(Controller, 0x05);
	NF_ADDR(Controller, (512*3)&0xFF);
	NF_ADDR(Controller, ((512*3)>>8)&0xFF);
	NF_CMD(Controller, 0xE0);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) *pBufferW++ = NF_RDDATA(Controller);	// Read one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)(&(NAND(Controller)->rNFDATA)), 1, (u32)pBufferW, 0, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, DMA1_NAND_TX, MEM, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_CMD(Controller, 0x05);
	NF_ADDR(Controller, (uPageSize+21)&0xFF);
	NF_ADDR(Controller, ((uPageSize+21)>>8)&0xFF);
	NF_CMD(Controller, 0xE0);

	for(i=21;i<28;i++) 
	{
		if(pSpareBuffer)	*pSpareBuffer++ = NF_RDDATA8(Controller);
		else				uTemp = NF_RDDATA8(Controller);
	}	

	while(!g_Nand_4bitEccDecDone); // 6400 Code ISR Using. 
	NF_MECC_Lock(Controller);

	eError = Nand_CheckECCError(Controller);
	if(eError != eNAND_NoError)
	{
		NF_MECC_Lock(Controller);		
		NF_nFCE_H(Controller);
		uTemp = Nand_ReadNFCONRegister(Controller) & ~(1<<12);	// 4bit ECC decoding Interrupt enable
		Outp32(&(NAND(Controller)->rNFCONT), uTemp);		
		return eError;
	}

	NF_nFCE_H(Controller);

	uTemp = Nand_ReadNFCONRegister(Controller) & ~((1<<12)|(1<<13));	// 4bit ECC decoding/encoding Interrupt disable
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);
	
	INTC_Disable(NUM_NFC);
	
	return eNAND_NoError;
}

static NAND_eERROR Nand_WritePageSLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *aSpareBuffer)
{
	u32 i, uBlockPage, uPageSize, uSpareSize;
	u32 uMecc, uSecc, uStatus;
	NAND_eERROR eError;	
	u32 *pBufferW;
	u32 uNFCONFreg, uNFCONTreg;
			
	pBufferW = (u32 *)pBuffer;			
	g_Nand_RnBTransition = 0;


	uNFCONFreg = Inp32(&(NAND(Controller)->rNFCONF)) & (~(0x1<<30)) & (~(0x3<<23)) | ((7<<12)|(7<<8)|(7<<4));
	// HCLK is more than 66Mhz., 1bit ECC Select, Timing 7,7,7 setting.
	Outp32(&(NAND(Controller)->rNFCONF), uNFCONFreg ); // NFCONF Register setting. 

	uNFCONTreg = Inp32(&(NAND(Controller)->rNFCONT)) & ~((1<<13)|(1<<12)) | (1<<10)|(1<<9);
	// 4bit ECC encoding/decoding completion Interrupt disable, Illegal Access & RnB Transition Interrupt enable
	Outp32(&(NAND(Controller)->rNFCONT), uNFCONTreg); // write setting. NFCONT Register
	Outp32(&(NAND(Controller)->rNFSTAT), (1<<5)|(1<<4)); // Illegal Access & RnB Interrupt Pending bit clear

	current_interrupt_id = Controller;
	INTC_Enable(NUM_NFC); // Interrupt Enable
	
	uBlockPage=(uBlock<<(5 + NAND_Inform[Controller].uNandType)) + uPage;
	uPageSize = NAND_Inform[Controller].uPageSize;
	uSpareSize = NAND_Inform[Controller].uSpareSize;

	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);
	NF_nFCE_L(Controller);    
	NF_CLEAR_RnB(Controller);

	NF_CMD(Controller, 0x80);	// Program command
	NF_ADDR(Controller, 0);
	if(NAND_Inform[Controller].uNandType == NAND_Advanced8bit) NF_ADDR(Controller, 0); 
	NF_ADDR(Controller, uBlockPage&0xff);		//
	NF_ADDR(Controller, (uBlockPage>>8)&0xff);	// Block & Page num.
	NF_ADDR(Controller, (uBlockPage>>16)&0xff);	//

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<uPageSize/4 ; i++) NF_WRDATA(Controller, *pBufferW++);	// Read one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)pBufferW, 0, (u32)(&(NAND(Controller)->rNFDATA)), 1, WORD, uPageSize/4, DEMAND, MEM, DMA1_NAND_RX, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
#endif

	NF_MECC_Lock(Controller);
	g_Nand_IllegalAccError  = 0;

	uMecc = Inp32(&NAND(Controller)->rNFMECC0);		// Main Area ECC

	if(NAND_Inform[Controller].uNandType == NAND_Normal8bit)
	{
		aNand_Spare_Data[0] = (u8)(uMecc&0xFF);
		aNand_Spare_Data[1] = (u8)((uMecc&0xFF00)>>8);
		aNand_Spare_Data[2] = (u8)((uMecc&0xFF0000)>>16);
		aNand_Spare_Data[3] = (u8)((uMecc&0xFF000000)>>24);
		aNand_Spare_Data[5] = (u8)0xFF;			//Marking Valid Block

		NF_SECC_UnLock(Controller); // Spare area data write. and start spare ecc making.
		for(i=0;i<4;i++) NF_WRDATA8(Controller, aNand_Spare_Data[i]);		// Write spare array(Main ECC) 4Byte Main ECC Data
		NF_SECC_Lock(Controller); // Spare ECC Unlock 
		NF_WRDATA8(Controller, aNand_Spare_Data[4]);
	}
	else
	{	
		aNand_Spare_Data[0] = (u8)0xFF;				//Marking Valid Block
		aNand_Spare_Data[1] = (u8)(uMecc&0xFF);
		aNand_Spare_Data[2] = (u8)((uMecc&0xFF00)>>8);
		aNand_Spare_Data[3] = (u8)((uMecc&0xFF0000)>>16);
		aNand_Spare_Data[4] = (u8)((uMecc&0xFF000000)>>24);

		NF_WRDATA8(Controller, aNand_Spare_Data[0]);	//Write Marking Valid Block
		NF_SECC_UnLock(Controller); // Spare ECC create , Spare Ecc Unlock
		for(i=1;i<5;i++) NF_WRDATA8(Controller, aNand_Spare_Data[i]);	// Write spare array(Main ECC)
		NF_SECC_Lock(Controller); // Spare ECC Lock , End
	}

	uSecc = Inp32(&NAND(Controller)->rNFSECC) & 0xFFFF;		// Spare Area ECC
	aNand_Spare_Data[8] = (u8)(uSecc&0xFF); 
	aNand_Spare_Data[9] = (u8)((uSecc&0xFF00)>>8);
	for(i=5;i<uSpareSize;i++) NF_WRDATA8(Controller, aNand_Spare_Data[i]);  // Write spare array(Spare ECC and Mark)

	NF_CLEAR_RnB(Controller);
	NF_CMD(Controller, 0x10);	 // Write 2nd command	

	while(!g_Nand_RnBTransition)
	{
		if(g_Nand_IllegalAccError == 1)
		{
			g_Nand_IllegalAccError = 0;
			NF_CMD(Controller, 0x70); // Read Status register
			NF_nFCE_H(Controller);
			
			uNFCONTreg = Inp32(&(NAND(Controller)->rNFCONT)) &= ~((1<<10) | (1<<9));	// Illegal Access & RnB Transition Interrupt disable
			Outp32(&(NAND(Controller)->rNFCONT), uNFCONTreg);
			INTC_Disable(NUM_NFC);
			return eNAND_ProgramError;
		}
	}

	NF_CMD(Controller, 0x70); //Read Program Status
	while(!((uStatus = NF_RDDATA8(Controller)) & (1<<6)));
	if(uStatus & 0x01)	eError = eNAND_ProgramError;		// Error in Program
	else				eError = eNAND_NoError;
	
	NF_nFCE_H(Controller);
	
	uNFCONTreg = Inp32(&(NAND(Controller)->rNFCONT)) & ~((1<<10) | (1<<9));		// Illegal Access & RnB Transition Interrupt disable
	Outp32(&(NAND(Controller)->rNFCONT), uNFCONTreg);
	if(aSpareBuffer) for(i=0;i<uSpareSize;i++) *aSpareBuffer++ = aNand_Spare_Data[i];
	
	INTC_Disable(NUM_NFC);
	return eError;
}

static NAND_eERROR Nand_WritePageMLC(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *aSpareBuffer)
{
	u32 i, uBlockPage, uSpareSize, uTemp;
	u32 uMecc0, uMecc1, uStatus;
	NAND_eERROR eError;
	u32 *pBufferW;
			
	pBufferW = (u32 *)pBuffer;
	g_Nand_4bitEccEncDone = 0;
	g_Nand_RnBTransition = 0;
	g_Nand_IllegalAccError = 0;
	
	uTemp = Nand_ReadNFCONRegister(Controller)|(1<<18)|(1<<13)|(1<<10)|(1<<9);
	// 4bit ECC encoding & Illegal Access & RnB Transition Interrupt enable
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);
	Outp32(&(NAND(Controller)->rNFSTAT), (1<<7));		// 4bit ECC encoding finish bit clear

	current_interrupt_id = Controller;
	INTC_Enable(NUM_NFC);

	uBlockPage=(uBlock<<7) + uPage;
	uSpareSize = NAND_Inform[Controller].uSpareSize;

	//////////////////////////
	// Write 0 ~ 511
	//////////////////////////
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);
	NF_nFCE_L(Controller);    
	NF_CLEAR_RnB(Controller);

	NF_CMD(Controller, 0x00);	// 1st Read command..????
	NF_CMD(Controller, 0x80);	// 1st Write command
	NF_ADDR(Controller, 0); 	
	NF_ADDR(Controller, 0); 	
	NF_ADDR(Controller, uBlockPage&0xff);		//
	NF_ADDR(Controller, (uBlockPage>>8)&0xff);	// Block & Page num.
	NF_ADDR(Controller, (uBlockPage>>16)&0xff);	//

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) NF_WRDATA(Controller, *pBufferW++);	// Write one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)pBufferW, 0, (u32)(&(NAND(Controller)->rNFDATA)), 1, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, MEM, DMA1_NAND_RX, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
   	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_MECC_Lock(Controller);
	while(!g_Nand_4bitEccEncDone);

	uMecc0 = Inp32(&(NAND(Controller)->rNFMECC0));
	uMecc1 = Inp32(&(NAND(Controller)->rNFMECC1)) & 0xFFFFFF;
	aNand_Spare_Data[0]=(u8)(uMecc0&0xff);
	aNand_Spare_Data[1]=(u8)((uMecc0>>8) & 0xff);
	aNand_Spare_Data[2]=(u8)((uMecc0>>16) & 0xff);
	aNand_Spare_Data[3]=(u8)((uMecc0>>24) & 0xff);
	aNand_Spare_Data[4]=(u8)(uMecc1&0xff);
	aNand_Spare_Data[5]=(u8)((uMecc1>>8) & 0xff);
	aNand_Spare_Data[6]=(u8)((uMecc1>>16) & 0xff);	

	//////////////////////////
	// Write 512 ~ 1023
	//////////////////////////
	g_Nand_4bitEccEncDone = 0;
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) NF_WRDATA(Controller, *pBufferW++);	// Write one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)pBufferW, 0, (u32)(&(NAND(Controller)->rNFDATA)), 1, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, MEM, DMA1_NAND_RX, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
   	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_MECC_Lock(Controller);
	while(!g_Nand_4bitEccEncDone);

	uMecc0 = Inp32(&(NAND(Controller)->rNFMECC0));
	uMecc1 = Inp32(&(NAND(Controller)->rNFMECC1)) & 0xFFFFFF;
	aNand_Spare_Data[7]=(u8)(uMecc0&0xff);
	aNand_Spare_Data[8]=(u8)((uMecc0>>8) & 0xff);
	aNand_Spare_Data[9]=(u8)((uMecc0>>16) & 0xff);
	aNand_Spare_Data[10]=(u8)((uMecc0>>24) & 0xff);
	aNand_Spare_Data[11]=(u8)(uMecc1&0xff);
	aNand_Spare_Data[12]=(u8)((uMecc1>>8) & 0xff);
	aNand_Spare_Data[13]=(u8)((uMecc1>>16) & 0xff);
	
	//////////////////////////
	// Write 1024 ~ 1535
	//////////////////////////
	g_Nand_4bitEccEncDone = 0;
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) NF_WRDATA(Controller, *pBufferW++);	// Write one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)pBufferW, 0, (u32)(&(NAND(Controller)->rNFDATA)), 1, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, MEM, DMA1_NAND_RX, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
   	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_MECC_Lock(Controller);
	while(!g_Nand_4bitEccEncDone);

	uMecc0 = Inp32(&(NAND(Controller)->rNFMECC0));
	uMecc1 = Inp32(&(NAND(Controller)->rNFMECC1)) & 0xFFFFFF;
	aNand_Spare_Data[14]=(u8)(uMecc0&0xff);
	aNand_Spare_Data[15]=(u8)((uMecc0>>8) & 0xff);
	aNand_Spare_Data[16]=(u8)((uMecc0>>16) & 0xff);
	aNand_Spare_Data[17]=(u8)((uMecc0>>24) & 0xff);
	aNand_Spare_Data[18]=(u8)(uMecc1&0xff);
	aNand_Spare_Data[19]=(u8)((uMecc1>>8) & 0xff);
	aNand_Spare_Data[20]=(u8)((uMecc1>>16) & 0xff);
	
	//////////////////////////
	// Write 1536 ~ 2047
	//////////////////////////
	g_Nand_4bitEccEncDone = 0;
	NF_RSTECC(Controller);    // Initialize ECC
	NF_MECC_UnLock(Controller);

#if NAND_TRANSFER_MODE == POLLING_TRANSFER
	for(i=0 ; i<NAND_MLC_TRANS_SIZE/4 ; i++) NF_WRDATA(Controller, *pBufferW++);	// Write one page
#elif NAND_TRANSFER_MODE == DMA_TRANSFER
	Nand_DmaDone = 0;
	DMACH_Setup(DMA_A, 0x0, (u32)pBufferW, 0, (u32)(&(NAND(Controller)->rNFDATA)), 1, WORD, NAND_MLC_TRANS_SIZE/4, DEMAND, MEM, DMA1_NAND_RX, BURST128, &g_oNandDmac1);
	DMACH_Start(&g_oNandDmac1);
	while(!Nand_DmaDone);	
   	pBufferW += NAND_MLC_TRANS_SIZE/4;
#endif

	NF_MECC_Lock(Controller);
	while(!g_Nand_4bitEccEncDone);

	uMecc0 = Inp32(&(NAND(Controller)->rNFMECC0));
	uMecc1 = Inp32(&(NAND(Controller)->rNFMECC1)) & 0xFFFFFF;
	aNand_Spare_Data[21]=(u8)(uMecc0&0xff);
	aNand_Spare_Data[22]=(u8)((uMecc0>>8) & 0xff);
	aNand_Spare_Data[23]=(u8)((uMecc0>>16) & 0xff);
	aNand_Spare_Data[24]=(u8)((uMecc0>>24) & 0xff);
	aNand_Spare_Data[25]=(u8)(uMecc1&0xff);
	aNand_Spare_Data[26]=(u8)((uMecc1>>8) & 0xff);
	aNand_Spare_Data[27]=(u8)((uMecc1>>16) & 0xff);

	//////////////////////////
	// Spare Area Write
	//////////////////////////
	for(i=0 ; i<uSpareSize ; i++) NF_WRDATA8(Controller, aNand_Spare_Data[i]);

	NF_CMD(Controller, 0x10);	// 2nd Write Command

	while(!g_Nand_RnBTransition)
	{
		if(g_Nand_IllegalAccError == 1)
		{
			g_Nand_IllegalAccError = 0;
			NF_CMD(Controller, 0x70);
			NF_nFCE_H(Controller);
	
			uTemp = Nand_ReadNFCONRegister(Controller) & ~((1<<13)|(1<<12)|(1<<10)|(1<<9));		// 4bit ECC encoding & Illegal Access & RnB Transition Interrupt disable
			Outp32(&(NAND(Controller)->rNFCONT), uTemp);
			INTC_Disable(NUM_NFC);	
			
			return eNAND_ProgramError;
		}
	}
	
	//Read Program Status
	NF_CMD(Controller, 0x70);
	while(!((uStatus = NF_RDDATA8(Controller)) & (1<<6)));
	if(uStatus & 0x01)	eError = eNAND_ProgramError;		// Error in Program
	else				eError = eNAND_NoError;

	NF_nFCE_H(Controller);
	
	uTemp = Nand_ReadNFCONRegister(Controller) & ~((1<<13)|(1<<12)|(1<<10)|(1<<9));
	// 4bit ECC encoding & Illegal Access & RnB Transition Interrupt disable
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);
	if(aSpareBuffer) for(i=0 ; i<uSpareSize ; i++) *aSpareBuffer++ = aNand_Spare_Data[i];
	
	INTC_Disable(NUM_NFC);
	
	return eError;
}

NAND_eERROR Nand_Device_SoftLockingBlock(u32 Controller, u32 uStartBlock, u32 uEndBlock)
{
	u32 uTemp;

	// Nand Type Select
	Outp32(&(NAND(Controller)->rNFSBLK), uStartBlock<<(5 + NAND_Inform[Controller].uNandType));
	Outp32(&(NAND(Controller)->rNFEBLK), (uEndBlock)<<(5 + NAND_Inform[Controller].uNandType));
	
	uTemp = Nand_ReadNFCONRegister(Controller)|(1<<16);		// Enable Soft lock
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);	

	return eNAND_NoError;
}

NAND_eERROR Nand_Device_LockTightBlock(u32 Controller, u32 uStartBlock, u32 uEndBlock)
{
	u32 uTemp;

	// Nand Type Select
	Outp32(&(NAND(Controller)->rNFSBLK), uStartBlock<<(5 + NAND_Inform[Controller].uNandType));
	Outp32(&(NAND(Controller)->rNFEBLK), (uEndBlock)<<(5 + NAND_Inform[Controller].uNandType));
	
	uTemp = Nand_ReadNFCONRegister(Controller)|(1<<17);		// Enable Lock-tight
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);	

	return eNAND_NoError;
}

NAND_eERROR Nand_Device_SoftUnLockBlock(u32 Controller)
{
	u32 uTemp;

	uTemp = Nand_ReadNFCONRegister(Controller);
	if(!(uTemp & (1<<16)))	return eNAND_NoError;
	
	Outp32(&(NAND(Controller)->rNFSBLK), 0);
	Outp32(&(NAND(Controller)->rNFEBLK), 0);
	
	uTemp &= ~(1<<16);		// Disable Soft lock
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);	

	uTemp = Nand_ReadNFCONRegister(Controller);
	if(uTemp & (1<<16))	return eNAND_EtcError;
	
	return eNAND_NoError;
}

NAND_eERROR Nand_Device_UnLockTightBlock(u32 Controller)
{
	u32 uTemp;

	uTemp = Nand_ReadNFCONRegister(Controller);
	if(!(uTemp & (1<<17)))	return eNAND_NoError;

	Outp32(&(NAND(Controller)->rNFSBLK), 0);
	Outp32(&(NAND(Controller)->rNFEBLK), 0);
	
	uTemp &= ~(1<<17);		// Disable lock-tight. but this bit is not cleared...for test
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);	

	uTemp = Nand_ReadNFCONRegister(Controller);
	if(!(uTemp & (1<<17)))	return eNAND_EtcError;
	
	return eNAND_NoError;
}

u8 Nand_Device_Init(u32 Controller)
{
	u32 uTemp;
	u32 uTacls, uTwrph0, uTwrph1;

	INTC_SetVectAddr(NUM_NFC, Nand_ISR0);
	Nand_GetDurationValue(Controller, &uTacls, &uTwrph0, &uTwrph1);

	if(NAND_Inform[Controller].uNandType == NAND_Normal8bit)
	{
		Outp32(&NAND(Controller)->rNFCONF, (uTacls<<12)|(uTwrph0<<8)|(uTwrph1<<4)|(0<<3)|(1<<2)|((NAND_Inform[Controller].uAddrCycle-3)<<1));
	}
	else if(NAND_Inform[Controller].uNandType == NAND_Advanced8bit)
	{
		Outp32(&NAND(Controller)->rNFCONF, (uTacls<<12)|(uTwrph0<<8)|(uTwrph1<<4)|(1<<3)|(1<<2)|((NAND_Inform[Controller].uAddrCycle-4)<<1));
	}
	else if(NAND_Inform[Controller].uNandType == NAND_MLC8bit)
	{
		Outp32(&NAND(Controller)->rNFCONF, (1<<24)|(uTacls<<12)|(uTwrph0<<8)|(uTwrph1<<4)|(1<<3)|(1<<2)|((NAND_Inform[Controller].uAddrCycle-4)<<1));
	}
	else
	{
		return FALSE;
	}
	
	Outp32(&NAND(Controller)->rNFCONT, (0<<17)|(0<<16)|(0<<10)|(0<<9)|(0<<8)|(1<<7)|(1<<6)|(1<<5)|(1<<4)|(0x3<<1)|(1<<0));

	uTemp = Inp32(&(NAND(Controller)->rNFCONF));
	if(PJBoot64.hclk > 66000000) uTemp &= ~(1<<30);
	
	Nand_Reset(Controller);

#if NAND_TRANSFER_MODE == DMA_TRANSFER
	DMAC_InitCh(SDMA0, DMA_ALL, &g_oNandDmac1);
	INTC_SetVectAddr(NUM_SDMA0,  Nand_DmaISR);
	INTC_Enable(NUM_SDMA0);	
#endif

	return TRUE;
}

u32 Nand_Device_ReadID(u32 Controller)
{
	u32 i, usID;

	NF_nFCE_L(Controller);

	NF_CMD(Controller, 0x90);
	NF_ADDR(Controller, 0x0);

	for (i=0; i<10; i++);
	
	usID = NF_RDDATA8(Controller) << 8;
	usID |= NF_RDDATA8(Controller);

	NF_nFCE_H(Controller);

	return 	usID;	
}

u32 Nand_Device_ReadID_NotInit(u32 Controller)
{
	u32 i, usID;

	// Default Init Nand Flash
	Outp32(&NAND(Controller)->rNFCONF, (7<<12)|(7<<8)|(7<<4)|(0<<3)|(1<<2)|0);
	Outp32(&NAND(Controller)->rNFCONT, (0<<17)|(0<<16)|(0<<10)|(0<<9)|(0<<8)|(1<<7)|(1<<6)|(1<<5)|(1<<4)|(0x3<<1)|(1<<0));
	
	// Nand Flash Reset
	NF_nFCE_L(Controller);
	NF_CLEAR_RnB(Controller);
	NF_CMD(Controller, 0xFF);	//reset command
	while(!(NAND(Controller)->rNFSTAT & (1<<0)));
	NF_nFCE_H(Controller);

	// Nand Flash Read ID
	NF_nFCE_L(Controller);
	NF_CMD(Controller, 0x90);
	NF_ADDR(Controller, 0x0);
	NF_ADDR(Controller, 0x0);
	while(!(NAND(Controller)->rNFSTAT & (1<<0)));
	usID = NF_RDDATA(Controller) & 0xffff;
	NF_nFCE_H(Controller);
		
	return SWAP16(usID);
}

NAND_eERROR Nand_Device_CheckInvalidBlock(u32 Controller, u32 uBlock)
{
	u32 uBlockPage, uPageSize;
	u8 ucData;

	g_Nand_RnBTransition =0;
		
	INTC_Disable(NUM_NFC);
	
	switch(NAND_Inform[Controller].uNandType)
	{
		case NAND_Normal8bit:
		case NAND_Advanced8bit:
		case NAND_MLC8bit:
			uBlock=(uBlock<<(5 + NAND_Inform[Controller].uNandType));
			uPageSize = NAND_Inform[Controller].uPageSize;
			break;
		default:
			return eNAND_EtcError;
			
	}
	
	if(NAND_Inform[Controller].uNandType == NAND_Normal8bit)
	{
		uBlockPage=(uBlock<<5);
		uPageSize = NAND_Inform[Controller].uPageSize;
	}
	else if(NAND_Inform[Controller].uNandType == NAND_Advanced8bit)
	{
		uBlockPage=(uBlock<<6);
		uPageSize = NAND_Inform[Controller].uPageSize;
	}
	else if(NAND_Inform[Controller].uNandType == NAND_MLC8bit)
	{
		uBlockPage=(uBlock<<7);
		uPageSize = NAND_Inform[Controller].uPageSize;
	}
	else 
	{
		return eNAND_EtcError;
	}

	NF_nFCE_L(Controller);
	NF_CLEAR_RnB(Controller);

	if(NAND_Inform[Controller].uAddrCycle == 4)
	{
		NF_CMD(Controller, 0x50);		 				// Spare array read command
		NF_ADDR(Controller, (uPageSize+5)&0xf);			// 6th byte in the Spare Area
	}
	else if(NAND_Inform[Controller].uAddrCycle == 5)
	{
		NF_CMD(Controller, 0x00);						// 1st Read Command
		NF_ADDR(Controller, (uPageSize+0)&0xff);
		NF_ADDR(Controller, ((uPageSize+0)>>8)&0xff);	// 1st byte in the Spare Area		
	}			

	NF_ADDR(Controller, uBlockPage&0xff);				// The mark of bad block is in 0 page
	NF_ADDR(Controller, (uBlockPage>>8)&0xff);			// For block number A[24:17]
	NF_ADDR(Controller, (uBlockPage>>16)&0xff);			// For block number A[25]
	if(NAND_Inform[Controller].uNandType != NAND_Normal8bit) NF_CMD(Controller, 0x30);	// 2'nd command
	 
	NF_DETECT_RnB(Controller);							// Wait tR(max 12us)
	NF_CLEAR_RnB(Controller);

	ucData=NF_RDDATA8(Controller);

	NF_CMD(Controller, 0x00);							// Define the starting address of the 1st half of the register
	NF_nFCE_H(Controller);    

	if(ucData!=0xff)	return eNAND_InvalidBlock;
	else				return eNAND_NoError;
}

NAND_eERROR Nand_Device_ReadPage(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *pSpareBuffer)
{
	NAND_eERROR eError;

	if( (NAND_Inform[Controller].uNandType == NAND_Normal8bit) || (NAND_Inform[Controller].uNandType == NAND_Advanced8bit) )
		eError = Nand_ReadPageSLC(Controller, uBlock, uPage, pBuffer, pSpareBuffer);
	else
		eError = Nand_ReadPageMLC(Controller, uBlock, uPage, pBuffer, pSpareBuffer);

	return eError;
}

NAND_eERROR Nand_Device_WritePage(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *aSpareBuffer)
{
	NAND_eERROR eError;

	if( (NAND_Inform[Controller].uNandType == NAND_Normal8bit) || (NAND_Inform[Controller].uNandType == NAND_Advanced8bit) )
		eError = Nand_WritePageSLC(Controller, uBlock, uPage, pBuffer, aSpareBuffer);
	else
		eError = Nand_WritePageMLC(Controller, uBlock, uPage, pBuffer, aSpareBuffer);

	return eError;
}

NAND_eERROR Nand_Device_EraseSingleBlock(u32 Controller, u32 uBlock)
{
	u32 uTemp, uBlockPage, uStatus;
	NAND_eERROR eError;

	if(NAND_Inform[Controller].uNandType == NAND_Normal8bit)		uBlockPage=(uBlock<<5);
	else if(NAND_Inform[Controller].uNandType == NAND_Advanced8bit)	uBlockPage=(uBlock<<6);
	else 															uBlockPage=(uBlock<<7);

	uTemp = Nand_ReadNFCONRegister(Controller)|(1<<10)|(1<<9);		// Illegal Access & RnB Transition Interrupt enable
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);
	Outp32(&(NAND(Controller)->rNFSTAT), (1<<5)|(1<<4));		// Illegal Access & RnB Interrupt Pending bit clear

	//Invalid Block Check
//	eError = Nand_Device_CheckInvalidBlock(Controller, uBlock);
//	if(eError != eNAND_NoError)	return eError;

	current_interrupt_id = Controller;
	INTC_Enable(NUM_NFC);
	
	g_Nand_RnBTransition = 0;
	g_Nand_IllegalAccError = 0;

	NF_nFCE_L(Controller);
	NF_CLEAR_RnB(Controller);

	NF_CMD(Controller, 0x60);					// 1st Erase Command
	NF_ADDR(Controller, uBlockPage&0xff);	 		// The mark of bad block is in 0 page
	NF_ADDR(Controller, (uBlockPage>>8)&0xff);	 	// For block number A[24:17]
	NF_ADDR(Controller, (uBlockPage>>16)&0xff);  		// For block number A[25]
	NF_CMD(Controller, 0xd0);								// 2nd Erase command

	while(!g_Nand_RnBTransition)
	{
		if(g_Nand_IllegalAccError == 1) break;
	}

	if(g_Nand_IllegalAccError == 1)
	{
		g_Nand_IllegalAccError = 0;
		eError = eNAND_IllegalAccError;
		NF_CMD(Controller, 0x70);   // Read status command
	}
	else
	{
		NF_CMD(Controller, 0x70);   // Read status command
		while(!((uStatus = NF_RDDATA8(Controller)) & (1<<6)));
		if(uStatus & 0x01)	eError = eNAND_EraseError;		// Error in Program
		else				eError = eNAND_NoError;		
	}

	NF_nFCE_H(Controller);

	uTemp = Nand_ReadNFCONRegister(Controller) & ~((1<<10)|(1<<9));		// Illegal Access & RnB Transition Interrupt disable
	Outp32(&(NAND(Controller)->rNFCONT), uTemp);
	INTC_Disable(NUM_NFC);
	
	return eError;
}

