#ifndef __NAND_DEV_H__
#define __NAND_DEV_H__
#ifdef __cplusplus
extern "C" {
#endif

// Nand device type
#define NAND_Normal8bit				(0)
#define NAND_Advanced8bit			(1)
#define NAND_MLC8bit				(2)

// Nand Page size
#define NAND_PAGE_512				(512)
#define NAND_PAGE_2048				(2048)
#define NAND_PAGE_MAX				(2048)

// Nand Spare Area Size
#define NAND_SPARE_16				(16)
#define NAND_SPARE_64				(64)
#define NAND_SPARE_MAX				(64)

// Nand data transfer mode selection
#define POLLING_TRANSFER			(0)
#define DMA_TRANSFER				(1)
//#define NAND_TRANSFER_MODE		POLLING_TRANSFER
#define NAND_TRANSFER_MODE			DMA_TRANSFER

// Nand Interupt Source
#define NAND_4BITECC_ENC			(1<<7)
#define NAND_4BITECC_DEC			(1<<6)
#define NAND_ILLEGAL_ACCESS			(1<<5)
#define NAND_RnB_TRANS				(1<<4)

// MLC Type Nand Transfer Size(512byte ECC)
#define NAND_MLC_TRANS_SIZE			(512)
#define NAND_MLC_ECC_SIZE			(7)

// ECC Error Type
#define NAND_ECCERR_1BIT			(1)
#define NAND_ECCERR_2BIT			(2)
#define NAND_ECCERR_3BIT			(3)
#define NAND_ECCERR_4BIT			(4)
#define NAND_ECCERR_5BIT			(5)
#define NAND_ECCERR_6BIT			(6)
#define NAND_ECCERR_7BIT			(7)
#define NAND_ECCERR_8BIT			(8)

#define NAND_ECCERR_UNCORRECT		(5)
#define NAND_ECCERR_UNCORRECT6BIT	(6)
#define NAND_ECCERR_UNCORRECT7BIT	(7)
#define NAND_ECCERR_MULTI			(2)
#define NAND_ECCERR_AREA			(3)

// ECC Type Definitoin
#define ECC1Bit						(0)
#define ECC4Bit						(1)
#define ECC8Bit						(2)

u8 Nand_Device_Init(u32 Controller);
u32 Nand_Device_ReadID(u32 Controller);
u32 Nand_Device_ReadID_NotInit(u32 Controller);
NAND_eERROR Nand_Device_CheckInvalidBlock(u32 Controller, u32 uBlock);
NAND_eERROR Nand_Device_ReadPage(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *pSpareBuffer);
NAND_eERROR Nand_Device_WritePage(u32 Controller, u32 uBlock, u32 uPage, u8 *pBuffer, u8 *aSpareBuffer);
NAND_eERROR Nand_Device_EraseSingleBlock(u32 Controller, u32 uBlock);
NAND_eERROR Nand_Device_SoftLockingBlock(u32 Controller, u32 uStartBlock, u32 uEndBlock);
NAND_eERROR Nand_Device_LockTightBlock(u32 Controller, u32 uStartBlock, u32 uEndBlock);
NAND_eERROR Nand_Device_SoftUnLockBlock(u32 Controller);
NAND_eERROR Nand_Device_UnLockTightBlock(u32 Controller);

#ifdef __cplusplus
}
#endif
#endif //__NAND_DEV_H__



