#include "common.h"
#include "intc.h"
#include "touch_dev.h"

#define ADCTS_BASE				(0x7E00B000)
#define ADCTS					( (volatile oADCTS_REGS *) (ADCTS_BASE))

typedef struct tag_ADCTS_REGS
{
	u32 rADCCON;
	u32 rADCTSC;
	u32 rADCDLY;
	u32 rADCDAT0;
	u32 rADCDAT1;
	u32 rADCUPDN;
	u32 rADCCLRINT;
} oADCTS_REGS;

volatile ADCTS_oInform g_oADCTSInform;

/*---------------------------------- APIs of rADCCON ---------------------------------*/
u8	ADCTS_Init(void)
{
	ADCTS_SetADCClearInt();
	ADCTS_SetADCClearWKInt();
	return TRUE;
}

void ADC_InitADCCON(void)
{
	u32 uADCCONValue;

#if RESSEL12BIT
	// 12-bit, Enable Prescaler, presclar value, Normal operation, Disabled start by read operation
	uADCCONValue = (1<<16)|(1<<14)|(ADCTS_PRSCVL<<6)|(0<<2)|(0<<1);;
#else
	// 10-bit, Enable Prescaler, presclar value, Normal operation, Disabled start by read operation
	uADCCONValue = (0<<16)|(1<<14)|(ADCTS_PRSCVL<<6)|(0<<2)|(0<<1);;
#endif

	ADCTS_SetRegValue(eADCTS_ADCCON, uADCCONValue);	
}

u8 ADC_IsEOCFlag(void)
{
	if ( ((ADCTS_GetRegValue(eADCTS_ADCCON) & 0x8000) >> 15) == 1 )	return true;
	else 															return false;
}

void ADC_EnableReadStart(u8 ucSel)
{
	u32 uADCCONValue;

	uADCCONValue = (ADCTS_GetRegValue(eADCTS_ADCCON) & ~(1<<1))|(ucSel<<1);
	ADCTS_SetRegValue(eADCTS_ADCCON, uADCCONValue);
}

void ADC_EnableStart(u8 ucSel)
{
	u32 uADCCONValue;

	uADCCONValue = (ADCTS_GetRegValue(eADCTS_ADCCON) & ~(1<<0))|(ucSel<<0);
	ADCTS_SetRegValue(eADCTS_ADCCON, uADCCONValue);
}

u8 ADC_IsAfterStartup(void)
{
	if ((ADCTS_GetRegValue(eADCTS_ADCCON) & 0x1) == 0)	return true;
	else 												return false;
}
/*---------------------------------- APIs of rADCTSC ---------------------------------*/
void ADCTS_SetMeasureMode(u8 ucMeasureMode)
{
	u32 uADCTSCValue = 0;
	
	switch(ucMeasureMode)
	{
		case eADCTS_MEASURE_NOP :		uADCTSCValue = ADCTS_GetRegValue(eADCTS_ADCTSC);	break;
		case eADCTS_MEASURE_MANUAL_X :	uADCTSCValue = 0x69;								break;
		case eADCTS_MEASURE_MANUAL_Y :	uADCTSCValue = 0x9a;								break;
		case eADCTS_MEASURE_AUTO_SEQ :	uADCTSCValue = 0x9c;								break;
		case eADCTS_MEASURE_FOR_STYLUS :uADCTSCValue = 0xd3;								break;
	}

	ADCTS_SetRegValue(eADCTS_ADCTSC, uADCTSCValue);
}

void ADCTS_SetStylusDetectMode(u8 ucStylusDetectMode)
{
	u32	uADCTSCValue;

	uADCTSCValue = (ADCTS_GetRegValue(eADCTS_ADCTSC) & ~(1<<8))|(ucStylusDetectMode<<8);
	ADCTS_SetRegValue(eADCTS_ADCTSC, uADCTSCValue);
}
/*---------------------------------- APIs of rADCDLY ---------------------------------*/
void ADCTS_SetDelayClkSrc(u8 uClkSrc)
{
	u32	uADCDLYValue;

	uADCDLYValue = (ADCTS_GetRegValue(eADCTS_ADCDLY) & ~(1<<16))|(uClkSrc<<16);
	ADCTS_SetRegValue(eADCTS_ADCDLY, uADCDLYValue);
}

void ADCTS_SetDelay(u32 uDelayNum)
{
	u32 uADCDLYValue;

	uADCDLYValue = (ADCTS_GetRegValue(eADCTS_ADCDLY) & ~(0xffff))|(uDelayNum&0xffff);
	ADCTS_SetRegValue(eADCTS_ADCDLY, uADCDLYValue);
}
/*---------------------------------- APIs of rADCDATx ---------------------------------*/
u8 ADCTS_GetXPStylusIntState(void)
{
	u8	ucXPState;

	ucXPState = (ADCTS_GetRegValue(eADCTS_ADCDAT0) & (1<<15)) >> 15;
	return ucXPState;
}

u32 ADCTS_GetXPData(void)
{
	u32 uADCDAT0Value;
	
	uADCDAT0Value = ADCTS_GetRegValue(eADCTS_ADCDAT0);
	return (uADCDAT0Value & 0xfff);
}

u32 ADCTS_GetYPData(void)
{
	u32 uADCDAT1Value;
	
	uADCDAT1Value = ADCTS_GetRegValue(eADCTS_ADCDAT1);
	return (uADCDAT1Value & 0xfff);
}
/*---------------------------------- APIs of INT Clear Registers ---------------------------------*/
void ADCTS_SetADCClearInt(void)
{
	u32	uADCCLRINTValue;

	uADCCLRINTValue	= ADCTS_GetRegValue(eADCTS_ADCCLRINT);
	ADCTS_SetRegValue(eADCTS_ADCCLRINT, uADCCLRINTValue);	
}

void ADCTS_SetADCClearWKInt(void)
{
	u32	uADCCLRINTPNDNUPValue;

	uADCCLRINTPNDNUPValue = ADCTS_GetRegValue(eADCTS_ADCCLRINTPNDNUP);
	ADCTS_SetRegValue(eADCTS_ADCCLRINTPNDNUP, uADCCLRINTPNDNUPValue);	
}
/*---------------------------------- APIs of general ADCTS ---------------------------------*/
u32 ADCTS_GetRegValue(u8 ucADCTSRegId)
{
	volatile u32 *	puADCTSBaseAddr;
	volatile u32 *	puADCTSReg;	

	puADCTSBaseAddr	= &(ADCTS->rADCCON);
	puADCTSReg		= puADCTSBaseAddr + ucADCTSRegId;

	return *puADCTSReg;	
}

void ADCTS_SetRegValue(u8 ucADCTSRegId, u32 uValue)
{
	volatile u32 *	puADCTSBaseAddr;
	volatile u32 *	puADCTSReg;	

	puADCTSBaseAddr		= &(ADCTS->rADCCON);
	puADCTSReg			= puADCTSBaseAddr + ucADCTSRegId;

	*puADCTSReg		= uValue;
}
