//#include "def.h"
#include "common.h"
#include "main.h"
#include "intc.h"
#include "gpio_dev.h"
#include "uart_dev.h"

typedef struct tag_UART_REGS
{
	u32 rUlCon;
	u32 rUCon;
	u32 rUfCon;
	u32 rUmCon;
	u32 rUtrStat;
	u32 rUerStat;
	u32 rUfStat;
	u32 rUmStat;
	u32 rUtxh;
	u32 rUrxh;
	u32 rUbrDiv;
	u32 rUdivSlot;
	u32 rUintP;
	u32 rUintSp;
	u32 rUintM;
} UART_REGS;

#define UART_BASE			(0x7F005000)
#define UART_OFFSET			0x400
#define UART_CHANNEL_PORT	(volatile UART_REGS *)(UART_BASE+UART_OFFSET * ch)

#define TX_FIFO_RESET		(1<<2)
#define RX_FIFO_RESET		(1<<1)
#define TX_INT_TYPE			(1<<9) // 0:pulse 1:level
#define RX_INT_TYPE			(0<<8) // 0:pulse 1:level

#define RX_TIMEOUT_EN		(1<<7) // 0:disable 1:enable, disable for FIFO test
#define RX_ERR_INT_EN		(1<<6) // 0:disable 1:enable
#define RTS_ACTIVE			(1) // In normal mode, nRTS signal  0:low, 1:High

/////////// Global variables
volatile UART_REGS *uart_port;

///////////////////////////////////

void Uart_Putc(u8 ch, char data)
{
	uart_port = UART_CHANNEL_PORT;
	if(data=='\n')
	{
		while(!(Inp32(&uart_port->rUtrStat) & 0x2));
		Outp8(&uart_port->rUtxh,'\r');
	}
	while(!(Inp32(&uart_port->rUtrStat) & 0x2));
	Outp8(&uart_port->rUtxh,data);
	
	return;
}

s8 Uart_Getc(u8 ch)
{
	char temp8;
	
	uart_port = UART_CHANNEL_PORT;
	
	while(!(Inp32(&uart_port->rUtrStat) & 0x1));
	temp8 = Inp8(&uart_port->rUrxh);
	return temp8;
}

u8 Uart_GetKey(u8 ch)
{
	uart_port = UART_CHANNEL_PORT;
	
	if(Inp32(&uart_port->rUtrStat) & 0x1)	return Inp8(&uart_port->rUrxh);
	else									return 0;
}

u8 Uart_GetStatus(u8 ch)
{
	uart_port = UART_CHANNEL_PORT;
	return (Inp32(&uart_port->rUtrStat) & 0x1);
}

u8 Uart_Getchar(u8 ch)
{
	uart_port = UART_CHANNEL_PORT;
	return Inp8(&uart_port->rUrxh);
}

u32 UART_InitializeREG(volatile UART_REGS *pUartRegs, UART_CON *pUartCon) // Initialize register set with current control set
{
	u32 aSlotTable[16] = {0x0000, 0x0080, 0x0808, 0x0888, 0x2222, 0x4924, 0x4a52, 0x54aa,	0x5555, 0xd555, 0xd5d5, 0xddd5, 0xdddd, 0xdfdd, 0xdfdf, 0xffdf};
	float tempDiv;
	u32 nSlot;
	
	tempDiv = (PJBoot64.pclk/(16.*pUartCon->uBaudrate)) - 1;
	nSlot = (u32)(((tempDiv - (int)tempDiv) * 16)+0.5);
	Outp32(&pUartRegs->rUbrDiv , (u32)(tempDiv));
	Outp32(&pUartRegs->rUdivSlot , aSlotTable[nSlot]);	
	
	Outp32(&pUartRegs->rUfCon , (pUartCon->cTxTrig<<6)|(pUartCon->cRxTrig<<4)|TX_FIFO_RESET|RX_FIFO_RESET|(pUartCon->cEnableFifo));
	Outp32(&pUartRegs->rUlCon , (pUartCon->cSelUartIrda<<6)|(pUartCon->cParityBit<<3)|(pUartCon->cStopBit<<2)|(pUartCon->cDataBit));
	Outp32(&pUartRegs->rUCon , (pUartCon->cOpClock<<10)|TX_INT_TYPE|RX_INT_TYPE|RX_ERR_INT_EN|RX_TIMEOUT_EN	|(pUartCon->cLoopTest<<5)| (pUartCon->cSendBreakSignal<<4) |(pUartCon->cTxMode<<2)|(pUartCon->cRxMode));		
	if(pUartCon->cSendBreakSignal) return SENDBREAK;
	
	Outp32(&pUartRegs->rUmCon , (pUartCon->cRtsTrig<<5)|(pUartCon->cAfc<<4)|RTS_ACTIVE);	
	Outp32(&pUartRegs->rUintM , 0xf); // mask
	Outp32(&pUartRegs->rUintSp , 0xf); // source pending clear
	Outp32(&pUartRegs->rUintP , 0xf); // pending clear

	return 0;
}

u8 Uart_Open(u8 ch, UART_CON option) // setting H/W & initializing regiter
{
	uart_port = UART_CHANNEL_PORT;

	switch(ch)
	{
		case 0 :
			rGPACON = (rGPACON & ~(0xffff)) | (0x2222);
			if(UART_InitializeREG(uart_port, &option)  == SENDBREAK) return SENDBREAK; // Initialize register set for Uart 0
			break;
		case 1 :
			rGPACON = (rGPACON & ~(0xffff<<16)) | (0x2222<<16);
			if(UART_InitializeREG(uart_port, &option) == SENDBREAK)	 return SENDBREAK;
			break;
		case 2 :
			rGPBCON = (rGPBCON & ~(0xff)) | (0x22);
			if(UART_InitializeREG(uart_port, &option) == SENDBREAK)	 return SENDBREAK;
			break;
		case 3 :
			rGPBCON = (rGPBCON & ~(0xff<<8)) | (0x22<<8);
			if(UART_InitializeREG(uart_port, &option) == SENDBREAK)	 return SENDBREAK;
			break;
		default :
			return FALSE;
	}
	return TRUE;
}

u8 Uart_Close(u8 ch)  // return to reset value
{
	if (ch > 3) return FALSE;
	uart_port = UART_CHANNEL_PORT;

	while(!(Inp32(&uart_port->rUtrStat)&0x4));	//wait until tx buffer and shift register become empty
	Delay(1000);	//because of pad delay

	uart_port->rUintM |= ((1<<3)|(1<<2)|(1<<1)|(1));
	
	switch(ch)
	{
		case 0 :	INTC_Disable(NUM_UART0);	rGPACON = (rGPACON & ~(0xffff));	break;
		case 1 :	INTC_Disable(NUM_UART1);	rGPACON = (rGPACON & ~(0xffff<<16));break;
		case 2 :	INTC_Disable(NUM_UART2);	rGPBCON = (rGPBCON & ~(0xff));		break;
		case 3 :	INTC_Disable(NUM_UART3);	rGPACON = (rGPACON & ~(0xff<<8));	break;
		default :	return FALSE;
	}
	return TRUE;
}

