#ifndef __UART_DEV_H__
#define __UART_DEV_H__
#ifdef __cplusplus
extern "C" {
#endif

typedef struct tag_UART_CON
{
	u32 uBaudrate;
	u8 cSelUartIrda;	// Uart mode
	u8 cLoopTest;		// UCON[5]  (0 : normal, 1 : LoopBack)
	u8 cAfc;
	u8 cEnableFifo;
	u8 cOpClock;		// 0,2:PCLK, 1:UEXTCLK, 2:EPLL
	u8 cDataBit; 		// 0:5bit, 1:6bit, 2:7bit, 3:8bit
	u8 cParityBit; 		// 0:no parity, 1:odd, 2:even, 3:forced 1, 4:forced 0
	u8 cStopBit; 		// 0:one stopbit, 1:two stopbit
	u8 cTxMode; 		// 0:disable, 1:interrupt or polling, 2:DMA0, 3:DMA1
	u8 cTxTrig; 		// 0:empty, 1:16, 2:32, 3:48 (byte)
	u8 cRxMode; 		// 0:disable, 1:interrupt or polling, 2:DMA0, 3:DMA1
	u8 cRxTrig; 		// 0:1, 1:8, 2:16, 3:32 (byte)
	u8 cRtsTrig; 		// 0:63, 1:56, 2:48, ... , 7:8 (byte)
	u8 cSendBreakSignal;// ULCON[4]  (0 : normal, 1 : Send Break Signal)
} UART_CON;

#define No_IrDaMode			0
#define IrDaMode			1

#define No_LoopTest			0
#define LoopTest			1

#define EnableAFCMode		1
#define DisableAFCMode		0

#define EnableFIFO			1
#define DisableFIFO			0

#define UARTPCLK			2
#define UARTEXTCLK			1
#define UARTEPLL			3

#define DataLength5bit		0
#define DataLength6bit		1
#define DataLength7bit		2
#define DataLength8bit		3

#define NoParity			0
#define OddParity			4
#define EvenParity			5

#define OneStopBit			0
#define TwoStopBit			1

#define TxMode_IRQ			1
#define TxMode_DMA0			2
#define TxMode_DMA1			3

#define RxMode_IRQ			1
#define RxMode_DMA0			2
#define RxMode_DMA1			3

#define No_SendBreakSiganl	0
#define SendBreakSiganl		1

#define SENDBREAK			(0x99)

void Uart_Putc(u8 ch, char data);
s8 Uart_Getc(u8 ch);
u8 Uart_GetKey(u8 ch);
u8 Uart_Open(u8 ch, UART_CON option);
u8 Uart_Close(u8 ch);
u8 Uart_GetStatus(u8 ch);
u8 Uart_Getchar(u8 ch);

#ifdef __cplusplus
}
#endif
#endif
