#ifndef	__TYPE_DEFS_H__
 #define __TYPE_DEFS_H__
#ifdef __cplusplus
extern "C" {
#endif

// Type defines 
typedef unsigned long		u32;
typedef unsigned short		u16;
typedef unsigned char		u8;

typedef signed long			s32;
typedef signed short		s16;
typedef signed char			s8;

/* bsd */
typedef unsigned char		u_char;
typedef unsigned short		u_short;
typedef unsigned int		u_int;
typedef unsigned long		u_long;

/* sysv */
typedef unsigned char		unchar;
typedef unsigned short		ushort;
typedef unsigned int		uint;
typedef unsigned long		ulong;

typedef unsigned char		uchar;
typedef volatile unsigned long	vu_long;
typedef volatile unsigned short vu_short;
typedef volatile unsigned char	vu_char;

/* Preferred datatypes to be used in the project */
typedef unsigned int	UINT32;
typedef signed int		INT32;
typedef unsigned short	UINT16;
typedef signed short	INT16;
typedef unsigned char	UINT8;
typedef signed char	BOOLEAN;

/* REG32 and UINT32 are synonymous, but preferrable with registers */
typedef UINT32		REG32;

#ifndef NO
#define NO		0
#endif 
#ifndef YES
#define YES		1
#endif 
#ifndef	OK
#define OK		0                   /* Can never work out should this be 1 or 0? */
#endif
#ifndef	ERROR
#define ERROR	-1                   /* Can never work out should this be 1 or 0? */
#endif
#ifndef	FALSE
#define FALSE		0
#endif
#ifndef	TRUE
#define TRUE		1
#endif
#ifndef	NULL
#define	NULL	0
#endif

#ifndef	false
#define false				(0)
#define true				(1)
#endif

/*!
 @brief  make Bit mask
 @param width : bit mask width
 @param pos : start position
*/
#ifndef BIT_MASK_FIELD
#define BIT_MASK_FIELD(width, pos)  ((UINT32) ((1<<width)-1) << pos)
#endif

/*!
 @brief  extracts some bits from a buffer
 @param reg_value : bits data
 @param pos : start bit position
 @param width : number of bits
 @code
 	example) GETBIT(buffer, 1, 5 ) means

		7   6   5   4   3   2   1   0
	buffer	|---|---|---|---|---|---|---|
		    |<----------------->|
	                              pos = 1
			width = 5
 @endcode 	
*/
#ifndef GETBIT
#define GETBIT( reg_value, pos, width) (((UINT32)reg_value<<(32-width-pos))>>(32-width))
#endif

/*!
 @brief  build masked bits
 @param pos : start bit position
 @param width : number of bits
 @param reg_value : bits data
*/
#ifndef BUILDBIT
#define BUILDBIT( pos, width, value) (((UINT32)value<<(32-width))>>(32-width-pos))
#endif

/*!
 @brief  set bit to register
 @param pos : start bit position
 @param width : number of bits
 @param reg_value : bits data
*/
#ifndef SETBIT
#define SETBIT( reg_value, pos, width, value) (reg_value=((UINT32)reg_value&(~(BIT_MASK_FIELD(width, pos))))|BUILDBIT( pos, width, value))
#endif

/*
 * General Purpose Utilities
 */
#define min(X, Y)				\
	({ typeof (X) __x = (X), __y = (Y);	\
		(__x < __y) ? __x : __y; })

#define max(X, Y)				\
	({ typeof (X) __x = (X), __y = (Y);	\
		(__x > __y) ? __x : __y; })

#ifdef __cplusplus
}
#endif

#endif	/* !__TYPE_DEFS_H__ */
