#include "common.h"
#include "main.h"
#include "library.h"
//#include "watchdogtimer.h"
#if 0
#include "usb_dep.h"
#endif
#include "uart_dep.h"
#include "gpio_dev.h"
#include "lcd_dep.h"
#include "glib_dep.h"
#if 0
#include "touch_dep.h"
#include "intc.h"
#include "sysc.h"
#include "cs8900_dep.h"
#endif
#include "pjboot_app.h"
#include "wince_app.h"
#include "linux_app.h"
#include "firmware_app.h"
#include "memo_app.h"
#include "common_app.h"
#include "picture_indep.h"
#include "nand_dep.h"
#include "nand_dev.h"

#include "option.h"
#define PJBoot64_Image_Size 		189000

Config PJBoot64;

static void *title_main_function[][6] =
{
//	Function						Icon Name			Pos. X		Pos. Y		BMP Icon			Move up
	(void*)Quick_Wince_Boot,		(char*)"WinCE",		(u32*)100,	(u32*)100,	(u8*)WinCE_BOOT,	(void*)ICON_NOMOVE,
	(void*)Quick_Linux_Boot,		(char*)"Linux",		(u32*)300,	(u32*)100,	(u8*)Linux_BOOT,	(void*)ICON_NOMOVE,
	(void*)Quick_Firmware_Boot,		(char*)"Firmware",	(u32*)500,	(u32*)100,	(u8*)Firmware_BOOT,	(void*)ICON_NOMOVE,
	0, 0, 0, 0, 0, 0
};

static void *menu_function[][3] =
{
	(void*)Draw_Main_Win0,			(void*)Main_Map_Menu,			(char*)"Main",
	(void*)Draw_Linux_Win0,			(void*)Linux_Map_Menu,			(char*)"Linux",
	(void*)Draw_Wince_Win0,			(void*)Wince_Map_Menu,			(char*)"WinCE",
	(void*)Draw_Firmware_Win0,		(void*)Firmware_Map_Menu,		(char*)"Firmware",
	0, 0, 0
};

//void start_armboot(void)
void main(void)
{
	u32 select, filesize = 0;
	u32 page = 0;
	void *main_menu, *map_menu;

	SYSTEM_InitException();
	SYSTEM_InitMmu();
	GetSysClk();
	CalibrateDelay();
	Hardware_Init();

	Delay(10000);

	////////////////// LCD Inital ////////////////////
	LCD_Init(PJBoot64.ram.FrameBuffer);
	if(PJBoot64.lcd_en)
	{
		MemoPad_Init(50, 50, PJBoot64.lcd_xsize - 100, PJBoot64.lcd_ysize - 100, 0xa, WIN3);
		
		// ʱ BMP   .
		BMP_DrawGRAY((u32)title_main_function[1][2], (u32)title_main_function[1][3], (u8*)title_main_function[1][4]);
		LCD_Start();
	}
	
	main_menu = Draw_Main_Win0();
	Draw_Main_Menu(page, menu_function);
	map_menu = Main_Map_Menu();
	LCD_Move_Arrow_Win(POSITION_UP, WIN1);
	//////////////////////////////////////////////////

	while(1)
	{
		select = Position_Check(page, main_menu, map_menu, menu_function, WIN1);

		if(select < 100)								// Main Menu, Map Menu, Both Menu
		{
			MemoPad_On();
			
			if(select < 10)			filesize = Single_Run(page, select, main_menu);			// Main Menu
			else if(select < 16)	filesize = Memory_Map_Run(page, select, filesize, 1);	// Memory Map
			else 					filesize = Both_Run(page, select, main_menu, map_menu);	// Both Main and Map
			
			Wait_Key();
			Switch_GLIBControl(WIN1);
			Each_Status_Check(page);
			Position_Menu_Draw(main_menu, map_menu, 0);
			Draw_Main_Menu(page, menu_function);
			MemoPad_Off();
		}
		else if(select < 150)								// Main Menu ̵...
		{	
			main_menu = ((void*(*)(void))(menu_function[select - 100][0]))();
			map_menu = ((void*(*)(void))(menu_function[select - 100][1]))();
			Draw_Main_Menu(select - 100, menu_function);
			if(page > select - 100) LCD_Move_Arrow_Win(POSITION_LEFT, WIN1);
			else					LCD_Move_Arrow_Win(POSITION_RIGHT, WIN1);
			page = select - 100;
		}
		else												// PJBoot64, Maker Menu ̵..
		{
			if(select == 150) 
			{
				Draw_PJBoot_Win0();
				LCD_Move_Arrow_Win(POSITION_UP, WIN1);
				PJBoot_Menu();
			}
			else
			{
				Draw_Maker_Win0();
				LCD_Move_Arrow_Win(POSITION_DOWN, WIN1);
				Maker_Menu();
			}
			
			// Main ʱȭ...
			main_menu = ((void*(*)(void))(menu_function[page][0]))();
			map_menu = ((void*(*)(void))(menu_function[page][1]))();
			Draw_Main_Menu(page, menu_function);
			if(select == 150)	LCD_Move_Arrow_Win(POSITION_DOWN, WIN1);
			else				LCD_Move_Arrow_Win(POSITION_UP, WIN1);
		}
	}
}

void Hardware_Init(void)
{
	u32 Start_Mem_Cfg;
	// Platform Init..
	////////////////// RAM Block /////////////////////
	PJBoot64.ram.Vector_table	= _Exception_Vector;
	PJBoot64.ram.Stack			= _MMUTT_BaseAddress;
	PJBoot64.ram.Heap			= 0x57f00000;
	PJBoot64.ram.Config			= 0x57e80000;
	PJBoot64.ram.bootload		= 0x57e00000;
	PJBoot64.ram.Image			= LCD_ICON_BUFFER;
	PJBoot64.ram.FrameBuffer	= 0x57600000;
	
	// Linux Memory Map
	PJBoot64.ram.LinuxKernel	= 0x50008000;
	PJBoot64.ram.LinuxRamdisk	= 0x50800000;

	// Wince Memory Map
	PJBoot64.ram.Eboot			= 0x50038000;
	PJBoot64.ram.WinceKernel	= 0x50100000;
	
	// Firmware Memory Map
	PJBoot64.ram.Firmware		= 0x51000000;
	PJBoot64.ram.FirmwareTemp	= 0x52000000;
	
	// Download Memory Map
	PJBoot64.ram.Temp			= 0x51000000;
	//////////////////////////////////////////////////
	
	///////////////// Default Setting ////////////////
	PJBoot64.lcd_xsize = 800;
	PJBoot64.lcd_ysize = 480;
	//////////////////////////////////////////////////
	///////////////// Hardware Initial ///////////////
	GPIO_Init();
	PJBoot64.board_id = *(u32 *)(Board_Serial);

	UART_InitDebug(1, 115200);
	InitHeap(PJBoot64.ram.Heap, CFG_MALLOC_LEN);

	Board_Nand_Init(0);

	Start_Mem_Cfg = SYSC_ReadMemStat();
	if(!((Start_Mem_Cfg >> 7) & 0x1))		// Nor Boot
	{
		printf("Nor Booting Mode\n");
		StepLoader_Write();
	}
	else if(PJBoot64.board_id < 3 && PJBoot64.board_id > 4)
	{
		UART_Printf("Not Supported Board!\n");
		printf("Not Supported Board!\n");
		while(1);
	}

	Touch_Init();
#if 0
	CS8900_Init();
#endif
	//////////////////////////////////////////////////
	
	////////////////// Icon Process //////////////////
	Nand_Read_Process(0, PJBoot64.nand.Boot_Icon, PJBoot64.ram.Image, 0);
	if(Icon_Search())
	{
		printf("Icon Not Found\n");
		printf("goto : 150. PJBoot64 -> 1 or 2. Download Menu select\n");
		PJBoot64.lcd_en = FALSE;
		Touch_Off();
	}
	else PJBoot64.lcd_en = TRUE;
	//////////////////////////////////////////////////	
}

void* Draw_Main_Win0(void)
{
	printf("\n");
	printf("+-----------------------------------------------------+\n");
	printf("|  PJBoot64 %-6s (%s)                       |\n", VERSION, REL_DATE);
	printf("+-----------------------------------------------------+\n");
   	printf("|  MPLL=%d.%dfMHz, ARMCLK=%d.%dfMHz                   |\n"
   	, PJBoot64.mpll/1000000,PJBoot64.mpll%1000000
   	, PJBoot64.armclk/1000000, PJBoot64.armclk%1000000);
	printf("|  HCLK=%d.%dMHz, PCLK=%d.%dfMHz                      |\n"
		, PJBoot64.hclk/1000000, PJBoot64.hclk%1000000
		, PJBoot64.pclk/1000000, PJBoot64.pclk%1000000);
	printf("+-----------------------------------------------------+\n");

	Switch_GLIBControl(WIN0);
	GLIB_ClearFrame(WHITE, 1);
	Position_Menu_Draw(title_main_function, NULL, 0);
	return title_main_function;
};

void* Main_Map_Menu(void)
{
	return NULL;
}

extern unsigned char mango64_nor_slc_booter[];
extern unsigned char mango64_adv_slc_booter[];
extern unsigned char mango64_adv_mlc_booter[];

void StepLoader_Write(void)
{
	u32 filesize;
	u8* addr = (u8*)PJBoot64.ram.Temp;
	u32 key, i;
	
	while(1) {

		printf("\n Select Your Board!!\n");
		printf("  1. Mango64 - LCD(FL500WVR) !!\n");
		printf("  2. Mango64 - LCD(LTE480WV) !!\n");
		printf("\n Select the function to : ");

		// Uart
		key = UART_GetIntNum();
		if(key > 0 && key < 3) break;
	}
	
	// Nand Common Type Setting..
	switch(PJBoot64.nand_type)
	{
		case NAND_Normal8bit:	// Normal SLC
			for(i=0;i<4096;i++)	*(addr + i) = mango64_nor_slc_booter[i];
			break;
		case NAND_Advanced8bit:	// Advanced SLC
			for(i=0;i<4096;i++)	*(addr + i) = mango64_adv_slc_booter[i];
			break;
		case NAND_MLC8bit:		// Advanced MLC
			for(i=0;i<4096;i++)	*(addr + i) = mango64_adv_mlc_booter[i];
			break;
		default:
			printf("Not Supported Nand Flash\n");
			while(1);
	}
	
	// Insert ID
	*(addr + Board_Serial) = key + 2;
	
	// Nand Flash Format...
	printf("Nand Format Processing...\n");
	Nand_Format_Process(0);
	
	// StepLoader Write Process
	filesize = 4096;
	if(Nand_NoHead_Write_Process(0, 0, filesize, (u32)addr))
	{
		printf("StepLoader Write Failed\n");
		while(1);
	}
	printf("\nStepLoader Write Success\n");

	// PJBoot64 Write Process
	filesize = PJBoot64_Image_Size;
	memcpy((void*)PJBoot64.ram.Temp, (void*)PJBoot64.ram.bootload, filesize);
	if(Nand_Write_Process(0, 1, filesize, PJBoot64.ram.Temp, 0))
	{
		printf("PJBoot64 Write Failed\n");
		while(1);
	}

	printf("PJBoot64 Write Success\n\n");
	printf("Please change to Nand Boot Jumper Setting...!!\n");
	printf("And Press Reset Button\n");
	while(1);
}
